﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class BarcodeLabel : Library.BaseCodeForm
    {
        private long ActiveLabelID;
        private char CurrentMeasure;
        private int LabelWidth;
        private int LabelHeight;
        private bool IgnoreSizeChange;
        private string BarcodeFontName;
        private bool Barcode39;

        public BarcodeLabel()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            BarcodeFontName = GetBarcodeFont();
            if (BarcodeFontName.Length == 0)
                return -1L;
            ActiveLabelID = -1L;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveLabelID;
            else
                return -1L;
        }

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManageBarcodeTemplates];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManageBarcodeTemplates];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManageBarcodeTemplates];
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the record.
            string sqlText;
            DataCommand sqlRun;
            LibraryTransaction transInfo = null;

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete this barcode label template?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
                return false;

            // ----- Delete the record.
            try
            {
                transInfo = TransactionBegin();

                sqlText = "DELETE FROM BarcodeLabelItem WHERE BarcodeLabel = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", recordID);
                ExecuteSQL(sqlRun, transInfo);

                sqlText = "DELETE FROM BarcodeLabel WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", recordID);
                ExecuteSQL(sqlRun, transInfo);

                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                if (transInfo != null)
                    TransactionRollback(transInfo);
                GeneralError("BarcodeLabel.DeleteRecord", ex);
                return false;
            }
            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record.
            BarcodeFontName = GetBarcodeFont();
            if (BarcodeFontName.Length == 0)
                return -1L;
            ActiveLabelID = recordID;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveLabelID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items.
            string sqlText;
            DataTable dbInfo;

            // ----- Clear any existing values.
            destList.Items.Clear();
            exceededMatches = false;

            // ----- Retrieve the values from the database.
            sqlText = "SELECT BL.ID, BL.FullName, BS.FullName AS PageName " +
                "FROM BarcodeLabel AS BL INNER JOIN BarcodeSheet AS BS " +
                "ON BL.BarcodeSheet = BS.ID ORDER BY BL.FullName, BS.FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("BarcodeLabel.FillListWithRecords", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                destList.Items.Add(new ListItemData(DBGetText(dbRow["FullName"]) +
                    " on " + DBGetText(dbRow["PageName"]), DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            string result;

            // ----- Initialize.
            result = "";

            // ----- Retrieve the value from the database.
            sqlText = "SELECT BL.FullName, BS.FullName AS PageName " +
                "FROM BarcodeLabel AS BL INNER JOIN BarcodeSheet AS BS " +
                "ON BL.BarcodeSheet = BS.ID WHERE BL.ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("BarcodeLabel.FormatRecordName", ex);
                return "Error";
            }
            if (dbInfo.Rows.Count > 0)
            {
                oneRow = dbInfo.Rows[0];
                result = DBGetText(oneRow["FullName"]) + " on " +
                    DBGetText(oneRow["PageName"]);
            }
            dbInfo.Dispose();
            dbInfo = null;
            return result;
        }

        public override string GetEditDescription()
        {
            // ----- Return a description of this editor.
            return "Use these codes to identify the format of printed barcode labels. " +
                "You can define the layout of a label using text, barcodes, and shapes.";
        }

        public override string GetEditTitle()
        {
            // ----- Return the title-bar text for this editor.
            return "Barcode Label Templates";
        }

        private void PrepareFormFields()
        {
            // ----- Prepare the form.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            BarcodeItemBarcodeImage newBarcodeImage;
            BarcodeItemBarcodeNumber newBarcodeText;
            BarcodeItemText newStaticText;
            BarcodeItemLine newStaticLine;
            BarcodeItemRect newStaticRect;

            // ----- Load in the list of templates.
            if (RecordTemplate.Items.Count == 0)
            {
                RecordTemplate.Items.Add(new ListItemData("<Not Selected>", -1L));
                sqlText = "SELECT ID, FullName FROM BarcodeSheet ORDER BY FullName";
                try
                {
                    dbInfo = CreateDataTable(sqlText);
                }
                catch (Exception ex)
                {
                    GeneralError("BarcodeLabel.PrepareFormFields.Template", ex);
                    return;
                }
                foreach (DataRow dbRow in dbInfo.Rows)
                {
                    RecordTemplate.Items.Add(new ListItemData(
                        DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
                }
                dbInfo.Dispose();
                dbInfo = null;
            }

            // ----- Fill in the list of units.
            if (RecordUnits.Items.Count == 0)
            {
                RecordUnits.Items.Add(new ListItemData("Inches", 'I'));
                RecordUnits.Items.Add(new ListItemData("Centimeters", 'C'));
                RecordUnits.Items.Add(new ListItemData("Points", 'P'));
                RecordUnits.Items.Add(new ListItemData("Twips", 'T'));
                RecordUnits.SelectedIndex = 0;
            }
            CurrentMeasure = 'I';

            // ----- Clear the existing data.
            IgnoreSizeChange = true;
            LabelWidth = -1;
            LabelHeight = -1;
            RecordName.Text = "";
            RecordTemplate.SelectedIndex = RecordTemplate.Items.IndexOf(-1L);
            if (RecordTemplate.SelectedIndex == -1)
                RecordTemplate.SelectedIndex = 0;
            RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(CurrentMeasure);
            if (RecordUnits.SelectedIndex == -1)
                RecordUnits.SelectedIndex = 0;
            DisplayItems.Items.Clear();
            ActUp.Enabled = false;
            ActDown.Enabled = false;
            ActDelete.Enabled = false;
            ItemProperties.SelectedObject = null;
            RecordSample.Text = "12345678";
            PreviewArea.Invalidate();
            IgnoreSizeChange = false;
            Barcode39 = GetBoolean(GetSystemValue("BarcodeCode39"));
            if (ActiveLabelID == -1L)
                return;

            // ----- Load in the record.
            sqlText = "SELECT * FROM BarcodeLabel WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveLabelID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("BarcodeLabel.PrepareFormFields.Retrieve", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            IgnoreSizeChange = true;
            RecordName.Text = DBGetText(oneRow["FullName"]);
            RecordTemplate.SelectedIndex = RecordTemplate.Items.IndexOf(
                DBGetLong(oneRow["BarcodeSheet"]));
            if (RecordTemplate.SelectedIndex == -1)
                RecordTemplate.SelectedIndex = 0;
            RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(DBGetText(oneRow["UnitType"])[0]);
            if (RecordUnits.SelectedIndex == -1)
                RecordUnits.SelectedIndex = 0;
            CurrentMeasure = (ListItemData)RecordUnits.SelectedItem;
            IgnoreSizeChange = false;
            dbInfo.Dispose();

            // ----- Determine the size of the label.
            RefreshLabelTemplate();
            RefreshSizeDisplay();

            // ----- Load in the list of objects.
            sqlText = "SELECT * FROM BarcodeLabelItem " +
                "WHERE BarcodeLabel = @RecordID ORDER BY Priority";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveLabelID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("BarcodeLabel.PrepareFormFields.Objects", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Process this object.
                switch (DBGetText(dbRow["ItemType"]).ToUpper())
                {
                    case "B":
                        // ----- Barcode image.
                        newBarcodeImage = new BarcodeItemBarcodeImage();
                        newBarcodeImage.Alignment = (ContentAlignment)DBGetInteger(dbRow["Alignment"]);
                        newBarcodeImage.BarcodeColor = Color.FromArgb(DBGetInteger(dbRow["Color1"]));
                        newBarcodeImage.BarcodeSize = DBGetDecimal(dbRow["FontSize"]);
                        newBarcodeImage.Left = DBGetDecimal(dbRow["PosLeft"]);
                        newBarcodeImage.Top = DBGetDecimal(dbRow["PosTop"]);
                        newBarcodeImage.Width = DBGetDecimal(dbRow["PosWidth"]);
                        newBarcodeImage.Height = DBGetDecimal(dbRow["PosHeight"]);
                        newBarcodeImage.RotationAngle = (short)DBGetInteger(dbRow["Rotation"]);
                        newBarcodeImage.PadDigits = (byte)(DBGetInteger(dbRow["PadDigits"]));
                        DisplayItems.Items.Add(newBarcodeImage);
                        break;
                    case "L":
                        // ----- Line.
                        newStaticLine = new BarcodeItemLine();
                        newStaticLine.LineColor = Color.FromArgb(DBGetInteger(dbRow["Color1"]));
                        newStaticLine.X1 = DBGetDecimal(dbRow["PosLeft"]);
                        newStaticLine.X2 = DBGetDecimal(dbRow["PosWidth"]);
                        newStaticLine.Y1 = DBGetDecimal(dbRow["PosTop"]);
                        newStaticLine.Y2 = DBGetDecimal(dbRow["PosHeight"]);
                        DisplayItems.Items.Add(newStaticLine);
                        break;
                    case "N":
                        // ----- Barcode text.
                        newBarcodeText = new BarcodeItemBarcodeNumber();
                        newBarcodeText.Alignment = (ContentAlignment)DBGetInteger(dbRow["Alignment"]);
                        newBarcodeText.Left = DBGetDecimal(dbRow["PosLeft"]);
                        newBarcodeText.Top = DBGetDecimal(dbRow["PosTop"]);
                        newBarcodeText.Width = DBGetDecimal(dbRow["PosWidth"]);
                        newBarcodeText.Height = DBGetDecimal(dbRow["PosHeight"]);
                        newBarcodeText.RotationAngle = (short)DBGetInteger(dbRow["Rotation"]);
                        newBarcodeText.TextColor = Color.FromArgb(DBGetInteger(dbRow["Color1"]));
                        newBarcodeText.Font = new Font(DBGetText(dbRow["FontName"]),
                            (float)DBGetDecimal(dbRow["FontSize"]),
                            BuildFontStyle(DBGetText(dbRow["FontStyle"])));
                        newBarcodeText.PadDigits = (byte)(DBGetInteger(dbRow["PadDigits"]));
                        DisplayItems.Items.Add(newBarcodeText);
                        break;
                    case "R":
                        // ----- Rectangle.
                        newStaticRect = new BarcodeItemRect();
                        newStaticRect.BorderColor = Color.FromArgb(DBGetInteger(dbRow["Color1"]));
                        newStaticRect.FillColor = Color.FromArgb(DBGetInteger(dbRow["Color2"]));
                        newStaticRect.Left = DBGetDecimal(dbRow["PosLeft"]);
                        newStaticRect.Top = DBGetDecimal(dbRow["PosTop"]);
                        newStaticRect.Width = DBGetDecimal(dbRow["PosWidth"]);
                        newStaticRect.Height = DBGetDecimal(dbRow["PosHeight"]);
                        newStaticRect.RotationAngle = (short)DBGetInteger(dbRow["Rotation"]);
                        DisplayItems.Items.Add(newStaticRect);
                        break;
                    case "T":
                        // ----- Static text.
                        newStaticText = new BarcodeItemText();
                        newStaticText.DisplayText = DBGetText(dbRow["StaticText"]);
                        newStaticText.Alignment = (ContentAlignment)DBGetInteger(dbRow["Alignment"]);
                        newStaticText.Left = DBGetDecimal(dbRow["PosLeft"]);
                        newStaticText.Top = DBGetDecimal(dbRow["PosTop"]);
                        newStaticText.Width = DBGetDecimal(dbRow["PosWidth"]);
                        newStaticText.Height = DBGetDecimal(dbRow["PosHeight"]);
                        newStaticText.RotationAngle = (short)DBGetInteger(dbRow["Rotation"]);
                        newStaticText.TextColor = Color.FromArgb(DBGetInteger(dbRow["Color1"]));
                        newStaticText.Font = new Font(DBGetText(dbRow["FontName"]),
                            (float)DBGetDecimal(dbRow["FontSize"]),
                            BuildFontStyle(DBGetText(dbRow["FontStyle"])));
                        DisplayItems.Items.Add(newStaticText);
                        break;
                }
            }
            dbInfo.Dispose();
            PreviewArea.Invalidate();
        }

        private void RefreshSizeDisplay()
        {
            // ----- Show the size of the label.
            char toUnit;

            toUnit = (ListItemData)RecordUnits.SelectedItem;
            if ((LabelWidth == -1) | (LabelHeight == -1))
                LabelPreview.Text = "Label Preview";
            else
                LabelPreview.Text = string.Format("Label Preview: {0} (w) by {1} (h)",
                    ConvertPageUnits('T', toUnit, LabelWidth),
                    ConvertPageUnits('T', toUnit, LabelHeight));
        }

        private bool ValidateForm()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            string sqlText;
            DataCommand sqlRun;

            // ----- Name is a required field.
            if (RecordName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the label template name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordName.Focus();
                return false;
            }

            // ----- Make sure a template is selected.
            if ((long)(ListItemData)RecordTemplate.SelectedItem == -1L)
            {
                MessageBox.Show("Please select a page template.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordTemplate.Focus();
                return false;
            }

            // ----- Make sure the name is not already in use.
            sqlText = "SELECT COUNT(*) FROM BarcodeLabel " +
                "WHERE UPPER(FullName) = @TestName AND BarcodeSheet = @SheetID";
            if (ActiveLabelID != -1L)
                sqlText += " AND ID <> @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestName", RecordName.Text.Trim().ToUpper());
            sqlRun.AddLong("SheetID", (long)(ListItemData)RecordTemplate.SelectedItem);
            sqlRun.AddLong("RecordID", ActiveLabelID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    // ----- Duplicate entry.
                    MessageBox.Show("A label template using the page template '" +
                        RecordTemplate.Text + "' already exists with the name '" +
                        RecordName.Text.Trim() + "'.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordName.Focus();
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("BarcodeLabel.ValidateForm", ex);
                return false;
            }

            // ----- At least one item is required.
            if (DisplayItems.Items.Count == 0)
            {
                MessageBox.Show("The label template must include at least one item.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            // ----- Success.
            return true;
        }

        private void RefreshLabelTemplate()
        {
            // ----- Given the active template, determine the width and height of a
            //       single label, in twips.
            decimal oneWidth;
            decimal oneHeight;
            string sqlText;
            DataCommand sqlRun;
            char useMeasure;
            DataTable dbInfo;
            DataRow oneRow;

            // ----- A width and height of -1 indicates an unknown size.
            LabelWidth = -1;
            LabelHeight = -1;
            if (RecordTemplate.SelectedIndex == -1)
                return;
            if ((long)(ListItemData)RecordTemplate.SelectedItem == -1L)
                return;
            this.Cursor = Cursors.WaitCursor;

            // ----- Retrieve the label details.
            sqlText = "SELECT * FROM BarcodeSheet WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", (long)(ListItemData)RecordTemplate.SelectedItem);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("BarcodeLabel.RefreshLabelTemplate", ex);
                return;
            }

            // ----- The label size is what is left over on the page after the
            //       margins and the space between labels is removed.
            oneRow = dbInfo.Rows[0];
            oneWidth = (DBGetDecimal(oneRow["PageWidth"]) -
                DBGetDecimal(oneRow["MarginLeft"]) - DBGetDecimal(oneRow["MarginRight"]) -
                ((DBGetDecimal(oneRow["ColumnsCount"]) - 1.0M) *
                 DBGetDecimal(oneRow["IntraColumn"]))) /
                 DBGetDecimal(oneRow["ColumnsCount"]);
            oneHeight = (DBGetDecimal(oneRow["PageHeight"]) -
                DBGetDecimal(oneRow["MarginTop"]) - DBGetDecimal(oneRow["MarginBottom"]) -
                ((DBGetDecimal(oneRow["RowsCount"]) - 1.0M) *
                 DBGetDecimal(oneRow["IntraRow"]))) /
                 DBGetDecimal(oneRow["RowsCount"]);
            useMeasure = Convert.ToChar(DBGetText(oneRow["UnitType"]));
            dbInfo.Dispose();

            // ----- Convert the result into twips.
            LabelWidth = (int)ConvertPageUnits(useMeasure, 'T', oneWidth);
            LabelHeight = (int)ConvertPageUnits(useMeasure, 'T', oneHeight);
            this.Cursor = Cursors.Default;
        }

        private void AdjustListItemButtons()
        {
            // ----- Adjust the display based on the selection.
            if (DisplayItems.SelectedIndex == -1)
            {
                ItemProperties.SelectedObject = null;
                ActUp.Enabled = false;
                ActDown.Enabled = false;
                ActDelete.Enabled = false;
            }
            else
            {
                ItemProperties.SelectedObject = DisplayItems.SelectedItem;
                ActUp.Enabled = (DisplayItems.SelectedIndex > 0);
                ActDown.Enabled = (DisplayItems.SelectedIndex <
                    (DisplayItems.Items.Count - 1));
                ActDelete.Enabled = true;
            }
        }

        private void RecordName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordName.SelectAll();
        }

        private void RecordSample_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordSample.SelectAll();
        }

        private void DisplayItems_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Adjust the display based on the selection.
            AdjustListItemButtons();
            PreviewArea.Invalidate();
        }

        private void ActText_Click(object sender, EventArgs e)
        {
            // ----- Add a new static text item.
            BarcodeItemText newStaticText;
            char fromUnit;

            fromUnit = (ListItemData)RecordUnits.SelectedItem;
            newStaticText = new BarcodeItemText();
            newStaticText.DisplayText = "Text";
            newStaticText.Alignment = ContentAlignment.TopLeft;
            newStaticText.RotationAngle = 0;
            newStaticText.TextColor = Color.Black;
            newStaticText.Font = new Font("Arial", 12);
            newStaticText.Left = 0M;
            newStaticText.Top = 0M;
            newStaticText.Width = ConvertPageUnits('I', fromUnit, 1.0M);
            newStaticText.Height = ConvertPageUnits('I', fromUnit, 0.5M);
            DisplayItems.Items.Add(newStaticText);
            DisplayItems.SelectedItem = newStaticText;
            ItemProperties.SelectedObject = newStaticText;
            PreviewArea.Invalidate();
        }

        private void ActBarcode_Click(object sender, EventArgs e)
        {
            // ----- Add a new barcode item.
            BarcodeItemBarcodeImage newBarcodeImage;
            char fromUnit;

            fromUnit = (ListItemData)RecordUnits.SelectedItem;
            newBarcodeImage = new BarcodeItemBarcodeImage();
            newBarcodeImage.Alignment = ContentAlignment.TopLeft;
            newBarcodeImage.BarcodeColor = Color.Black;
            newBarcodeImage.BarcodeSize = 10.0M;
            newBarcodeImage.Left = 0M;
            newBarcodeImage.Top = 0M;
            newBarcodeImage.Width = ConvertPageUnits('I', fromUnit, 1.0M);
            newBarcodeImage.Height = ConvertPageUnits('I', fromUnit, 0.5M);
            newBarcodeImage.RotationAngle = 0;
            newBarcodeImage.PadDigits = 0;
            DisplayItems.Items.Add(newBarcodeImage);
            DisplayItems.SelectedItem = newBarcodeImage;
            ItemProperties.SelectedObject = newBarcodeImage;
            PreviewArea.Invalidate();
        }

        private void ActBarNumber_Click(object sender, EventArgs e)
        {
            // ----- Add a new barcode text item.
            BarcodeItemBarcodeNumber newBarcodeText;
            char fromUnit;

            fromUnit = (ListItemData)RecordUnits.SelectedItem;
            newBarcodeText = new BarcodeItemBarcodeNumber();
            newBarcodeText.Alignment = ContentAlignment.TopLeft;
            newBarcodeText.Left = 0M;
            newBarcodeText.Top = 0M;
            newBarcodeText.Width = ConvertPageUnits('I', fromUnit, 1.0M);
            newBarcodeText.Height = ConvertPageUnits('I', fromUnit, 0.5M);
            newBarcodeText.RotationAngle = 0;
            newBarcodeText.TextColor = Color.Black;
            newBarcodeText.PadDigits = 0;
            newBarcodeText.Font = new Font("Arial", 12);
            DisplayItems.Items.Add(newBarcodeText);
            DisplayItems.SelectedItem = newBarcodeText;
            ItemProperties.SelectedObject = newBarcodeText;
            PreviewArea.Invalidate();
        }

        private void ActLine_Click(object sender, EventArgs e)
        {
            // ----- Add a new line item.
            BarcodeItemLine newStaticLine;
            char fromUnit;

            fromUnit = (ListItemData)RecordUnits.SelectedItem;
            newStaticLine = new BarcodeItemLine();
            newStaticLine.LineColor = Color.Black;
            newStaticLine.X1 = 0M;
            newStaticLine.X2 = ConvertPageUnits('I', fromUnit, 1.0M);
            newStaticLine.Y1 = 0M;
            newStaticLine.Y2 = ConvertPageUnits('I', fromUnit, 0.5M);
            DisplayItems.Items.Add(newStaticLine);
            DisplayItems.SelectedItem = newStaticLine;
            ItemProperties.SelectedObject = newStaticLine;
            PreviewArea.Invalidate();
        }

        private void ActRectangle_Click(object sender, EventArgs e)
        {
            // ----- Add a new rectangle item.
            BarcodeItemRect newStaticRect;
            char fromUnit;

            fromUnit = (ListItemData)RecordUnits.SelectedItem;
            newStaticRect = new BarcodeItemRect();
            newStaticRect.BorderColor = Color.Black;
            newStaticRect.FillColor = Color.White;
            newStaticRect.Left = 0M;
            newStaticRect.Top = 0M;
            newStaticRect.Width = ConvertPageUnits('I', fromUnit, 1.0M);
            newStaticRect.Height = ConvertPageUnits('I', fromUnit, 0.5M);
            newStaticRect.RotationAngle = 0;
            DisplayItems.Items.Add(newStaticRect);
            DisplayItems.SelectedItem = newStaticRect;
            ItemProperties.SelectedObject = newStaticRect;
            PreviewArea.Invalidate();
        }

        private void ItemProperties_PropertyValueChanged(object sender, PropertyValueChangedEventArgs e)
        {
            // ----- Update the preview when any property changes.
            PreviewArea.Invalidate();

            // ----- Update the text of the list box item.
            DisplayItems.Items[DisplayItems.SelectedIndex] = DisplayItems.SelectedItem;
        }

        private void ActUp_Click(object sender, EventArgs e)
        {
            // ----- Move the selected item up one position.
            object holdObject;

            // ----- Ignore if no item is selected.
            if (DisplayItems.SelectedIndex <= 0)
                return;

            // ----- Reverse the items.
            holdObject = DisplayItems.Items[DisplayItems.SelectedIndex - 1];
            DisplayItems.Items.RemoveAt(DisplayItems.SelectedIndex - 1);
            DisplayItems.Items.Insert(DisplayItems.SelectedIndex + 1, holdObject);

            // ----- Update the display.
            AdjustListItemButtons();
            PreviewArea.Invalidate();
        }

        private void ActDown_Click(object sender, EventArgs e)
        {
            // ----- Move the selected item down one position.
            object holdObject;

            // ----- Ignore if no item is selected.
            if (DisplayItems.SelectedIndex == -1)
                return;
            if (DisplayItems.SelectedIndex == (DisplayItems.Items.Count - 1))
                return;

            // ----- Reverse the items.
            holdObject = DisplayItems.Items[DisplayItems.SelectedIndex + 1];
            DisplayItems.Items.RemoveAt(DisplayItems.SelectedIndex + 1);
            DisplayItems.Items.Insert(DisplayItems.SelectedIndex, holdObject);

            // ----- Update the display.
            AdjustListItemButtons();
            PreviewArea.Invalidate();
        }

        private void ActDelete_Click(object sender, EventArgs e)
        {
            // ----- Delete the selected item.
            if (DisplayItems.SelectedIndex == -1)
                return;
            DisplayItems.Items.RemoveAt(DisplayItems.SelectedIndex);

            // ----- Refresh the display.
            ItemProperties.SelectedObject = null;
            AdjustListItemButtons();
            PreviewArea.Invalidate();
        }

        private void PreviewArea_Paint(object sender, PaintEventArgs e)
        {
            // ----- Display a preview of the image.
            int pageWidth;
            int pageHeight;
            int pageLeft; // Start of the "white" area.
            int pageTop; // Start of the "white" area.
            decimal useRatio;
            decimal useRatioText;
            char fromUnit;
            string textMessage;
            BarcodeItemGeneric useGeneric;
            BarcodeItemBarcodeImage useBarImage;
            BarcodeItemBarcodeNumber useBarText;
            BarcodeItemText useText;
            BarcodeItemLine useLine;
            BarcodeItemRect useRect;
            Font useFont;
            GraphicsState holdState;
            Pen pixelPen;
            SolidBrush pixelBrush;
            Color textColor;
            int textAngle;
            ContentAlignment textAlign;
            StringFormat textFormat = new StringFormat();
            float X1;
            float X2;
            float Y1;
            float Y2;

            // ----- Draw in the background color.
            e.Graphics.Clear(SystemColors.Control);
            if ((LabelWidth <= 0) | (LabelHeight <= 0))
                return;

            // ----- Determine the page area.
            pageWidth = PreviewArea.Width - 4;
            pageHeight = PreviewArea.Height - 4;
            useRatio = pageWidth / (decimal)LabelWidth;
            useRatioText = pageWidth / ConvertPageUnits('T', 'P', LabelWidth);
            if ((int)(LabelHeight * useRatio) > pageHeight)
            {
                // ----- Scale based on the height.
                useRatio = pageHeight / (decimal)LabelHeight;
                useRatioText = pageHeight / ConvertPageUnits('T', 'P', LabelHeight);
                pageWidth = (int)(LabelWidth * useRatio);
            }
            else
            {
                // ----- Scale based on the width.
                pageHeight = (int)(LabelHeight * useRatio);
            }
            pageLeft = (int)((PreviewArea.Width - 4 - pageWidth) / 2.0) + 1;
            pageTop = (int)((PreviewArea.Height - 4 - pageHeight) / 2.0) + 1;

            // ----- Draw the page area.
            e.Graphics.FillRectangle(SystemBrushes.ControlDark, pageLeft + 1,
                pageTop + 1, pageWidth + 2, pageHeight + 2);
            e.Graphics.FillRectangle(SystemBrushes.ControlDark, pageLeft + 2,
                pageTop + 2, pageWidth + 2, pageHeight + 2);
            e.Graphics.FillRectangle(Brushes.Black, pageLeft - 1, pageTop - 1,
                pageWidth + 2, pageHeight + 2);
            e.Graphics.FillRectangle(Brushes.White,
                pageLeft, pageTop, pageWidth, pageHeight);

            // ----- What size of units are we using for item display?
            if (RecordUnits.SelectedIndex == -1)
                return;
            fromUnit = (ListItemData)RecordUnits.SelectedItem;

            // ----- Transform a page of twips into a display page.
            e.Graphics.TranslateTransform(pageLeft, pageTop);
            e.Graphics.ScaleTransform((float)useRatio, (float)useRatio);

            // ----- Process each item in the list.
            for (int counter = 0; counter < DisplayItems.Items.Count; counter++)
            {
                // ----- Save the current state of the graphics area.
                holdState = e.Graphics.Save();

                // ----- Different code for each type of drawing object.
                useGeneric = (BarcodeItemGeneric)DisplayItems.Items[counter];
                switch (useGeneric.ItemType)
                {
                    case "B":
                        // ----- Barcode image.
                        useBarImage = (BarcodeItemBarcodeImage)useGeneric;
                        textMessage = RecordSample.Text.Trim();
                        if (textMessage.Length == 0)
                            textMessage = "12345678";
                        if (textMessage.Length < useBarImage.PadDigits)
                            textMessage = new string('0',
                                useBarImage.PadDigits - textMessage.Length) + textMessage;
                        if (Barcode39 == true)
                            textMessage = "*" + textMessage + "*";
                        textColor = useBarImage.BarcodeColor;
                        textAngle = useBarImage.RotationAngle;
                        textAlign = useBarImage.Alignment;

                        // ----- Get the font to use.
                        useFont = new Font(BarcodeFontName,
                            (float)useBarImage.BarcodeSize, FontStyle.Regular);

                        // ----- Get the coordinates in points.
                        X1 = (float)ConvertPageUnits(fromUnit, 'P', useBarImage.Left);
                        X2 = (float)ConvertPageUnits(fromUnit, 'P', useBarImage.Width);
                        Y1 = (float)ConvertPageUnits(fromUnit, 'P', useBarImage.Top);
                        Y2 = (float)ConvertPageUnits(fromUnit, 'P', useBarImage.Height);

                        // ----- The rest is the same as drawing normal text.
                        goto DisplayTheText;
                    case "L":
                        // ----- Line.
                        useLine = (BarcodeItemLine)useGeneric;

                        // ----- Create a one-pixel pen to use.
                        pixelPen = new Pen(useLine.LineColor, 1 / e.Graphics.DpiX);

                        // ----- Get the coordinates in twips.
                        X1 = (float)ConvertPageUnits(fromUnit, 'T', useLine.X1);
                        X2 = (float)ConvertPageUnits(fromUnit, 'T', useLine.X2);
                        Y1 = (float)ConvertPageUnits(fromUnit, 'T', useLine.Y1);
                        Y2 = (float)ConvertPageUnits(fromUnit, 'T', useLine.Y2);

                        // ----- Draw the line.
                        e.Graphics.DrawLine(pixelPen, X1, Y1, X2, Y2);
                        pixelPen.Dispose();

                        // ----- If this is the active line object, draw a thin outline
                        //       around the object.
                        if (DisplayItems.SelectedIndex == counter)
                        {
                            pixelPen = new Pen(Color.LightGray, 1 / e.Graphics.DpiX);
                            pixelPen.DashStyle = DashStyle.Dash;
                            e.Graphics.DrawRectangle(pixelPen,
                                X1 - 15, Y1 - 15, X2 - X1 + 30, Y2 - Y1 + 30);
                            pixelPen.Dispose();
                        }
                        break;
                    case "N":
                        // ----- Barcode text.
                        useBarText = (BarcodeItemBarcodeNumber)useGeneric;
                        textMessage = RecordSample.Text.Trim();
                        if (textMessage.Length == 0)
                            textMessage = "12345678";
                        if (textMessage.Length < useBarText.PadDigits)
                            textMessage = new string('0',
                                useBarText.PadDigits - textMessage.Length) + textMessage;
                        textColor = useBarText.TextColor;
                        textAngle = useBarText.RotationAngle;
                        textAlign = useBarText.Alignment;

                        // ----- Get the font to use.
                        useFont = new Font(useBarText.Font.Name,
                            useBarText.Font.SizeInPoints, useBarText.Font.Style);

                        // ----- Get the coordinates in points.
                        X1 = (float)ConvertPageUnits(fromUnit, 'P', useBarText.Left);
                        X2 = (float)ConvertPageUnits(fromUnit, 'P', useBarText.Width);
                        Y1 = (float)ConvertPageUnits(fromUnit, 'P', useBarText.Top);
                        Y2 = (float)ConvertPageUnits(fromUnit, 'P', useBarText.Height);

                        // ----- The rest is the same as drawing normal text.
                        goto DisplayTheText;
                    case "R":
                        // ----- Rectangle.
                        useRect = (BarcodeItemRect)useGeneric;

                        // ----- Create a one-pixel pen to use.
                        pixelPen = new Pen(useRect.BorderColor, 1 / e.Graphics.DpiX);
                        pixelBrush = new SolidBrush(useRect.FillColor);

                        // ----- Get the coordinates in twips.
                        X1 = (float)ConvertPageUnits(fromUnit, 'T', useRect.Left);
                        X2 = (float)ConvertPageUnits(fromUnit, 'T', useRect.Width);
                        Y1 = (float)ConvertPageUnits(fromUnit, 'T', useRect.Top);
                        Y2 = (float)ConvertPageUnits(fromUnit, 'T', useRect.Height);

                        // ----- Rotate the surface if requested.
                        if (useRect.RotationAngle != 0)
                        {
                            e.Graphics.TranslateTransform(X1, Y1);
                            e.Graphics.RotateTransform(useRect.RotationAngle);
                            X1 = 0.0F;
                            Y1 = 0.0F;
                        }

                        // ----- Draw the rectangle.
                        e.Graphics.FillRectangle(pixelBrush, X1, Y1, X2, Y2);
                        e.Graphics.DrawRectangle(pixelPen, X1, Y1, X2, Y2);
                        pixelBrush.Dispose();
                        pixelPen.Dispose();

                        // ----- If this is the active line object, draw a thin outline
                        //       around the object.
                        if (DisplayItems.SelectedIndex == counter)
                        {
                            pixelPen = new Pen(Color.LightGray, 1 / e.Graphics.DpiX);
                            pixelPen.DashStyle = DashStyle.Dash;
                            e.Graphics.DrawRectangle(pixelPen, X1 - 1 / e.Graphics.DpiX,
                                Y1 - 1 / e.Graphics.DpiX, X2 + 2 / e.Graphics.DpiX,
                                Y2 + 2 / e.Graphics.DpiX);
                            pixelPen.Dispose();
                        }
                        break;
                    case "T":
                        // ----- Text.
                        useText = (BarcodeItemText)useGeneric;
                        textMessage = useText.DisplayText;
                        textColor = useText.TextColor;
                        textAngle = useText.RotationAngle;
                        textAlign = useText.Alignment;

                        // ----- Get the font to use.
                        useFont = new Font(useText.Font.Name,
                            useText.Font.SizeInPoints, useText.Font.Style);

                        // ----- Get the coordinates in points.
                        X1 = (float)ConvertPageUnits(fromUnit, 'P', useText.Left);
                        X2 = (float)ConvertPageUnits(fromUnit, 'P', useText.Width);
                        Y1 = (float)ConvertPageUnits(fromUnit, 'P', useText.Top);
                        Y2 = (float)ConvertPageUnits(fromUnit, 'P', useText.Height);

        DisplayTheText:
                        // ----- For text, we do a new transformation. Everything is
                        //       measured in points instead of twips.
                        e.Graphics.ResetTransform();
                        e.Graphics.TranslateTransform(pageLeft, pageTop);
                        e.Graphics.ScaleTransform((float)useRatioText, (float)useRatioText);

                        // ----- Rotate around the upper-left corner of the text box
                        //       if needed.
                        if (textAngle != 0)
                        {
                            e.Graphics.TranslateTransform(X1, Y1);
                            e.Graphics.RotateTransform(textAngle);
                            X1 = 0.0F;
                            Y1 = 0.0F;
                        }

                        // ----- If this is the active text object, draw a thin outline
                        //       around the object.
                        if (DisplayItems.SelectedIndex == counter)
                        {
                            pixelPen = new Pen(Color.LightGray, 1 / e.Graphics.DpiX);
                            pixelPen.DashStyle = DashStyle.Dash;
                            e.Graphics.DrawRectangle(pixelPen, X1, Y1, X2, Y2);
                            pixelPen.Dispose();
                        }

                        // ----- Position the text within the bounding box.
                        switch (textAlign)
                        {
                            case ContentAlignment.TopLeft:
                                textFormat.Alignment = StringAlignment.Near;
                                textFormat.LineAlignment = StringAlignment.Near;
                                break;
                            case ContentAlignment.MiddleLeft:
                                textFormat.Alignment = StringAlignment.Near;
                                textFormat.LineAlignment = StringAlignment.Center;
                                break;
                            case ContentAlignment.BottomLeft:
                                textFormat.Alignment = StringAlignment.Near;
                                textFormat.LineAlignment = StringAlignment.Far;
                                break;
                            case ContentAlignment.TopCenter:
                                textFormat.Alignment = StringAlignment.Center;
                                textFormat.LineAlignment = StringAlignment.Near;
                                break;
                            case ContentAlignment.MiddleCenter:
                                textFormat.Alignment = StringAlignment.Center;
                                textFormat.LineAlignment = StringAlignment.Center;
                                break;
                            case ContentAlignment.BottomCenter:
                                textFormat.Alignment = StringAlignment.Center;
                                textFormat.LineAlignment = StringAlignment.Far;
                                break;
                            case ContentAlignment.TopRight:
                                textFormat.Alignment = StringAlignment.Far;
                                textFormat.LineAlignment = StringAlignment.Near;
                                break;
                            case ContentAlignment.MiddleRight:
                                textFormat.Alignment = StringAlignment.Far;
                                textFormat.LineAlignment = StringAlignment.Center;
                                break;
                            case ContentAlignment.BottomRight:
                                textFormat.Alignment = StringAlignment.Far;
                                textFormat.LineAlignment = StringAlignment.Far;
                                break;
                        }
                        textFormat.FormatFlags = textFormat.FormatFlags |
                            StringFormatFlags.NoWrap;

                        // ----- Print the text.
                        e.Graphics.DrawString(textMessage, useFont, new SolidBrush(textColor),
                            new RectangleF(X1, Y1, X2, Y2), textFormat);
                        useFont.Dispose();
                        break;
                }

                // ----- Restore the original transformed state of the graphics surface.
                e.Graphics.Restore(holdState);
            }

            // ----- Remove the page-size transformations.
            e.Graphics.ResetTransform();
        }

        private void RecordTemplate_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- The user selected a different page template.
            if (IgnoreSizeChange == true)
                return;
            RefreshLabelTemplate();
            RefreshSizeDisplay();
            PreviewArea.Invalidate();
        }

        private void RecordSample_TextChanged(object sender, EventArgs e)
        {
            // ----- Refresh the display on a change.
            PreviewArea.Invalidate();
        }

        private void RecordUnits_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- The user is altering the size.
            char toUnit;
            BarcodeItemGeneric useGeneric;
            BarcodeItemBarcodeImage useBarImage;
            BarcodeItemBarcodeNumber useBarText;
            BarcodeItemText useText;
            BarcodeItemLine useLine;
            BarcodeItemRect useRect;

            RefreshSizeDisplay();
            if (IgnoreSizeChange == true)
                return;
            if (RecordUnits.SelectedIndex == -1)
                return;
            toUnit = (ListItemData)RecordUnits.SelectedItem;

            // ----- Change the units of each object to the new size.
            for (int counter = 0; counter < DisplayItems.Items.Count; counter++)
            {
                useGeneric = (BarcodeItemGeneric)(DisplayItems.Items[counter]);
                switch (useGeneric.ItemType)
                {
                    case "B":
                        // ----- Barcode image.
                        useBarImage = (BarcodeItemBarcodeImage)useGeneric;
                        useBarImage.Left = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Left);
                        useBarImage.Top = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Top);
                        useBarImage.Width = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Width);
                        useBarImage.Height = ConvertPageUnits(CurrentMeasure, toUnit, useBarImage.Height);
                        break;
                    case "L":
                        // ----- Line.
                        useLine = (BarcodeItemLine)useGeneric;
                        useLine.X1 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.X1);
                        useLine.X2 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.X2);
                        useLine.Y1 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.Y1);
                        useLine.Y2 = ConvertPageUnits(CurrentMeasure, toUnit, useLine.Y2);
                        break;
                    case "N":
                        // ----- Barcode text.
                        useBarText = (BarcodeItemBarcodeNumber)useGeneric;
                        useBarText.Left = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Left);
                        useBarText.Top = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Top);
                        useBarText.Width = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Width);
                        useBarText.Height = ConvertPageUnits(CurrentMeasure, toUnit, useBarText.Height);
                        break;
                    case "R":
                        // ----- Rectangle.
                        useRect = (BarcodeItemRect)useGeneric;
                        useRect.Left = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Left);
                        useRect.Top = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Top);
                        useRect.Width = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Width);
                        useRect.Height = ConvertPageUnits(CurrentMeasure, toUnit, useRect.Height);
                        break;
                    case "T":
                        // ----- Text.
                        useText = (BarcodeItemText)useGeneric;
                        useText.Left = ConvertPageUnits(CurrentMeasure, toUnit, useText.Left);
                        useText.Top = ConvertPageUnits(CurrentMeasure, toUnit, useText.Top);
                        useText.Width = ConvertPageUnits(CurrentMeasure, toUnit, useText.Width);
                        useText.Height = ConvertPageUnits(CurrentMeasure, toUnit, useText.Height);
                        break;
                }

                // ----- Update the display item in the list.
                DisplayItems.Items[counter] = DisplayItems.Items[counter];
            }

            // ----- Update the properties area.
            if (DisplayItems.SelectedIndex != -1)
                ItemProperties.SelectedObject = DisplayItems.SelectedItem;
            CurrentMeasure = toUnit;
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            string sqlText;
            DataCommand sqlRun;
            BarcodeItemGeneric useGeneric;
            BarcodeItemBarcodeImage useBarImage;
            BarcodeItemBarcodeNumber useBarText;
            BarcodeItemText useText;
            BarcodeItemLine useLine;
            BarcodeItemRect useRect;
            long newID;
            LibraryTransaction transInfo = null;

            // ----- Confirm the data.
            if (ValidateForm() == false)
                return;
            try
            {
                transInfo = TransactionBegin();
            }
            catch (Exception ex)
            {
                GeneralError("BarcodeLabel.ActOK_Click.Trans", ex);
                return;
            }
            this.Cursor = Cursors.WaitCursor;

            // ----- Save the parent record data.
            if (ActiveLabelID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO BarcodeLabel (FullName, BarcodeSheet, UnitType) " +
                    "OUTPUT INSERTED.ID VALUES (@NewName, @NewSheet, @NewUnit)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordName.Text.Trim());
                sqlRun.AddLong("NewSheet", (long)(ListItemData)RecordTemplate.SelectedItem);
                sqlRun.AddText("NewUnit", ((char)(ListItemData)RecordUnits.SelectedItem).ToString());
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun, transInfo));
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    TransactionRollback(transInfo);
                    GeneralError("BarcodeLabel.ActOK_Click.AddParent", ex);
                    return;
                }
                ActiveLabelID = newID;
            }
            else
            {
                // ----- Update the existing entry.
                sqlText = "UPDATE BarcodeLabel SET FullName = @NewName, " +
                    "BarcodeSheet = @NewSheet, UnitType = @NewUnit " +
                    "WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordName.Text.Trim());
                sqlRun.AddLong("NewSheet", (long)(ListItemData)RecordTemplate.SelectedItem);
                sqlRun.AddText("NewUnit", ((char)(ListItemData)RecordUnits.SelectedItem).ToString());
                sqlRun.AddLong("RecordID", ActiveLabelID);
                try
                {
                    ExecuteSQL(sqlRun, transInfo);
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    TransactionRollback(transInfo);
                    GeneralError("BarcodeLabel.ActOK_Click.UpdateParent", ex);
                    return;
                }

                // ----- Remove the detail records.
                sqlText = "DELETE FROM BarcodeLabelItem WHERE BarcodeLabel = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", ActiveLabelID);
                try
                {
                    ExecuteSQL(sqlRun, transInfo);
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    TransactionRollback(transInfo);
                    GeneralError("BarcodeLabel.ActOK_Click.ClearItem", ex);
                    return;
                }
            }

            // ----- Add the detail records.
            for (int counter = 0; counter < DisplayItems.Items.Count; counter++)
            {
                // ----- Build the command to add the record.
                useGeneric = (BarcodeItemGeneric)DisplayItems.Items[counter];
                sqlText = "INSERT INTO BarcodeLabelItem (Priority, BarcodeLabel, ItemType, " +
                    "PosLeft, PosTop, PosWidth, PosHeight, Rotation, FontName, FontSize, " +
                    "FontStyle, StaticText, Color1, Color2, Alignment, PadDigits) VALUES (" +
                    "@NewPriority, @NewBarcode, @NewType, @NewLeft, @NewTop, @NewWidth, " +
                    "@NewHeight, @NewRotation, @NewFont, @NewSize, @NewStyle, @NewText, " +
                    "@NewColor1, @NewColor2, @NewAlign, @NewPad)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddInteger("NewPriority", counter + 1);
                sqlRun.AddLong("NewBarcode", ActiveLabelID);
                sqlRun.AddText("NewType", useGeneric.ItemType);
                switch (useGeneric.ItemType)
                {
                    case "B":
                        // ----- Barcode image.
                        useBarImage = (BarcodeItemBarcodeImage)useGeneric;
                        sqlRun.AddDecimal("NewLeft", (decimal)useBarImage.Left);
                        sqlRun.AddDecimal("NewTop", (decimal)useBarImage.Top);
                        sqlRun.AddDecimal("NewWidth", (decimal)useBarImage.Width);
                        sqlRun.AddDecimal("NewHeight", (decimal)useBarImage.Height);
                        sqlRun.AddInteger("NewRotation", useBarImage.RotationAngle);
                        sqlRun.AddText("NewFont");
                        sqlRun.AddDecimal("NewSize", (decimal)useBarImage.BarcodeSize);
                        sqlRun.AddText("NewStyle");
                        sqlRun.AddText("NewText");
                        sqlRun.AddLong("NewColor1", useBarImage.BarcodeColor.ToArgb());
                        sqlRun.AddLong("NewColor2");
                        sqlRun.AddInteger("NewAlign", (int)useBarImage.Alignment);
                        sqlRun.AddInteger("NewPad", useBarImage.PadDigits);
                        break;
                    case "L":
                        // ----- Line.
                        useLine = (BarcodeItemLine)useGeneric;
                        sqlRun.AddDecimal("NewLeft", (decimal)useLine.X1);
                        sqlRun.AddDecimal("NewTop", (decimal)useLine.Y1);
                        sqlRun.AddDecimal("NewWidth", (decimal)useLine.X2);
                        sqlRun.AddDecimal("NewHeight", (decimal)useLine.Y2);
                        sqlRun.AddInteger("NewRotation", 0);
                        sqlRun.AddText("NewFont");
                        sqlRun.AddDecimal("NewSize");
                        sqlRun.AddText("NewStyle");
                        sqlRun.AddText("NewText");
                        sqlRun.AddLong("NewColor1", useLine.LineColor.ToArgb());
                        sqlRun.AddLong("NewColor2");
                        sqlRun.AddInteger("NewAlign");
                        sqlRun.AddInteger("NewPad");
                        break;
                    case "N":
                        // ----- Barcode text.
                        useBarText = (BarcodeItemBarcodeNumber)useGeneric;
                        sqlRun.AddDecimal("NewLeft", (decimal)useBarText.Left);
                        sqlRun.AddDecimal("NewTop", (decimal)useBarText.Top);
                        sqlRun.AddDecimal("NewWidth", (decimal)useBarText.Width);
                        sqlRun.AddDecimal("NewHeight", (decimal)useBarText.Height);
                        sqlRun.AddInteger("NewRotation", useBarText.RotationAngle);
                        sqlRun.AddText("NewFont", useBarText.Font.Name);
                        sqlRun.AddDecimal("NewSize", (decimal)useBarText.Font.SizeInPoints);
                        sqlRun.AddText("NewStyle", DBFontStyle(useBarText.Font));
                        sqlRun.AddText("NewText");
                        sqlRun.AddLong("NewColor1", useBarText.TextColor.ToArgb());
                        sqlRun.AddLong("NewColor2");
                        sqlRun.AddInteger("NewAlign", (int)useBarText.Alignment);
                        sqlRun.AddInteger("NewPad", useBarText.PadDigits);
                        break;
                    case "R":
                        // ----- Rectangle.
                        useRect = (BarcodeItemRect)useGeneric;
                        sqlRun.AddDecimal("NewLeft", (decimal)useRect.Left);
                        sqlRun.AddDecimal("NewTop", (decimal)useRect.Top);
                        sqlRun.AddDecimal("NewWidth", (decimal)useRect.Width);
                        sqlRun.AddDecimal("NewHeight", (decimal)useRect.Height);
                        sqlRun.AddInteger("NewRotation", useRect.RotationAngle);
                        sqlRun.AddText("NewFont");
                        sqlRun.AddDecimal("NewSize");
                        sqlRun.AddText("NewStyle");
                        sqlRun.AddText("NewText");
                        sqlRun.AddLong("NewColor1", useRect.BorderColor.ToArgb());
                        sqlRun.AddLong("NewColor2", useRect.FillColor.ToArgb());
                        sqlRun.AddInteger("NewAlign");
                        sqlRun.AddInteger("NewPad");
                        break;
                    case "T":
                        // ----- Text.
                        useText = (BarcodeItemText)useGeneric;
                        sqlRun.AddDecimal("NewLeft", (decimal)useText.Left);
                        sqlRun.AddDecimal("NewTop", (decimal)useText.Top);
                        sqlRun.AddDecimal("NewWidth", (decimal)useText.Width);
                        sqlRun.AddDecimal("NewHeight", (decimal)useText.Height);
                        sqlRun.AddInteger("NewRotation", useText.RotationAngle);
                        sqlRun.AddText("NewFont", useText.Font.Name);
                        sqlRun.AddDecimal("NewSize", (decimal)useText.Font.SizeInPoints);
                        sqlRun.AddText("NewStyle", DBFontStyle(useText.Font));
                        sqlRun.AddText("NewText", useText.DisplayText);
                        sqlRun.AddLong("NewColor1", useText.TextColor.ToArgb());
                        sqlRun.AddLong("NewColor2");
                        sqlRun.AddInteger("NewAlign", (int)useText.Alignment);
                        sqlRun.AddInteger("NewPad");
                        break;
                }

                // ----- Add the record.
                try
                {
                    ExecuteSQL(sqlRun, transInfo);
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    TransactionRollback(transInfo);
                    GeneralError("BarcodeLabel.ActOK_Click.Item", ex);
                    return;
                }
            }

            // ----- Commit all changes.
            try
            {
                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                TransactionRollback(transInfo);
                GeneralError("BarcodeLabel.ActOK_Click.Commit", ex);
                return;
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }

            // ----- Continue with AddRecord() or EditRecord()
            this.DialogResult = DialogResult.OK;
        }
    }
}
