﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class CodeSeries : Library.BaseCodeForm
    {
        private long ActiveID;
        private bool StartingOver;

        public CodeSeries()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            ActiveID = -1L;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        // ***** Insert Code Snippet #7 here.

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManageSeries];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManageSeries];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManageSeries];
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the records.
            string sqlText;
            DataCommand sqlRun;

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete the series?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                    DialogResult.Yes)
                return false;

            // ----- Make sure this record is not in use.
            sqlText = "SELECT COUNT(*) FROM NamedItem WHERE Series = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    MessageBox.Show("You cannot delete this record because it is in use.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("CodeSeries.DeleteRecord.Check", ex);
                return false;
            }

            // ----- Delete the record.
            sqlText = "DELETE FROM CodeSeries WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("CodeSeries.DeleteRecord.Delete", ex);
                return false;
            }

            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record.
            ActiveID = recordID;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items.
            string sqlText;
            DataTable dbInfo;

            // ----- Clear any existing values.
            destList.Items.Clear();
            exceededMatches = false;

            // ----- Retrieve the values from the database.
            sqlText = "SELECT * FROM CodeSeries ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("CodeSeries.FillListWithRecords", ex);
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                destList.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;

            // ----- Retrieve the value from the database.
            sqlText = "SELECT FullName FROM CodeSeries WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                return Convert.ToString(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("CodeSeries.FormatRecordName", ex);
                return "Error";
            }
        }

        public override string GetEditDescription()
        {
            // ----- Return the list form display title.
            return "Use these codes to identify groups of library items that were " +
                "designed by the publisher to appear under a named series.";
        }

        public override string GetEditTitle()
        {
            // ----- Return the list form display title.
            return "Series Names";
        }

        private void PrepareFormFields()
        {
            // ----- Fill in the data.
            string sqlText;
            DataCommand sqlRun;
            int inUse;

            // ----- Clear the existing data.
            StartingOver = true;
            RecordFullName.Text = "";
            RecordStatus.Text = "New series record";
            if (ActiveID == -1L)
                return;

            // ----- Load in the values stored in the database.
            sqlText = "SELECT FullName FROM CodeSeries WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                RecordFullName.Text = DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("CodeSeries.PrepareFormFields.Retrieve", ex);
            }

            // ----- Find out how many records link to this publisher.
            sqlText = "SELECT COUNT(*) AS TheCount FROM NamedItem " +
                "WHERE Series = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                inUse = DBGetInteger(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("CodeSeries.PrepareFormFields.Link", ex);
                inUse = 0;
            }
            RecordStatus.Text = "Linked to " + inUse.ToString() + " library item";
            if (inUse != 1)
                RecordStatus.Text += "s";
        }

        private bool SaveFormData()
        {
            // ----- The user wants to save changes.
            string sqlText;
            DataCommand sqlRun;
            long newID;

            this.Cursor = Cursors.WaitCursor;

            // ----- Save the data.
            if (ActiveID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO CodeSeries (FullName) " +
                    "OUTPUT INSERTED.ID VALUES (@NewName)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    GeneralError("CodeSeries.SaveFormData.Add", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
                ActiveID = newID;
            }
            else
            {
                // ----- Update the existing entry.
                sqlText = "UPDATE CodeSeries SET FullName = @NewName " +
                    "WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                sqlRun.AddLong("RecordID", ActiveID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("CodeSeries.SaveFormData.Update", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }

            // ----- Success.
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            string sqlText;
            DataCommand sqlRun;

            // ----- Name is a required field.
            if (RecordFullName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the series name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordFullName.Focus();
                return false;
            }

            // ----- Make sure the name is not already in use.
            sqlText = "SELECT COUNT(*) FROM CodeSeries " +
                "WHERE UPPER(FullName) = @TestName";
            if (ActiveID != -1L)
                sqlText += " AND ID <> @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestName", RecordFullName.Text.Trim().ToUpper());
            if (ActiveID != -1L)
                sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    // ----- Duplicate entry.
                    MessageBox.Show("A record already exists with the name '" +
                        RecordFullName.Text.Trim() + "'.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordFullName.Focus();
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("CodeSeries.ValidateFormData", ex);
                return false;
            }

            // ----- Success.
            return true;
        }

        private void RecordFullName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordFullName.SelectAll();
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void CodeSeries_Activated(object sender, EventArgs e)
        {
            // ----- Return the focus to the main field.
            if (StartingOver)
                RecordFullName.Focus();
            StartingOver = false;
        }
    }
}
