﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class UserName : Library.BaseCodeForm
    {
        private long ActiveID;
        private bool StartingOver;
        private string OrigLoginID;

        public UserName()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            ActiveID = -1L;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManageUsers];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManageUsers];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManageUsers];
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the record.
            string sqlText;
            DataCommand sqlRun;
            LibraryTransaction transInfo = null;

            // ----- Do not delete the current user.
            if (LoggedInUserID == recordID)
            {
                MessageBox.Show("You cannot delete the active user record.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete the selected user record?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                    DialogResult.Yes)
                return false;

            // ----- Delete the record.
            try
            {
                transInfo = TransactionBegin();

                sqlText = "UPDATE PatronPayment SET UserID = NULL WHERE UserID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", recordID);
                ExecuteSQL(sqlRun, transInfo);

                sqlText = "DELETE FROM UserName WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", recordID);
                ExecuteSQL(sqlRun, transInfo);

                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                if (transInfo != null)
                    TransactionRollback(transInfo);
                GeneralError("UserName.DeleteRecord", ex);
                return false;
            }
            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record.
            ActiveID = recordID;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items.
            string sqlText;
            DataTable dbInfo;
            string itemText;

            // ----- Clear any existing values.
            destList.Items.Clear();
            exceededMatches = false;

            // ----- Retrieve the values from the database.
            sqlText = "SELECT ID, FullName, Active FROM UserName ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("UserName.FillListWithRecords", ex);
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                itemText = DBGetText(dbRow["FullName"]);
                if (DBGetBool(dbRow["Active"]) == false)
                    itemText += " (Inactive)";
                destList.Items.Add(new ListItemData(itemText, DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            string itemText;

            // ----- Initialize.
            itemText = "";

            // ----- Retrieve the value from the database.
            sqlText = "SELECT FullName, Active FROM UserName WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("UserName.FormatRecordName", ex);
                return "Error";
            }
            if (dbInfo.Rows.Count > 0)
            {
                oneRow = dbInfo.Rows[0];
                itemText = DBGetText(oneRow["FullName"]);
                if (DBGetBool(oneRow["Active"]) == false)
                    itemText += " (Inactive)";
            }
            dbInfo.Dispose();
            dbInfo = null;
            return itemText;
        }

        public override string GetEditDescription()
        {
            // ----- Return the list form display title.
            return "Use these codes to identify the administrative uses who have " +
                "access to the system.";
        }

        public override string GetEditTitle()
        {
            // ----- Return the list form display title.
            return "Administrative Users";
        }

        private void PrepareFormFields()
        {
            // ----- Fill in the data.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;

            // ----- Load in the list of security groups.
            if (RecordGroup.Items.Count == 0)
            {
                // ----- This is the first time. Load from the database.
                RecordGroup.Items.Clear();
                RecordGroup.Items.Add(new ListItemData("<Not Selected>", -1L));
                sqlText = "SELECT * FROM GroupName ORDER BY FullName";
                try
                {
                    dbInfo = CreateDataTable(sqlText);
                }
                catch (Exception ex)
                {
                    GeneralError("UserName.PrepareFormFields.Groups", ex);
                    return;
                }

                foreach (DataRow dbRow in dbInfo.Rows)
                {
                    RecordGroup.Items.Add(new ListItemData(
                        DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
                }
                dbInfo.Dispose();
                dbInfo = null;
            }

            // ----- Clear the existing data.
            StartingOver = true;
            RecordFullName.Text = "";
            RecordLoginID.Text = "";
            RecordPassword.Text = "";
            RecordConfirm.Text = "";
            RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(-1L);
            RecordActive.Checked = true;
            OrigLoginID = "";
            if (ActiveID == -1L)
                return;

            // ----- Load in the values stored in the database.
            sqlText = "SELECT * FROM UserName WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("UserName.PrepareFormFields.Retrieve", ex);
                return;
            }
            if (dbInfo.Rows.Count > 0)
            {
                // ----- We never reload the password.
                oneRow = dbInfo.Rows[0];
                RecordFullName.Text = DBGetText(oneRow["FullName"]);
                RecordLoginID.Text = DBGetText(oneRow["LoginID"]);
                OrigLoginID = DBGetText(oneRow["LoginID"]);
                RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(
                    DBGetLong(oneRow["GroupID"]));
                RecordActive.Checked = DBGetBool(oneRow["Active"]);
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        private bool SaveFormData()
        {
            // ----- The user wants to save changes. Return True on success.
            string sqlText;
            DataCommand sqlRun;
            string passwordResult = "";
            bool savePassword;
            long newID = -1L;

            // ----- Prepare to save the data.
            this.Cursor = Cursors.WaitCursor;

            // ----- Save the data.
            if (ActiveID == -1L)
            {
                // ----- Encrypt the password text.
                passwordResult = EncryptPassword(
                    RecordLoginID.Text.Trim(), RecordPassword.Text.Trim());

                // ----- Create a new entry.
                sqlText = "INSERT INTO UserName(FullName, LoginID, [Password], " +
                    "Active, GroupID) OUTPUT INSERTED.ID VALUES (@NewName, " +
                    "@NewLogin, @NewPassword, @NewActive, @NewGroup)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                sqlRun.AddText("NewLogin", RecordLoginID.Text.Trim());
                sqlRun.AddText("NewPassword", passwordResult);
                sqlRun.AddBool("NewActive", RecordActive.Checked);
                sqlRun.AddLong("NewGroup", DBComboParam(RecordGroup));
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    GeneralError("UserName.SaveFormData.Add", ex);
                    return false;
                }
                ActiveID = newID;
            }
            else
            {
                // ----- Update the existing entry.
                sqlText = "UPDATE UserName SET FullName = @NewName" +
                    ", LoginID = @NewLogin, Active = @NewActive" +
                    ", GroupID = @NewGroup";

                // ----- See if the password should be saved.
                savePassword = false;
                if (OrigLoginID.Length == 0)
                    savePassword = true;
                if (OrigLoginID.Trim().ToUpper() != RecordLoginID.Text.Trim().ToUpper())
                    savePassword = true;
                if (RecordPassword.Text.Trim().Length != 0)
                    savePassword = true;
                if (savePassword == true)
                {
                    if (RecordPassword.Text.Trim().ToUpper() == "NP")
                    {
                        // ----- NP removes a password.
                        passwordResult = EncryptPassword(RecordLoginID.Text.Trim(), "");
                    }
                    else
                    {
                        passwordResult = EncryptPassword(
                            RecordLoginID.Text.Trim(), RecordPassword.Text.Trim());
                    }
                    sqlText += ", [Password] = @NewPassword";
                }
                sqlText += " WHERE ID = @RecordID";

                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                sqlRun.AddText("NewLogin", RecordLoginID.Text.Trim());
                sqlRun.AddBool("NewActive", RecordActive.Checked);
                sqlRun.AddLong("NewGroup", DBComboParam(RecordGroup));
                if (savePassword == true)
                    sqlRun.AddText("NewPassword", passwordResult);
                sqlRun.AddLong("RecordID", ActiveID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    GeneralError("UserName.SaveFormData.Update", ex);
                    return false;
                }
            }

            // ----- This change may affect this user.
            if (LoggedInGroupID == ActiveID)
                ReprocessSecuritySet();

            // ----- Success.
            this.Cursor = Cursors.Default;
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            string sqlText;
            DataCommand sqlRun;

            // ----- Name is a required field.
            if (RecordFullName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the User Name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordFullName.Focus();
                return false;
            }

            // ----- Login ID is a required field.
            if (RecordLoginID.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the Login ID.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordLoginID.Focus();
                return false;
            }

            // ----- Password and confirm must match.
            if (RecordPassword.Text != RecordConfirm.Text)
            {
                MessageBox.Show("The Password and Confirm fields must match.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordPassword.Focus();
                return false;
            }

            // ----- If the login ID changed and there is no password, warn the user that
            //       the new login ID will have no password.
            if ((OrigLoginID.Trim().ToUpper() != RecordLoginID.Text.Trim().ToUpper()) &
                (OrigLoginID.Length != 0) & (RecordPassword.Text.Trim().Length == 0))
            {
                if (MessageBox.Show("When changing the Login ID, the password must also be " +
                        "supplied. If you leave the Password field blank, no password will be " +
                        "assigned to this user.  Do you wish to continue with a blank password?",
                        ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                        DialogResult.Yes)
                    return false;
            }

            // ----- Make sure the login ID is not already in use.
            sqlText = "SELECT COUNT(*) FROM UserName WHERE UPPER(LoginID) = @TestLogin";
            if (ActiveID != -1L)
                sqlText += " AND ID <> @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestLogin", RecordLoginID.Text.Trim().ToUpper());
            if (ActiveID != -1L)
                sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    MessageBox.Show("A record already exists with the Login ID '" +
                        RecordLoginID.Text.Trim() + "'.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordLoginID.Focus();
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("UserName.ValidateFormData", ex);
                return false;
            }

            // ----- Success.
            return true;
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- Save changes and return to the summary form.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void RecordConfirm_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordConfirm.SelectAll();
        }

        private void RecordFullName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordFullName.SelectAll();
        }

        private void RecordLoginID_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordLoginID.SelectAll();
        }

        private void RecordPassword_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordPassword.SelectAll();
        }

        private void UserName_Activated(object sender, EventArgs e)
        {
            // ----- Return the focus to the main field.
            if (StartingOver)
                RecordFullName.Focus();
            StartingOver = false;
        }
    }
}
