﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class CodeMediaType : Library.BaseCodeForm
    {
        private long ActiveID;
        private bool StartingOver;

        public CodeMediaType()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            ActiveID = -1L;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManageMediaTypes];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManageMediaTypes];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManageMediaTypes];
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the records.
            string sqlText;
            DataCommand sqlRun;

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete the media type?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                    DialogResult.Yes)
                return false;


            // ----- Make sure this record is not in use.
            sqlText = "SELECT COUNT(*) FROM NamedItem WHERE MediaType = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    MessageBox.Show("You cannot delete this record because it is in use.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("CodeMediaType.DeleteRecord.Check", ex);
                return false;
            }

            // ----- Delete the record.
            sqlText = "DELETE FROM CodeMediaType WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("CodeMediaType.DeleteRecord.Delete", ex);
                return false;
            }
            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record.
            ActiveID = recordID;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items.
            string sqlText;
            DataTable dbInfo;

            // ----- Clear any existing values.
            destList.Items.Clear();
            exceededMatches = false;

            // ----- Retrieve the values from the database.
            sqlText = "SELECT ID, FullName FROM CodeMediaType ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("CodeMediaType.FillListWithRecords", ex);
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                destList.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;

            // ----- Retrieve the value from the database.
            sqlText = "SELECT FullName FROM CodeMediaType WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                return DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("CodeMediaType.FormatRecordName", ex);
                return "Error";
            }
        }

        public override string GetEditDescription()
        {
            // ----- Return the list form display title.
            return "Use these codes to distinguish between the different types of " +
                "materials available in the library, such as books, CDs, etc.";
        }

        public override string GetEditTitle()
        {
            // ----- Return the list form display title.
            return "Media Types";
        }

        private void PrepareFormFields()
        {
            // ----- Fill in the data.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;

            // ----- Clear the existing data.
            StartingOver = true;
            RecordFullName.Text = "";
            RecordCheckoutDays.Text = "";
            RecordRenewDays.Text = "";
            RecordRenewTimes.Text = "";
            RecordDailyFine.Text = "";
            if (ActiveID == -1L)
                return;

            // ----- Load in the values stored in the database.
            sqlText = "SELECT * FROM CodeMediaType WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("CodeMediaType.PrepareFormFields", ex);
                return;
            }

            if (dbInfo.Rows.Count > 0)
            {
                oneRow = dbInfo.Rows[0];
                RecordFullName.Text = DBGetText(oneRow["FullName"]);
                RecordCheckoutDays.Text = DBGetInteger(oneRow["CheckoutDays"]).ToString();
                RecordRenewDays.Text = DBGetInteger(oneRow["RenewDays"]).ToString();
                RecordRenewTimes.Text = DBGetInteger(oneRow["RenewTimes"]).ToString();
                RecordDailyFine.Text = DBGetDecimal(oneRow["DailyFine"]).ToString("0.00");
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        private bool SaveFormData()
        {
            // ----- Save changes to the database.
            string sqlText;
            DataCommand sqlRun;
            long newID;

            this.Cursor = Cursors.WaitCursor;

            // ----- Save the data.
            if (ActiveID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO CodeMediaType (FullName, CheckoutDays, " +
                    "RenewDays, RenewTimes, DailyFine) OUTPUT INSERTED.ID " +
                    "VALUES (@NewName, @NewCheckout, @NewRenew, @NewTimes, @NewFine)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                sqlRun.AddInteger("NewCheckout", RecordCheckoutDays.Text);
                sqlRun.AddInteger("NewRenew", RecordRenewDays.Text);
                sqlRun.AddInteger("NewTimes", RecordRenewTimes.Text);
                sqlRun.AddDecimal("NewFine", RecordDailyFine.Text);
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    GeneralError("CodeMediaType.SaveFormData.Add", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
                ActiveID = newID;
            }
            else
            {
                // ----- Update the existing entry.
                sqlText = "UPDATE CodeMediaType SET FullName = @NewName, " +
                    "CheckoutDays = @NewCheckout, RenewDays = @NewRenew, " +
                    "RenewTimes = @NewTimes, DailyFine = @NewFine " +
                    "WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                sqlRun.AddInteger("NewCheckout", RecordCheckoutDays.Text);
                sqlRun.AddInteger("NewRenew", RecordRenewDays.Text);
                sqlRun.AddInteger("NewTimes", RecordRenewTimes.Text);
                sqlRun.AddDecimal("NewFine", RecordDailyFine.Text);
                sqlRun.AddLong("RecordID", ActiveID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("CodeMediaType.SaveFormData.Update", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }

            // ----- Success.
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            string sqlText;
            DataCommand sqlRun;
            int holdValue;

            // ----- Name is a required field.
            if (RecordFullName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the media type name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordFullName.Focus();
                return false;
            }

            // ----- Checkout days must be an integer.
            if (int.TryParse(RecordCheckoutDays.Text, out holdValue) == false)
                holdValue = -1;
            if ((RecordCheckoutDays.Text.Contains(".") == true) | (holdValue < 0) | (holdValue > 99))
            {
                MessageBox.Show("Checkout Days must be an integer between 0 and 99.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordCheckoutDays.Focus();
                return false;
            }

            // ----- Renewal Days must be an integer.
            if (int.TryParse(RecordRenewDays.Text, out holdValue) == false)
                holdValue = -1;
            if ((RecordRenewDays.Text.Contains(".") == true) | (holdValue < 0) | (holdValue > 99))
            {
                MessageBox.Show("Renewal Days must be an integer between 0 and 99.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordRenewDays.Focus();
                return false;
            }

            // ----- Renewal Limit must be an integer.
            if (int.TryParse(RecordRenewTimes.Text, out holdValue) == false)
                holdValue = -1;
            if ((RecordRenewTimes.Text.Contains(".") == true) | (holdValue < 0) | (holdValue > 99))
            {
                MessageBox.Show("Renewal Limit must be an integer between 0 and 99.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordRenewTimes.Focus();
                return false;
            }

            // ----- Daily Fine must be a number.
            if (CheckDecimal(RecordDailyFine.Text) == false)
            {
                MessageBox.Show("Daily Fine must be a valid number.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordDailyFine.Focus();
                return false;
            }

            // ----- Make sure the name is not already in use.
            sqlText = "SELECT COUNT(*) FROM CodeMediaType " +
                "WHERE UPPER(FullName) = @NewName ";
            if (ActiveID != -1L)
                sqlText += " AND ID <> @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("NewName", RecordFullName.Text.Trim().ToUpper());
            if (ActiveID != -1L)
                sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    // ----- Duplicate entry.
                    MessageBox.Show("A record already exists with the name '" +
                        RecordFullName.Text.Trim() + "'.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordFullName.Focus();
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("CodeMediaType.ValidateFormData", ex);
                return false;
            }

            // ----- Success.
            return true;
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void CodeMediaType_Activated(object sender, EventArgs e)
        {
            // ----- Return the focus to the main field.
            if (StartingOver)
                RecordFullName.Focus();
            StartingOver = false;
        }

        private void RecordFullName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordFullName.SelectAll();
        }

        private void RecordCheckoutDays_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordCheckoutDays.SelectAll();
        }

        private void RecordRenewDays_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordRenewDays.SelectAll();
        }

        private void RecordRenewTimes_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordRenewTimes.SelectAll();
        }

        private void RecordDailyFine_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordDailyFine.SelectAll();
        }

        private void RecordCheckoutDays_KeyPress(object sender, KeyPressEventArgs e)
        {
            // ----- Only allow digits and backspaces.
            if ((e.KeyChar.ToString() == "\b") | (Char.IsDigit(e.KeyChar) == true))
                return;
            e.Handled = true;
        }

        private void RecordRenewDays_KeyPress(object sender, KeyPressEventArgs e)
        {
            // ----- Only allow digits and backspaces.
            if ((e.KeyChar.ToString() == "\b") | (Char.IsDigit(e.KeyChar) == true))
                return;
            e.Handled = true;
        }

        private void RecordRenewTimes_KeyPress(object sender, KeyPressEventArgs e)
        {
            // ----- Only allow digits and backspaces.
            if ((e.KeyChar.ToString() == "\b") | (Char.IsDigit(e.KeyChar) == true))
                return;
            e.Handled = true;
        }

        private void RecordDailyFine_KeyPress(object sender, KeyPressEventArgs e)
        {
            // ----- Only allow digits, decimals, and backspaces.
            if ((e.KeyChar.ToString() == "\b") | (Char.IsDigit(e.KeyChar) == true) |
                    (e.KeyChar == '.'))
                return;
            e.Handled = true;
        }
    }
}
