﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.ComponentModel;
using System.Drawing;

namespace Library
{
    public abstract class BarcodeItemGeneric
    {
        [Browsable(false)]
        public abstract string ItemType {get;}
        public abstract override string ToString();
    }

    public class BarcodeItemText : BarcodeItemGeneric
    {
        // ----- Includes a static text element in a barcode label.

        // ----- Private store of properties.
        private short StoredTextAngle;
        private string StoredDisplayText;

        // ----- The ItemType constant helps when saving the record
        //       to the database.
        [Browsable(false)]
        public override string ItemType
        {
            get
            {
                return "T";
            }
        }

        public override string ToString()
        {
            if (StoredDisplayText.Length == 0)
                return "Text: [Undefined]";
            else
                return $"Text: {StoredDisplayText}";
        }

        // ----- Upper-left corner of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the left-edge position of the text box.")]
        public decimal Left {get; set;}

        // ----- Upper-left corner of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the top-edge position of the text box.")]
        public decimal Top {get; set;}

        // ----- The width of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the width of the text box.")]
        public decimal Width {get; set;}

        // ----- The height of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the height of the text box.")]
        public decimal Height {get; set;}

        // ----- The text color.
        [Browsable(true),
            DescriptionAttribute("Sets the color of the displayed text.")]
        public Color TextColor {get; set;}

        // ----- Rotation angle of the text.
        [Browsable(true),
            DescriptionAttribute("Sets the clockwise rotation angle, from 0 to 359 degrees.")]
        public short RotationAngle
        {
            get
            {
                return StoredTextAngle;
            }
            set
            {
                // ----- Make sure the angle is in the valid range.
                if ((value < 0) | (value > 359))
                    throw new ArgumentException($"Invalid rotation angle: {value}. " +
                        "Rotation angle must range from 0 to 359 degrees.");
                StoredTextAngle = value;
            }
        }

        // ----- The font of the text.
        [Browsable(true),
            DescriptionAttribute("Sets the font of the text.")]
        public Font Font {get; set;}

        // ----- The actual text.
        [Browsable(true),
            DescriptionAttribute("Sets the text to display.")]
        public string DisplayText
        {
            get
            {
                return StoredDisplayText;
            }
            set
            {
                // ----- Make sure the angle is in the valid range.
                if (value.Length > 100)
                    throw new ArgumentException(
                        "Invalid description. Text is limited to 100 characters maximum.");
                StoredDisplayText = value;
            }
        }

        // ----- The alignment of the text.
        [Browsable(true),
            DescriptionAttribute("Sets the alignment of the text.")]
        public ContentAlignment Alignment {get; set;}
    }

    public class BarcodeItemLine : BarcodeItemGeneric
    {
        // ----- Includes a basic line element in a barcode label.

        // ----- The ItemType constant helps when saving the record
        //       to the database.
        [Browsable(false)]
        public override string ItemType
        {
            get
            {
                return "L";
            }
        }

        public override string ToString()
        {
            return $"Line: ({this.X1},{this.Y1}) to ({this.X2},{this.Y2})";
        }

        // ----- Starting point of the line.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the starting horizontal position of the line.")]
        public decimal X1 {get; set;}

        // ----- Starting point of the line.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the starting vertical position of the line.")]
        public decimal Y1 {get; set;}

        // ----- Ending point of the line.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the ending horizontal position of the line.")]
        public decimal X2 {get; set;}

        // ----- Ending point of the line.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the ending vertical position of the line.")]
        public decimal Y2 {get; set;}

        // ----- The line color.
        [Browsable(true),
            DescriptionAttribute("Sets the color of the line.")]
        public Color LineColor {get; set;}
    }

    public class BarcodeItemBarcodeImage : BarcodeItemGeneric
    {
        // ----- Includes an actual barcode element in a barcode label.

        // ----- Private store of attributes.
        private short StoredBarAngle;
        private byte StoredBarPadDigits;

        // ----- The ItemType constant helps when saving the record
        //       to the database.
        [Browsable(false)]
        public override string ItemType
        {
            get
            {
                return "B";
            }
        }

        public override string ToString()
        {
            return $"Barcode: At ({this.Left},{this.Top})";
        }

        // ----- Upper-left corner of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the left-edge position of the barcode box.")]
        public decimal Left {get; set;}

        // ----- Upper-left corner of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the top-edge position of the barcode box.")]
        public decimal Top {get; set;}

        // ----- The width of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the width of the barcode box.")]
        public decimal Width {get; set;}

        // ----- The height of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the height of the barcode box.")]
        public decimal Height {get; set;}

        // ----- The barcode font size.
        [Browsable(true),
            DescriptionAttribute("Sets the size of the barcode, in points.")]
        public decimal BarcodeSize {get; set;}

        // ----- The barcode color.
        [Browsable(true),
            DescriptionAttribute("Sets the color of the displayed barcode.")]
        public Color BarcodeColor {get; set;}

        // ----- Rotation angle of the text.
        [Browsable(true),
            DescriptionAttribute("Sets the rotation angle, from 0 to 359 degrees.")]
        public short RotationAngle
        {
            get
            {
                return StoredBarAngle;
            }
            set
            {
                // ----- Make sure the angle is in the valid range.
                if ((value < 0) | (value > 359))
                    throw new ArgumentException($"Invalid rotation angle: {value}. " +
                        "Rotation angle must range from 0 to 359 degrees.");
                StoredBarAngle = value;
            }
        }

        // ----- The alignment of the text.
        [Browsable(true),
            DescriptionAttribute("Sets the alignment of the barcode.")]
        public ContentAlignment Alignment {get; set;}

        // ----- Number of pad digits.
        [Browsable(true), DescriptionAttribute(
            "The number of digits in which to pad the barcode number, or 0 to ignore.")]
        public byte PadDigits
        {
            get
            {
                return StoredBarPadDigits;
            }
            set
            {
                // ----- Make sure the count is in the valid range.
                if ((value < 0) | (value > 20))
                    throw new ArgumentException($"Invalid pad digits value: {value}. " +
                        "Pad digits must range from 0 to 20.");
                StoredBarPadDigits = value;
            }
        }
    }

    public class BarcodeItemBarcodeNumber : BarcodeItemGeneric
    {
        // ----- Includes the numeric version of a barcode in a barcode label.

        // ----- Private store of attributes.
        private short StoredTextAngle;
        private byte StoredTextPadDigits;

        // ----- The ItemType constant helps when saving the record
        //       to the database.
        [Browsable(false)]
        public override string ItemType
        {
            get
            {
                return "N";
            }
        }

        public override string ToString()
        {
            return $"Barcode Text: At ({this.Left},{this.Top})";
        }

        // ----- Upper-left corner of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the left-edge position of the text box.")]
        public decimal Left {get; set;}

        // ----- Upper-left corner of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the top-edge position of the text box.")]
        public decimal Top {get; set;}

        // ----- The width of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the width of the text box.")]
        public decimal Width {get; set;}

        // ----- The height of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the height of the text box.")]
        public decimal Height {get; set;}

        // ----- The text color.
        [Browsable(true),
            DescriptionAttribute("Sets the color of the displayed text.")]
        public Color TextColor {get; set;}

        // ----- Rotation angle of the text.
        [Browsable(true),
            DescriptionAttribute("Sets the rotation angle, from 0 to 359 degrees.")]
        public short RotationAngle
        {
            get
            {
                return StoredTextAngle;
            }
            set
            {
                // ----- Make sure the angle is in the valid range.
                if ((value < 0) | (value > 359))
                    throw new ArgumentException($"Invalid rotation angle: {value}. " +
                        "Rotation angle must range from 0 to 359 degrees.");
                StoredTextAngle = value;
            }
        }

        // ----- The font of the text.
        [Browsable(true),
            DescriptionAttribute("Sets the font of the text.")]
        public Font Font {get; set;}

        // ----- The alignment of the text.
        [Browsable(true),
            DescriptionAttribute("Sets the alignment of the text.")]
        public ContentAlignment Alignment {get; set;}

        // ----- Number of pad digits.
        [Browsable(true), DescriptionAttribute(
            "The number of digits in which to pad the barcode number, or 0 to ignore.")]
        public byte PadDigits
        {
            get
            {
                return StoredTextPadDigits;
            }
            set
            {
                // ----- Make sure the count is in the valid range.
                if ((value < 0) | (value > 20))
                    throw new ArgumentException($"Invalid pad digits value: {value}. " +
                        "Pad digits must range from 0 to 20.");
                StoredTextPadDigits = value;
            }
        }
    }

    public class BarcodeItemRect : BarcodeItemGeneric
    {
        // ----- Includes a basic rectangle element in a barcode label.

        // ----- Private store of attributes.
        private short StoredRectAngle;

        // ----- The ItemType constant helps when saving the record
        //       to the database.
        [Browsable(false)]
        public override string ItemType
        {
            get
            {
                return "R";
            }
        }

        public override string ToString()
        {
            return $"Rectangle: At ({this.Left},{this.Top})";
        }

        // ----- Upper-left corner of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the left-edge position of the rectangle.")]
        public decimal Left {get; set;}

        // ----- Upper-left corner of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the top-edge position of the rectangle.")]
        public decimal Top {get; set;}

        // ----- The width of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the width of the rectangle.")]
        public decimal Width {get; set;}

        // ----- The height of the text box.
        [Browsable(true), DefaultValueAttribute("0"),
            DescriptionAttribute("Sets the height of the rectangle.")]
        public decimal Height {get; set;}

        // ----- The line color.
        [Browsable(true),
            DescriptionAttribute("Sets the border color of the rectangle.")]
        public Color BorderColor {get; set;}

        // ----- The fill color.
        [Browsable(true),
            DescriptionAttribute("Sets the fill color of the rectangle.")]
        public Color FillColor {get; set;}

        // ----- Rotation angle of the rectangle.
        [Browsable(true),
            DescriptionAttribute("Sets the rotation angle, from 0 to 359 degrees.")]
        public short RotationAngle
        {
            get
            {
                return StoredRectAngle;
            }
            set
            {
                // ----- Make sure the angle is in the valid range.
                if ((value < 0) | (value > 359))
                    throw new ArgumentException($"Invalid rotation angle: {value}. " +
                        "Rotation angle must range from 0 to 359 degrees.");
                StoredRectAngle = value;
            }
        }
    }
}
