﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class BarcodePage : Library.BaseCodeForm
    {
        private const decimal MaxPageInInches = 22.0M;
        private const decimal TwipsPerInch = 1440.0M;

        private long ActiveSheetID;
        private bool StartingOver;
        private char CurrentMeasure;

        public BarcodePage()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            ActiveSheetID = -1L;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveSheetID;
            else
                return -1L;
        }

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManageBarcodeTemplates];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManageBarcodeTemplates];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManageBarcodeTemplates];
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the record.
            string sqlText;
            DataCommand sqlRun;

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete this barcode template?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question,
                    MessageBoxDefaultButton.Button2) != DialogResult.Yes)
                return false;

            // ----- Make sure this record is not in use.
            sqlText = "SELECT COUNT(*) FROM BarcodeLabel WHERE BarcodeSheet = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    MessageBox.Show("You cannot delete this template because it is " +
                        "in use by one or more labels templates.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("BarcodePage.DeleteRecord.Check", ex);
                return false;
            }

            // ----- Delete the record.
            sqlText = "DELETE FROM BarcodeSheet WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("BarcodePage.DeleteRecord.Delete", ex);
                return false;
            }
            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record.
            ActiveSheetID = recordID;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveSheetID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items.
            string sqlText;
            DataTable dbInfo;

            // ----- Clear any existing values.
            destList.Items.Clear();
            exceededMatches = false;

            // ----- Retrieve the values from the database.
            sqlText = "SELECT ID, FullName FROM BarcodeSheet ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("BarcodePage.FillListWithRecords", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                destList.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;

            // ----- Retrieve the value from the database.
            sqlText = "SELECT FullName FROM BarcodeSheet WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                return DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("BarcodePage.FormatRecordName", ex);
                return "Error";
            }
        }

        public override string GetEditDescription()
        {
            // ----- Return a description of this editor.
            return "Use these codes to identify the format of pages onto wihch " +
                "barcodes are printed. You can define the layout of the page and the " +
                "number and position of barcode labels.";
        }
        public override string GetEditTitle()
        {
            // ----- Return the title-bar text for this editor.
            return "Barcode Sheet Templates";
        }

        private void PrepareFormFields()
        {
            // ----- Prepare the form.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow dbRow;

            // ----- Fill in the list of units.
            if (RecordUnits.Items.Count == 0)
            {
                RecordUnits.Items.Add(new ListItemData("Inches", 'I'));
                RecordUnits.Items.Add(new ListItemData("Centimeters", 'C'));
                RecordUnits.Items.Add(new ListItemData("Points", 'P'));
                RecordUnits.Items.Add(new ListItemData("Twips", 'T'));
                RecordUnits.SelectedIndex = 0;
            }

            // ----- Default to inches.
            CurrentMeasure = 'I';
            RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(CurrentMeasure);

            // ----- Clear the existing data.
            StartingOver = true;
            RecordName.Text = "";
            BCPageWidth.Text = "";
            BCPageHeight.Text = "";
            BCLeftMargin.Text = "";
            BCRightMargin.Text = "";
            BCTopMargin.Text = "";
            BCBottomMargin.Text = "";
            BCInterColumn.Text = "";
            BCInterRow.Text = "";
            BCColumns.Text = "";
            BCRows.Text = "";
            PreviewArea.Invalidate();
            if (ActiveSheetID == -1L)
                return;

            // ----- Load in the record.
            sqlText = "SELECT * FROM BarcodeSheet WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveSheetID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("BarcodePage.PrepareFormFields", ex);
                return;
            }
            dbRow = dbInfo.Rows[0];
            RecordName.Text = DBGetText(dbRow["FullName"]);
            RecordUnits.SelectedIndex = RecordUnits.Items.IndexOf(DBGetText(dbRow["UnitType"])[0]);
            if (RecordUnits.SelectedIndex == -1)
                RecordUnits.SelectedIndex = 0;
            CurrentMeasure = (ListItemData)RecordUnits.SelectedItem;
            BCPageWidth.Text = DBGetDecimal(dbRow["PageWidth"]).ToString("0.####");
            BCPageHeight.Text = DBGetDecimal(dbRow["PageHeight"]).ToString("0.####");
            BCLeftMargin.Text = DBGetDecimal(dbRow["MarginLeft"]).ToString("0.####");
            BCRightMargin.Text = DBGetDecimal(dbRow["MarginRight"]).ToString("0.####");
            BCTopMargin.Text = DBGetDecimal(dbRow["MarginTop"]).ToString("0.####");
            BCBottomMargin.Text = DBGetDecimal(dbRow["MarginBottom"]).ToString("0.####");
            BCInterColumn.Text = DBGetDecimal(dbRow["IntraColumn"]).ToString("0.####");
            BCInterRow.Text = DBGetDecimal(dbRow["IntraRow"]).ToString("0.####");
            BCColumns.Text = DBGetInteger(dbRow["ColumnsCount"]).ToString();
            BCRows.Text = DBGetInteger(dbRow["RowsCount"]).ToString();
            dbInfo.Dispose();
            PreviewArea.Invalidate();
        }

        private void BarcodePage_Activated(object sender, EventArgs e)
        {
            // ----- Return the focus to the main field.
            if (StartingOver)
                RecordName.Focus();
            StartingOver = false;
        }

        private void RecordName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordName.SelectAll();
        }

        private void BCPageWidth_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            BCPageWidth.SelectAll();
        }

        private void BCPageHeight_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            BCPageHeight.SelectAll();
        }

        private void BCLeftMargin_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            BCLeftMargin.SelectAll();
        }

        private void BCRightMargin_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            BCRightMargin.SelectAll();
        }

        private void BCTopMargin_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            BCTopMargin.SelectAll();
        }

        private void BCBottomMargin_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            BCBottomMargin.SelectAll();
        }

        private void BCInterColumn_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            BCInterColumn.SelectAll();
        }

        private void BCInterRow_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            BCInterRow.SelectAll();
        }

        private void BCColumns_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            BCColumns.SelectAll();
        }

        private void BCRows_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            BCRows.SelectAll();
        }

        private bool ValidateForm()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            string sqlText;
            DataCommand sqlRun;

            // ----- Name is a required field.
            if (RecordName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply a template name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordName.Focus();
                return false;
            }

            // ----- Make sure the name is not already in use.
            sqlText = "SELECT COUNT(*) FROM BarcodeSheet WHERE UPPER(FullName) = @TestName";
            if (ActiveSheetID != -1L)
                sqlText += " AND ID <> @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestName", RecordName.Text.Trim().ToUpper());
            if (ActiveSheetID != -1L)
                sqlRun.AddLong("RecordID", ActiveSheetID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    // ----- Duplicate entry.
                    MessageBox.Show("A record already exists with the name '" +
                        RecordName.Text.Trim() + "'.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordName.Focus();
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("BarcodePage.ValidateForm", ex);
                return false;
            }

            // ----- Check the other fields.
            if (VerifyPageSize(false) == false)
                return false;
            if (VerifyLayout(false) == false)
                return false;

            // ----- Success.
            return true;
        }

        private bool VerifyLayout(bool silent)
        {
            // ----- Check the values supplied by the user to see if they are valid.
            //       This routine assumes that VerifyPageSize() has been called.
            if (CheckDecimal(BCLeftMargin.Text) == false)
            {
                if (silent == false)
                    MessageBox.Show("Left Margin is required.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            if (CheckDecimal(BCRightMargin.Text) == false)
            {
                if (silent == false)
                    MessageBox.Show("Right Margin is required.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            if (CheckDecimal(BCTopMargin.Text) == false)
            {
                if (silent == false)
                    MessageBox.Show("Top Margin is required.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            if (CheckDecimal(BCBottomMargin.Text) == false)
            {
                if (silent == false)
                    MessageBox.Show("Bottom Margin is required.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            if (CheckDecimal(BCInterColumn.Text) == false)
            {
                if (silent == false)
                    MessageBox.Show("Intra-Column Spacing is required.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            if (CheckDecimal(BCInterRow.Text) == false)
            {
                if (silent == false)
                    MessageBox.Show("Intra-Row Spacing is required.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            if (CheckInt(BCColumns.Text) == false)
            {
                if (silent == false)
                    MessageBox.Show("Column Count is required.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            if (CheckInt(BCRows.Text) == false)
            {
                if (silent == false)
                    MessageBox.Show("Row Count is required.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            // ----- Make sure there is space for the labels.
            if ((Convert.ToDecimal(BCPageWidth.Text) -
                ((Convert.ToDecimal(BCColumns.Text) - 1M) *
                Convert.ToDecimal(BCInterColumn.Text))) <= 0M)
            {
                if (silent == false)
                    MessageBox.Show("Insufficient horizontal space provided.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            if ((Convert.ToDecimal(BCPageHeight.Text) -
                ((Convert.ToDecimal(BCRows.Text) - 1M) *
                Convert.ToDecimal(BCInterRow.Text))) <= 0M)
            {
                if (silent == false)
                    MessageBox.Show("Insufficient vertical space provided.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            // ----- Success.
            return true;
        }

        private bool VerifyPageSize(bool silent)
        {
            // ----- Check the values supplied by the user to see if they are valid.
            decimal pageWidth;
            decimal pageHeight;
            char fromUnit;

            // ----- Check for valid width and height.
            if (decimal.TryParse(BCPageWidth.Text, out pageWidth) == false)
                pageWidth = 0M;
            if (pageWidth <= 0M)
            {
                if (silent == false)
                    MessageBox.Show("Page Width is required.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            if (decimal.TryParse(BCPageHeight.Text, out pageHeight) == false)
                pageHeight = 0M;
            if (pageHeight <= 0M)
            {
                if (silent == false)
                    MessageBox.Show("Page Height is required.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            // ----- The width and height are limited to a maximum size.
            fromUnit = (ListItemData)RecordUnits.SelectedItem;
            pageWidth = ConvertPageUnits(fromUnit, 'T', pageWidth);
            pageHeight = ConvertPageUnits(fromUnit, 'T', pageHeight);
            if (pageWidth > (TwipsPerInch * MaxPageInInches))
            {
                if (silent == false)
                    MessageBox.Show("The width of the page is limited to " +
                        ConvertPageUnits('T', fromUnit, MaxPageInInches) + " " +
                        RecordUnits.Text + ".", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            if (pageHeight > (TwipsPerInch * MaxPageInInches))
            {
                if (silent == false)
                    MessageBox.Show("The height of the page is limited to " +
                        ConvertPageUnits('T', fromUnit, MaxPageInInches) + " " +
                        RecordUnits.Text + ".", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            // ----- Success.
            return true;
        }

        private void PreviewArea_Paint(object sender, PaintEventArgs e)
        {
            // ----- Display a preview of the image.
            int pageWidth;
            int pageHeight;
            int trueWidth;
            int trueHeight;
            int pageLeft; // Start of the "white" area.
            int pageTop; // Start of the "white" area.
            decimal oneWidth;
            decimal oneHeight;
            int oneWidthTwips = 0;
            int oneHeightTwips= 0;
            int rowScan= 0;
            int colScan= 0;
            int leftOffset= 0;
            int topOffset= 0;
            decimal useRatio;
            char fromUnit;

            // ----- Draw in the background color.
            e.Graphics.Clear(SystemColors.Control);

            // ----- Make sure that a valid page size is supplied.
            if (VerifyPageSize(true) == false)
                return;

            // ----- Determine the page area.
            fromUnit = (ListItemData)RecordUnits.SelectedItem;
            trueWidth = (int)ConvertPageUnits(fromUnit, 'T', Convert.ToDecimal(BCPageWidth.Text));
            trueHeight = (int)ConvertPageUnits(fromUnit, 'T', Convert.ToDecimal(BCPageHeight.Text));
            if ((trueWidth == 0) | (trueHeight == 0))
                return;
            pageWidth = PreviewArea.Width - 4;
            pageHeight = PreviewArea.Height - 4;
            useRatio = pageWidth / (decimal)trueWidth;
            if ((int)(trueHeight * useRatio) > pageHeight)
            {
                // ----- Scale based on the height.
                useRatio = pageHeight / (decimal)trueHeight;
                pageWidth = (int)(trueWidth * useRatio);
            }
            else
            {
                // ----- Scale based on the width.
                pageHeight = (int)(trueHeight * useRatio);
            }
            pageLeft = (int)((PreviewArea.Width - 4 - pageWidth) / 2.0) + 1;
            pageTop = (int)((PreviewArea.Height - 4 - pageHeight) / 2.0) + 1;

            // ----- Draw the page area.
            e.Graphics.FillRectangle(SystemBrushes.ControlDark,
                pageLeft + 1, pageTop + 1, pageWidth + 2, pageHeight + 2);
            e.Graphics.FillRectangle(SystemBrushes.ControlDark,
                pageLeft + 2, pageTop + 2, pageWidth + 2, pageHeight + 2);
            e.Graphics.FillRectangle(Brushes.Black,
                pageLeft - 1, pageTop - 1, pageWidth + 2, pageHeight + 2);
            e.Graphics.FillRectangle(Brushes.White,
                pageLeft, pageTop, pageWidth, pageHeight);

            // ----- Check the rest of the fields.
            if (VerifyLayout(true) == false)
                return;

            // ----- Transform a page of twips into a display page.
            e.Graphics.TranslateTransform(pageLeft, pageTop);
            e.Graphics.ScaleTransform((float)useRatio, (float)useRatio);

            // ----- Figure out the size of one label.
            oneWidth = (Convert.ToDecimal(BCPageWidth.Text) -
                Convert.ToDecimal(BCLeftMargin.Text) - Convert.ToDecimal(BCRightMargin.Text) -
                ((Convert.ToDecimal(BCColumns.Text) - 1.0M) * Convert.ToDecimal(BCInterColumn.Text))) /
                Convert.ToDecimal(BCColumns.Text);
            oneHeight = (Convert.ToDecimal(BCPageHeight.Text) - Convert.ToDecimal(BCTopMargin.Text) -
                Convert.ToDecimal(BCBottomMargin.Text) - ((Convert.ToDecimal(BCRows.Text) - 1.0M) *
                Convert.ToDecimal(BCInterRow.Text))) / Convert.ToDecimal(BCRows.Text);
            oneWidthTwips = (int)ConvertPageUnits(fromUnit, 'T', oneWidth);
            oneHeightTwips = (int)ConvertPageUnits(fromUnit, 'T', oneHeight);

            // ----- Process each row and column.
            for (rowScan = 1; rowScan <= Convert.ToInt32(BCRows.Text); rowScan++)
            {
                for (colScan = 1; colScan <= Convert.ToInt32(BCColumns.Text); colScan++)
                {
                    leftOffset = (int)ConvertPageUnits(fromUnit, 'T',
                        Convert.ToDecimal(BCLeftMargin.Text) + ((colScan - 1) *
                        (Convert.ToDecimal(BCInterColumn.Text) + oneWidth)));
                    topOffset = (int)ConvertPageUnits(fromUnit, 'T',
                        Convert.ToDecimal(BCTopMargin.Text) + ((rowScan - 1) *
                        (Convert.ToDecimal(BCInterRow.Text) + oneHeight)));
                    e.Graphics.DrawRectangle(Pens.Cyan, leftOffset, topOffset,
                        oneWidthTwips, oneHeightTwips);
                }
            }

            // ----- Finished.
            e.Graphics.ResetTransform();
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            string sqlText;
            DataCommand sqlRun;
            long newID;

            // ----- Confirm the data.
            if (ValidateForm() == false)
                return;
            this.Cursor = Cursors.WaitCursor;

            // ----- Save the data.
            if (ActiveSheetID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO BarcodeSheet (FullName, UnitType, PageWidth, " +
                    "PageHeight, MarginLeft, MarginRight, MarginTop, MarginBottom, " +
                    "IntraColumn, IntraRow, ColumnsCount, RowsCount) OUTPUT INSERTED.ID " +
                    "VALUES (@NewName, @NewUnit, @NewWidth, @NewHeight, @NewLeft, @NewRight, " +
                    "@NewTop, @NewBottom, @NewColumn, @NewRow, @NewColCount, @NewRowCount)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordName.Text.Trim());
                sqlRun.AddText("NewUnit",
                    ((char)(ListItemData)RecordUnits.SelectedItem).ToString());
                sqlRun.AddDecimal("NewWidth", BCPageWidth.Text);
                sqlRun.AddDecimal("NewHeight", BCPageHeight.Text);
                sqlRun.AddDecimal("NewLeft", BCLeftMargin.Text);
                sqlRun.AddDecimal("NewRight", BCRightMargin.Text);
                sqlRun.AddDecimal("NewTop", BCTopMargin.Text);
                sqlRun.AddDecimal("NewBottom", BCBottomMargin.Text);
                sqlRun.AddDecimal("NewColumn", BCInterColumn.Text);
                sqlRun.AddDecimal("NewRow", BCInterRow.Text);
                sqlRun.AddInteger("NewColCount", BCColumns.Text);
                sqlRun.AddInteger("NewRowCount", BCRows.Text);
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    GeneralError("BarcodePage.ActOK_Click.Add", ex);
                    return;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
                ActiveSheetID = newID;
            }
            else
            {
                // ----- Update the existing entry.
                sqlText = "UPDATE BarcodeSheet SET FullName = @NewName, " +
                    "UnitType = @NewUnit, PageWidth = @NewWidth, " +
                    "PageHeight = @NewHeight, MarginLeft = @NewLeft, " +
                    "MarginRight = @NewRight, MarginTop = @NewTop, " +
                    "MarginBottom = @NewBottom, IntraColumn = @NewColumn, " +
                    "IntraRow = @NewRow, ColumnsCount = @NewColCount, " +
                    "RowsCount = @NewRowCount WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordName.Text.Trim());
                sqlRun.AddText("NewUnit",
                    ((char)(ListItemData)RecordUnits.SelectedItem).ToString());
                sqlRun.AddDecimal("NewWidth", BCPageWidth.Text);
                sqlRun.AddDecimal("NewHeight", BCPageHeight.Text);
                sqlRun.AddDecimal("NewLeft", BCLeftMargin.Text);
                sqlRun.AddDecimal("NewRight", BCRightMargin.Text);
                sqlRun.AddDecimal("NewTop", BCTopMargin.Text);
                sqlRun.AddDecimal("NewBottom", BCBottomMargin.Text);
                sqlRun.AddDecimal("NewColumn", BCInterColumn.Text);
                sqlRun.AddDecimal("NewRow", BCInterRow.Text);
                sqlRun.AddInteger("NewColCount", BCColumns.Text);
                sqlRun.AddInteger("NewRowCount", BCRows.Text);
                sqlRun.AddLong("RecordID", ActiveSheetID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("BarcodePage.ActOK_Click.Update", ex);
                    return;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }

            // ----- Continue with AddRecord() or EditRecord()
            this.DialogResult = DialogResult.OK;
        }

        private void BCFields_TextChanged(object sender, EventArgs e)
        {
            // ----- Refresh the preview.
            PreviewArea.Invalidate();
        }

        private void RecordUnits_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Refresh the preview.
            char newMeasure;
            decimal holdValue;

            // ----- See if the measure changed.
            if (RecordUnits.SelectedIndex == -1)
                return;
            newMeasure = (ListItemData)RecordUnits.SelectedItem;
            if (CurrentMeasure != newMeasure)
            {
                // ----- Convert the measure.
                if (decimal.TryParse(BCPageWidth.Text, out holdValue) == true)
                    BCPageWidth.Text = ConvertPageUnits(CurrentMeasure,
                        newMeasure, holdValue).ToString();
                if (decimal.TryParse(BCPageHeight.Text, out holdValue) == true)
                    BCPageHeight.Text = ConvertPageUnits(CurrentMeasure,
                        newMeasure, holdValue).ToString();
                if (decimal.TryParse(BCLeftMargin.Text, out holdValue) == true)
                    BCLeftMargin.Text = ConvertPageUnits(CurrentMeasure,
                        newMeasure, holdValue).ToString();
                if (decimal.TryParse(BCRightMargin.Text, out holdValue) == true)
                    BCRightMargin.Text = ConvertPageUnits(CurrentMeasure,
                        newMeasure, holdValue).ToString();
                if (decimal.TryParse(BCTopMargin.Text, out holdValue) == true)
                    BCTopMargin.Text = ConvertPageUnits(CurrentMeasure,
                        newMeasure, holdValue).ToString();
                if (decimal.TryParse(BCBottomMargin.Text, out holdValue) == true)
                    BCBottomMargin.Text = ConvertPageUnits(CurrentMeasure,
                        newMeasure, holdValue).ToString();
                if (decimal.TryParse(BCInterColumn.Text, out holdValue) == true)
                    BCInterColumn.Text = ConvertPageUnits(CurrentMeasure,
                        newMeasure, holdValue).ToString();
                if (decimal.TryParse(BCInterRow.Text, out holdValue) == true)
                    BCInterRow.Text = ConvertPageUnits(CurrentMeasure,
                        newMeasure, holdValue).ToString();
                CurrentMeasure = newMeasure;
            }
            PreviewArea.Invalidate();
        }
    }
}
