﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class LocateDatabase : Form
    {
        private const long AuthenticationTypeWindows = 1L;
        private const long AuthenticationTypeSQL = 2L;

        public LocateDatabase()
        {
            InitializeComponent();
        }

        public string PromptUser()
        {
            // ----- Prompt the user for database connection details. Return a
            //       valid ADO.NET connection string for SQL Server, or a blank
            //       string if the user clicks Cancel.
            string newConnection;

            // ----- Prompt the user.
            this.ShowDialog();

            // ----- Build the new connection string.
            if (this.DialogResult == DialogResult.OK)
            {
                newConnection = "Data Source=" + RecordServer.Text.Trim() +
                    ";Initial Catalog=" + RecordDatabase.Text.Trim();
                if ((long)(ListItemData)RecordAuthentication.SelectedItem == AuthenticationTypeWindows)
                {
                    // ----- Use Windows security.
                    newConnection += ";Integrated Security=true";
                }
                else
                {
                    // ----- Use SQL Server security.
                    newConnection += ";User ID=" + RecordUser.Text.Trim() +
                        ";Password=" + RecordPassword.Text.Trim();
                }
                return newConnection;
            }
            else
                return "";
        }

        private bool ValidateFormData()
        {
            // ----- Check the supplied data, and return True if it is all valid.
            //       Server name is required.
            if (RecordServer.Text.Trim().Length == 0)
            {
                MessageBox.Show("The Server or Host name is required.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordServer.Focus();
                return false;
            }

            // ----- Database name is required.
            if (RecordDatabase.Text.Trim().Length == 0)
            {
                MessageBox.Show("The Database Name is required.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordDatabase.Focus();
                return false;
            }

            // ----- For SQL Server authentication, the User ID is required.
            if ((long)(ListItemData)RecordAuthentication.SelectedItem == AuthenticationTypeSQL)
            {
                if (RecordUser.Text.Trim().Length == 0)
                {
                    MessageBox.Show("The User Name is required for SQL Server authentication.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordUser.Focus();
                    return false;
                }
            }

            // ----- Success.
            return true;
        }

        private void LocateDatabase_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            string connectionString;
            string oneKey;
            string oneValue;

            // ----- Add the available authentication schemes.
            RecordAuthentication.Items.Add(new ListItemData(
                "Microsoft Windows", AuthenticationTypeWindows));
            RecordAuthentication.SelectedIndex = 0;
            RecordAuthentication.Items.Add(new ListItemData(
                "SQL Server", AuthenticationTypeSQL));

            // ----- Load in the existing data.
            connectionString = Properties.Settings.Default.DBConnection + "";
            for (int counter = 1; counter <= CountSubStr(connectionString, ";") + 1; counter++)
            {
                // ----- Each comma-delimited part has the format "key=value".
                oneKey = connectionString.GetSubStr(";", counter);
                oneValue = oneKey.GetSubStr("=", 2).Trim();
                oneKey = oneKey.GetSubStr("=", 1).Trim().ToUpper().Replace(" ", "");

                // ----- Process each part.
                switch (oneKey)
                {
                    case "DATASOURCE":
                        // ----- Show the server host.
                        RecordServer.Text = oneValue;
                        break;
                    case "INITIALCATALOG":
                        // ----- Show the default database name.
                        RecordDatabase.Text = oneValue;
                        break;
                    case "INTEGRATEDSECURITY":
                        // ----- Only check for "true". False is assumed.
                        if (oneValue.ToUpper() == "TRUE")
                            RecordAuthentication.SelectedIndex =
                                RecordAuthentication.Items.IndexOf(AuthenticationTypeWindows);
                        break;
                    case "USERID":
                        // ----- A user ID forces SQL authentication.
                        RecordAuthentication.SelectedIndex =
                            RecordAuthentication.Items.IndexOf(AuthenticationTypeSQL);
                        RecordUser.Text = oneValue;
                        break;
                    case "PASSWORD":
                        // ----- A password forces SQL authentication.
                        RecordAuthentication.SelectedIndex =
                            RecordAuthentication.Items.IndexOf(AuthenticationTypeSQL);
                        RecordPassword.Text = oneValue;
                        break;
                }
            }
        }

        private void RecordDatabase_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordDatabase.SelectAll();
        }

        private void RecordPassword_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordPassword.SelectAll();
        }

        private void RecordServer_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordServer.SelectAll();
        }

        private void RecordUser_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordUser.SelectAll();
        }

        private void RecordAuthentication_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Enable dependent fields as needed.
            if ((long)(ListItemData)RecordAuthentication.SelectedItem == AuthenticationTypeWindows)
            {
                // ----- Windows authentication requires no user/password.
                LabelUser.Enabled = false;
                RecordUser.Enabled = false;
                LabelPassword.Enabled = false;
                RecordPassword.Enabled = false;
            }
            else
            {
                // ----- Requires specific SQL Server user and password.
                LabelUser.Enabled = true;
                RecordUser.Enabled = true;
                LabelPassword.Enabled = true;
                RecordPassword.Enabled = true;
            }
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }
    }
}
