﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class NamedItem : Library.BaseCodeForm
    {
        private long ActiveID;
        private bool StartingOver;
        private long PublisherID;
        private long SeriesID;
        private bool SessionSaved;

        public NamedItem()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            ActiveID = -1L;
            PrepareFormFields();
            this.ShowDialog();
            if ((this.DialogResult == DialogResult.OK) | (SessionSaved == true))
                return ActiveID;
            else
                return -1L;
        }

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManageItems];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManageItems];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManageItems];
        }

        private bool DeleteOneKeyword(long keywordID)
        {
            // ----- Remove a keyword from this record. Return True on success.
            string sqlText;
            DataCommand sqlRun;

            // ----- Delete the record.
            sqlText = "DELETE FROM ItemKeyword WHERE ItemID = @RecordID" +
                " AND KeywordID = @KeyID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            sqlRun.AddLong("KeyID", keywordID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.DeleteOneKeyword.Delete", ex);
                return false;
            }

            // ----- If no books are using this keyword, remove it.
            sqlText = "SELECT COUNT(*) FROM ItemKeyword WHERE KeywordID = @KeyID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("KeyID", keywordID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) == 0)
                {
                    sqlText = "DELETE FROM Keyword WHERE ID = @KeyID";
                    sqlRun = new DataCommand(sqlText);
                    sqlRun.AddLong("KeyID", keywordID);
                    ExecuteSQL(sqlRun);
                }
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.DeleteOneKeyword.Books", ex);
                return false;
            }
            return true;
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the records.
            string sqlText;
            DataCommand sqlRun;
            LibraryTransaction transInfo = null;

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete the item?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                    DialogResult.Yes)
                return false;

            // ----- Make sure this record is not in use.
            sqlText = "SELECT COUNT(*) FROM ItemCopy WHERE ItemID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    MessageBox.Show("You cannot delete this record because it has copies.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.DeleteRecord.Check", ex);
                return false;
            }

            // ----- Delete each of the keywords.
            for (int counter = 0; counter < KeywordList.Items.Count; counter++)
                DeleteOneKeyword((long)(ListItemData)KeywordList.Items[counter]);

            // ----- Delete the records.
            try
            {
                transInfo = TransactionBegin();

                sqlText = "DELETE FROM ItemAuthor WHERE ItemID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", recordID);
                ExecuteSQL(sqlRun, transInfo);

                sqlText = "DELETE FROM NamedItem WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", recordID);
                ExecuteSQL(sqlRun, transInfo);

                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                if (transInfo != null)
                    TransactionRollback(transInfo);
                GeneralError("NamedItem.DeleteRecord.Delete", ex);
                return false;
            }

            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record.
            ActiveID = recordID;
            PrepareFormFields();
            this.ShowDialog();
            if ((this.DialogResult == DialogResult.OK) | (SessionSaved == true))
                return ActiveID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items. Prompt the
            //       user first.
            destList.Items.Clear();
            SearchForRecord(destList, ref exceededMatches);
        }

        private string FormatItemName(DataRow dbRow)
        {
            // ----- Given an item record, return the formatted name.
            string nameText;

            // ----- Format the name.
            if (dbRow == null)
                return "";
            nameText = DBGetText(dbRow["Title"]);
            if (Convert.IsDBNull(dbRow["Subtitle"]) == false)
                nameText += ": " + DBGetText(dbRow["Subtitle"]);
            if (Convert.IsDBNull(dbRow["Edition"]) == false)
                nameText += ", " + DBGetText(dbRow["Edition"]);
            nameText += " [" + DBGetText(dbRow["MediaName"]) + "]";

            // ----- Finished.
            return nameText;
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            string result;

            // ----- Initialize.
            result = "";

            // ----- Retrieve the value from the database.
            sqlText = "SELECT NI.ID, NI.Title, NI.Subtitle, NI.Edition, CMT.FullName " +
                "AS MediaName FROM NamedItem AS NI INNER JOIN CodeMediaType AS CMT " +
                "ON NI.MediaType = CMT.ID WHERE NI.ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.FormatRecordName", ex);
                return "Error";
            }
            if (dbInfo.Rows.Count > 0)
                result = FormatItemName(dbInfo.Rows[0]);
            dbInfo.Dispose();
            dbInfo = null;
            return result;
        }

        public override string GetEditDescription()
        {
            // ----- Return the list form display title.
            return "Use this list to identify the items found in the library. " +
                "Each item may include more than one copy.";
        }

        public override string GetEditTitle()
        {
            // ----- Return the list form display title.
            return "Named Items";
        }

        private void PrepareFormFields()
        {
            // ----- Fill in the data.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            ListViewItem newItem;
            string holdText;

            // ----- Load in the list of security groups.
            RecordMediaType.Items.Clear();
            RecordMediaType.Items.Add(new ListItemData("<Not Selected>", -1L));
            sqlText = "SELECT * FROM CodeMediaType ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.PrepareFormFields.Groups", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
                RecordMediaType.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Clear the existing data.
            StartingOver = true;
            RecordID.Text = "Not Available";
            RecordTitle.Text = "";
            RecordSubtitle.Text = "";
            RecordDescription.Text = "";
            RecordCopyright.Text = "";
            RecordEdition.Text = "";
            RecordPublisher.Text = "Not Available";
            PublisherID = -1L;
            RecordDewey.Text = "";
            RecordLC.Text = "";
            RecordLCCN.Text = "";
            RecordISxN.Text = "";
            RecordMediaType.SelectedIndex = RecordMediaType.Items.IndexOf(-1L);
            RecordOutOfPrint.Checked = false;
            RecordSeries.Text = "No Series";
            SeriesID = -1L;
            AuthorsList.Items.Clear();
            ActAuthorProperties.Enabled = false;
            ActAuthorDelete.Enabled = false;
            ActAuthorMoveUp.Enabled = false;
            ActAuthorMoveDown.Enabled = false;
            SubjectList.Items.Clear();
            ActSubjectDelete.Enabled = false;
            ActSubjectProperties.Enabled = false;
            KeywordList.Items.Clear();
            ActKeywordDelete.Enabled = false;
            ActKeywordProperties.Enabled = false;
            CopyList.Items.Clear();
            ActCopyEdit.Enabled = false;
            ActCopyDelete.Enabled = false;
            ActCancel.Text = "Cancel";
            TabNamedItem.SelectedTab = TabGeneral;
            SessionSaved = false;
            if (ActiveID == -1L)
                return;

            // ----- Load in the values stored in the database.
            sqlText = "SELECT NI.*, PB.FullName AS PubName, CS.FullName As SeriesName " +
                "FROM NamedItem AS NI LEFT JOIN Publisher AS PB ON NI.Publisher = PB.ID " +
                "LEFT JOIN CodeSeries AS CS ON NI.Series = CS.ID " +
                "WHERE NI.ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.PrepareFormFields.Retrieve", ex);
                return;
            }
            if (dbInfo.Rows.Count > 0)
            {
                oneRow = dbInfo.Rows[0];
                RecordID.Text = DBGetLong(oneRow["ID"]).ToString();
                RecordTitle.Text = DBGetText(oneRow["Title"]);
                RecordSubtitle.Text = DBGetText(oneRow["Subtitle"]);
                RecordDescription.Text = DBGetText(oneRow["Description"]);
                RecordCopyright.Text = DBGetText(oneRow["Copyright"]);
                RecordEdition.Text = DBGetText(oneRow["Edition"]);
                if (Convert.IsDBNull(oneRow["PubName"]) == false)
                    RecordPublisher.Text = DBGetText(oneRow["PubName"]);
                if (Convert.IsDBNull(oneRow["Publisher"]) == false)
                    PublisherID = DBGetLong(oneRow["Publisher"]);
                RecordDewey.Text = DBGetText(oneRow["Dewey"]);
                RecordLC.Text = DBGetText(oneRow["LC"]);
                RecordLCCN.Text = DBGetText(oneRow["LCCN"]);
                RecordISxN.Text = DBGetText(oneRow["ISxN"]);
                RecordMediaType.SelectedIndex = RecordMediaType.Items.IndexOf(DBGetLong(oneRow["MediaType"]));
                RecordOutOfPrint.Checked = DBGetBool(oneRow["OutOfPrint"]);
                if (Convert.IsDBNull(oneRow["SeriesName"]) == false)
                    RecordSeries.Text = DBGetText(oneRow["SeriesName"]);
                if (Convert.IsDBNull(oneRow["Series"]) == false)
                    SeriesID = DBGetLong(oneRow["Series"]);
            }
            dbInfo.Dispose();

            // ----- Load in the authors for this item.
            sqlText = "SELECT AU.*, CT.FullName AS TypeName, CT.ID AS TypeID " +
                "FROM ItemAuthor AS IA INNER JOIN Author AS AU ON IA.AuthorID = AU.ID " +
                "INNER JOIN CodeAuthorType AS CT ON IA.AuthorType = CT.ID " +
                "WHERE IA.ItemID = @RecordID ORDER BY IA.Sequence";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.PrepareFormFields.Authors", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                newItem = AuthorsList.Items.Add(dbRow.FormatAuthorName());
                newItem.SubItems.Add(DBGetText(dbRow["TypeName"]));
                newItem.Tag = DBGetLong(dbRow["ID"]).ToString() + "," +
                    DBGetLong(dbRow["TypeID"]).ToString();
            }
            dbInfo.Dispose();

            // ----- Load in the subjects for this item.
            sqlText = "SELECT SB.* FROM ItemSubject AS IJ INNER JOIN Subject AS SB " +
                "ON IJ.SubjectID = SB.ID WHERE IJ.ItemID = @RecordID " +
                "ORDER BY SB.FullName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.PrepareFormFields.Subjects", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
                SubjectList.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            dbInfo.Dispose();

            // ----- Load in the keywords for this item.
            sqlText = "SELECT KW.* FROM ItemKeyword AS IK INNER JOIN Keyword AS KW " +
                "ON IK.KeywordID = KW.ID WHERE IK.ItemID = @RecordID " +
                "ORDER BY KW.FullName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.PrepareFormFields.Keywords", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
                KeywordList.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            dbInfo.Dispose();

            // ----- Load in the copies for this item.
            sqlText = "SELECT ID, CopyNumber, Barcode FROM ItemCopy WHERE ItemID = @RecordID " +
                "ORDER BY CopyNumber";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.PrepareFormFields.Copies", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                newItem = CopyList.Items.Add(DBGetInteger(dbRow["CopyNumber"]).ToString());
                newItem.SubItems.Add(""); // Add disposition later
                holdText = DBGetText(dbRow["Barcode"]);
                if (holdText.Length == 0)
                    holdText = "None";
                newItem.SubItems.Add(holdText);
                newItem.Tag = DBGetLong(dbRow["ID"]).ToString();
            }
            dbInfo.Dispose();

            // ----- Add the disposition for each copy. We can't do it in the
            //       earlier loop since that would cause two OleDB readers
            //       to be opened at once, and that doesn't work.
            for (int counter = 0; counter < CopyList.Items.Count; counter++)
            {
                newItem = CopyList.Items[counter];
                newItem.SubItems[1].Text = GetCopyDisposition(
                    Convert.ToInt64(newItem.Tag));
            }

            // ----- Finished.
            dbInfo = null;
        }

        private bool SaveFormData()
        {
            // ----- Save the changes to the database.
            string sqlText;
            DataCommand sqlRun;
            long newID;

            this.Cursor = Cursors.WaitCursor;

            // ----- Save the data.
            if (ActiveID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO NamedItem (Title, Subtitle, Description, " +
                    "Copyright, Edition, Publisher, Dewey, LC, LCCN, ISxN, MediaType, " +
                    "OutOfPrint, Series) OUTPUT INSERTED.ID VALUES (@NewTitle, @NewSubtitle, " +
                    "@NewDesc, @NewCopyright, @NewEdition, @NewPub, @NewDewey, @NewLC, " +
                    "@NewLCCN, @NewISXN, @NewMedia, @NewPrint, @NewSeries)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewTitle", RecordTitle.Text.Trim());
                sqlRun.AddText("NewSubtitle", RecordSubtitle.Text.Trim());
                sqlRun.AddText("NewDesc", RecordDescription.Text.Trim());
                sqlRun.AddInteger("NewCopyright", RecordCopyright.Text.Trim());
                sqlRun.AddText("NewEdition", RecordEdition.Text.Trim());
                sqlRun.AddID("NewPub", PublisherID);
                sqlRun.AddText("NewDewey", RecordDewey.Text.Trim());
                sqlRun.AddText("NewLC", RecordLC.Text.Trim());
                sqlRun.AddText("NewLCCN", RecordLCCN.Text.Trim());
                sqlRun.AddText("NewISXN", RecordISxN.Text.Trim());
                sqlRun.AddLong("NewMedia", DBComboParam(RecordMediaType));
                sqlRun.AddBool("NewPrint", RecordOutOfPrint.Checked);
                sqlRun.AddID("NewSeries", SeriesID);
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    GeneralError("NamedItem.SaveFormData.Add", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
                ActiveID = newID;
            }
            else
            {
                // ----- Update the existing entry.
                sqlText = "UPDATE NamedItem SET Title = @NewTitle, " +
                    "Subtitle = @NewSubtitle, Description = @NewDesc, " +
                    "Copyright = @NewCopyright, Edition = @NewEdition, " +
                    "Publisher = @NewPub, Dewey = @NewDewey, " +
                    "LC = @NewLC, LCCN = @NewLCCN, ISxN = @NewISXN, " +
                    "MediaType = @NewMedia, OutOfPrint = @NewPrint, " +
                    "Series = @NewSeries WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewTitle", RecordTitle.Text.Trim());
                sqlRun.AddText("NewSubtitle", RecordSubtitle.Text.Trim());
                sqlRun.AddText("NewDesc", RecordDescription.Text.Trim());
                sqlRun.AddInteger("NewCopyright", RecordCopyright.Text.Trim());
                sqlRun.AddText("NewEdition", RecordEdition.Text.Trim());
                sqlRun.AddID("NewPub", PublisherID);
                sqlRun.AddText("NewDewey", RecordDewey.Text.Trim());
                sqlRun.AddText("NewLC", RecordLC.Text.Trim());
                sqlRun.AddText("NewLCCN", RecordLCCN.Text.Trim());
                sqlRun.AddText("NewISXN", RecordISxN.Text.Trim());
                sqlRun.AddLong("NewMedia", DBComboParam(RecordMediaType));
                sqlRun.AddBool("NewPrint", RecordOutOfPrint.Checked);
                sqlRun.AddID("NewSeries", SeriesID);
                sqlRun.AddLong("RecordID", ActiveID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("NamedItem.SaveFormData.Update", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }

            // ----- Success.
            this.Cursor = Cursors.Default;
            ActCancel.Text = "Close";
            SessionSaved = true;
            return true;
        }

        public override void SearchForRecord(ListBox destList, ref bool exceededMatches)
        {
            // ----- Search for items using user-specified criteria.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            string userLimit;
            int resultCount;

            // ----- Prompt the user for the limited item title.
            exceededMatches = false;
            userLimit = (new ItemLimit()).PromptUser();
            if (userLimit.Length == 0)
                return;
            if (userLimit.Contains("*") == false)
                userLimit += "*";

            // ----- Clear any existing values.
            destList.Items.Clear();
            resultCount = 0;

            // ----- Retrieve the values from the database.
            sqlText = "SELECT NI.ID, NI.Title, NI.Subtitle, NI.Edition, CMT.FullName " +
                "AS MediaName FROM NamedItem AS NI INNER JOIN CodeMediaType AS CMT " +
                "ON NI.MediaType = CMT.ID WHERE NI.Title LIKE @TestTitle " +
                "ORDER BY NI.Title, NI.Subtitle, NI.Edition";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestTitle", userLimit.Trim().Replace("*", "%"));
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.SearchForRecord", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                resultCount += 1;
                if (resultCount > SearchMatchLimit)
                {
                    exceededMatches = true;
                    break;
                }
                destList.Items.Add(new ListItemData(
                    FormatItemName(dbRow), DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override bool UsesSearch()
        {
            // ----- NamedItem records are not searchable.
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            //       Title is a required field.
            if (RecordTitle.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the Title.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                TabNamedItem.SelectedTab = TabGeneral;
                RecordTitle.Focus();
                return false;
            }

            // ----- Media Type is a required field.
            if ((long)(ListItemData)RecordMediaType.SelectedItem == -1L)
            {
                MessageBox.Show("Please supply the Media Type.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                TabNamedItem.SelectedTab = TabClassification;
                RecordMediaType.Focus();
                return false;
            }

            // ----- Copyright Year, if supplied, must be numeric.
            if (RecordCopyright.Text.Trim().Length != 0)
            {
                if (DigitsOnly(RecordCopyright.Text) != RecordCopyright.Text.Trim())
                {
                    MessageBox.Show("Copyright Year, if supplied, must be a valid integer.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    TabNamedItem.SelectedTab = TabGeneral;
                    RecordCopyright.Focus();
                    return false;
                }
            }

            // ----- Success.
            return true;
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void NamedItem_Activated(object sender, EventArgs e)
        {
            // ----- Return the focus to the main field.
            if (StartingOver)
            {
                TabNamedItem.SelectedTab = TabGeneral;
                RecordTitle.Focus();
            }
            StartingOver = false;
        }

        private void NamedItem_Load(object sender, EventArgs e)
        {
            // ----- Adjust the author list columns.
            ColAuthorName.Width = LabelColNameType.Left - LabelColAuthor.Left;
            ColAuthorNameType.Width = (AuthorsList.Left + AuthorsList.Width) - LabelColNameType.Left - 12;

            // ----- Adjust the copy list columns.
            ColCopyNumber.Width = LabelColCopyStatus.Left - LabelColCopy.Left;
            ColCopyStatus.Width = LabelColCopyBarcode.Left - LabelColCopyStatus.Left;
            ColCopyBarcode.Width = (CopyList.Left + CopyList.Width) - LabelColCopyBarcode.Left - 12;
        }

        private void RecordCopyright_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordCopyright.SelectAll();
        }

        private void RecordDewey_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordDewey.SelectAll();
        }

        private void RecordEdition_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordEdition.SelectAll();
        }

        private void RecordISxN_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordISxN.SelectAll();
        }

        private void RecordLC_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordLC.SelectAll();
        }

        private void RecordLCCN_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordLCCN.SelectAll();
        }

        private void RecordSubtitle_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordSubtitle.SelectAll();
        }

        private void RecordTitle_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordTitle.SelectAll();
        }

        private void RecordCopyright_KeyPress(object sender, KeyPressEventArgs e)
        {
            // ----- Only allow digits and backspaces.
            if ((e.KeyChar.ToString() == "\b") | (Char.IsDigit(e.KeyChar)))
                return;
            e.Handled = true;
        }

        private void ActSelectPublisher_Click(object sender, EventArgs e)
        {
            // ----- Get a new publisher.
            long newPublisher;
            string displayText;

            // ----- Prompt the user.
            newPublisher = (new PublisherAddLocate()).PromptUser();
            if (newPublisher == -1L)
                return;

            // ----- Check to clear the publisher.
            if (newPublisher == -2L)
            {
                RecordPublisher.Text = "Not Available";
                PublisherID = -1L;
                return;
            }

            // ----- Save and display the publisher.
            PublisherID = newPublisher;
            displayText = (new Publisher()).FormatRecordName(PublisherID);
            RecordPublisher.Text = displayText;
        }

        private void ActSelectSeries_Click(object sender, EventArgs e)
        {
            // ----- Get a new series.
            long newSeries;
            string displayText;

            // ----- Prompt the user.
            newSeries = (new SeriesAddLocate()).PromptUser();
            if (newSeries == -1L)
                return;

            // ----- Check to clear the series.
            if (newSeries == -2L)
            {
                RecordSeries.Text = "No Series";
                SeriesID = -1L;
                return;
            }

            // ----- Save and display the series.
            SeriesID = newSeries;
            displayText = (new CodeSeries()).FormatRecordName(SeriesID);
            RecordSeries.Text = displayText;
        }

        private void ActKeywordDelete_Click(object sender, EventArgs e)
        {
            // ----- Delete the selected keyword.
            if (KeywordList.SelectedIndex == -1)
                return;

            // ----- Confirm with the user.
            if (MessageBox.Show("Really delete the keyword '" + KeywordList.Text +
                    "' from this item?", ProgramTitle, MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question) != DialogResult.Yes)
                return;

            // ----- Proceed with the removal.
            if (DeleteOneKeyword((long)(ListItemData)KeywordList.SelectedItem) == false)
                return;

            // ----- Remove this item from the display.
            KeywordList.Items.Remove(KeywordList.SelectedItem);
            KeywordList.SelectedIndex = -1;
            ActKeywordDelete.Enabled = false;
            ActKeywordProperties.Enabled = false;
            ActCancel.Text = "Close";
            SessionSaved = true;
        }

        private void ActKeywordAdd_Click(object sender, EventArgs e)
        {
            // ----- Let the user add a new keyword.
            long keywordID;
            string sqlText;
            DataCommand sqlRun;

            // ----- The record must be saved first.
            if (ActiveID == -1L)
            {
                // ----- Confirm with the user.
                if (MessageBox.Show("The item must be saved to the database before keywords " +
                        "can be added. Would you like to save the record now?", ProgramTitle,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
                    return;

                // ----- Verify and save the data.
                if (ValidateFormData() == false)
                    return;
                if (SaveFormData() == false)
                    return;
            }

            // ----- Prompt the user to add a keyword.
            keywordID = (new KeywordAdd()).AddRecord();
            if (keywordID == -1L)
                return;

            // ----- See if the record already exists.
            if (KeywordList.Items.IndexOf(keywordID) != -1)
                return;

            // ----- Add the keyword to the database record.
            sqlText = "INSERT INTO ItemKeyword (ItemID, KeywordID) " +
                "VALUES (@RecordID, @KeyID)";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            sqlRun.AddLong("KeyID", keywordID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.ActKeywordAdd_Click.Add", ex);
                return;
            }
            ActCancel.Text = "Close";
            SessionSaved = true;

            // ----- Add this item to the display.
            sqlText = "SELECT FullName FROM Keyword WHERE ID = @KeyID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("KeyID", keywordID);
            try
            {
                KeywordList.Items.Add(new ListItemData(
                    DBGetText(ExecuteSQLReturn(sqlRun)), keywordID));
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.ActKeywordAdd_Click.Name", ex);
                return;
            }
        }

        private void KeywordList_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Enable buttons as needed.
            ActKeywordDelete.Enabled = (KeywordList.SelectedIndex != -1);
            ActKeywordProperties.Enabled = ActKeywordDelete.Enabled;
        }

        private void KeywordList_Click(object sender, EventArgs e)
        {
            // ----- Enable buttons as needed.
            ActKeywordDelete.Enabled = (KeywordList.SelectedIndex != -1);
            ActKeywordProperties.Enabled = ActKeywordDelete.Enabled;
        }

        private void SubjectList_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Enable buttons as needed.
            ActSubjectDelete.Enabled = (SubjectList.SelectedIndex != -1);
            ActSubjectProperties.Enabled = ActSubjectDelete.Enabled;
        }

        private void SubjectList_Click(object sender, EventArgs e)
        {
            // ----- Enable buttons as needed.
            ActSubjectDelete.Enabled = (SubjectList.SelectedIndex != -1);
            ActSubjectProperties.Enabled = ActSubjectDelete.Enabled;
        }

        private void ActSubjectDelete_Click(object sender, EventArgs e)
        {
            // ----- Delete the selected subject.
            if (SubjectList.SelectedIndex == -1)
                return;

            // ----- Confirm with the user.
            if (MessageBox.Show("Really delete the subject '" + SubjectList.Text +
                    "' from this item?", ProgramTitle, MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question) != DialogResult.Yes)
                return;

            // ----- Proceed with the removal.
            if (DeleteOneSubject((long)(ListItemData)SubjectList.SelectedItem) == false)
                return;

            // ----- Remove this item from the display.
            SubjectList.Items.Remove(SubjectList.SelectedItem);
            SubjectList.SelectedIndex = -1;
            ActSubjectDelete.Enabled = false;
            ActSubjectProperties.Enabled = false;
            ActCancel.Text = "Close";
            SessionSaved = true;
        }

        private bool DeleteOneSubject(long subjectID)
        {
            // ----- Remove a subject from this record. Return True on success.
            string sqlText;
            DataCommand sqlRun;

            // ----- Delete the record.
            sqlText = "DELETE FROM ItemSubject WHERE ItemID = @RecordID " +
                "AND SubjectID = @SubID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            sqlRun.AddLong("SubID", subjectID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.DeleteOneSubject.Delete", ex);
                return false;
            }

            // ----- If no books are using this subject, remove it.
            sqlText = "SELECT COUNT(*) FROM ItemSubject WHERE SubjectID = @SubID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("SubID", subjectID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) == 0)
                {
                    sqlText = "DELETE FROM Subject WHERE ID = @SubID";
                    sqlRun = new DataCommand(sqlText);
                    sqlRun.AddLong("SubID", subjectID);
                    ExecuteSQL(sqlRun);
                }
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.DeleteOneSubject.Subject", ex);
                return false;
            }
            return true;
        }

        private void ActKeywordProperties_Click(object sender, EventArgs e)
        {
            // ----- Display some information about the selected keyword.
            string sqlText;
            DataCommand sqlRun;
            int inUse;
            string message;

            // ----- Ignore if no item selected.
            if (KeywordList.SelectedIndex == -1)
                return;

            // ----- Collect some statistics.
            sqlText = "SELECT COUNT(*) FROM ItemKeyword WHERE KeywordID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", (long)(ListItemData)KeywordList.SelectedItem);
            try
            {
                inUse = DBGetInteger(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.ActKeywordProperties_Click", ex);
                return;
            }

            // ----- Display an informational message.
            message = "Keyword: " + KeywordList.Text + Environment.NewLine + Environment.NewLine +
                "This keyword is in use by " + inUse.ToString() + " library item";
            if (inUse == 1)
                message += ".";
            else
                message += "s.";
            MessageBox.Show(message, ProgramTitle,
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void ActSubjectAdd_Click(object sender, EventArgs e)
        {
            // ----- Let the user add a new subject.
            long subjectID;
            string sqlText;
            DataCommand sqlRun;

            // ----- The record must be saved first.
            if (ActiveID == -1L)
            {
                // ----- Confirm with the user.
                if (MessageBox.Show("The item must be saved to the database before subjects " +
                        "can be added. Would you like to save the record now?", ProgramTitle,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
                    return;

                // ----- Verify and save the data.
                if (ValidateFormData() == false)
                    return;
                if (SaveFormData() == false)
                    return;
            }

            // ----- Prompt the user to add a subject.
            subjectID = (new SubjectAdd()).AddRecord();
            if (subjectID == -1L)
                return;

            // ----- See if the record already exists.
            if (SubjectList.Items.IndexOf(subjectID) != -1)
                return;

            // ----- Add the subject to the database record.
            sqlText = "INSERT INTO ItemSubject (ItemID, SubjectID) " +
                "VALUES (@RecordID, @SubID)";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            sqlRun.AddLong("SubID", subjectID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.ActSubjectAdd_Click.Add", ex);
                return;
            }
            ActCancel.Text = "Close";
            SessionSaved = true;

            // ----- Add this item to the display.
            sqlText = "SELECT FullName FROM Subject WHERE ID = @SubID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("SubID", subjectID);
            try
            {
                SubjectList.Items.Add(new ListItemData(
                    DBGetText(ExecuteSQLReturn(sqlRun)), subjectID));
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.ActSubjectAdd_Click.Name", ex);
            }
        }

        private void ActSubjectProperties_Click(object sender, EventArgs e)
        {
            // ----- Display some information about the selected subject.
            string sqlText;
            DataCommand sqlRun;
            int inUse;
            string message;

            // ----- Ignore if no item selected.
            if (SubjectList.SelectedIndex == -1)
                return;

            // ----- Collect some statistics.
            sqlText = "SELECT COUNT(*) FROM ItemSubject WHERE SubjectID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", (long)(ListItemData)SubjectList.SelectedItem);
            try
            {
                inUse = DBGetInteger(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.ActSubjectProperties_Click", ex);
                return;
            }

            // ----- Display an informational message.
            message = "Subject: " + SubjectList.Text + Environment.NewLine + Environment.NewLine +
                "This subject is in use by " + inUse.ToString() + " library item";
            if (inUse == 1)
                message += ".";
            else
                message += "s.";
            MessageBox.Show(message, ProgramTitle,
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void SubjectList_DoubleClick(object sender, EventArgs e)
        {
            // ----- Same as the properties button.
            if (ActSubjectProperties.Enabled == true)
                ActSubjectProperties.PerformClick();
        }

        private void KeywordList_DoubleClick(object sender, EventArgs e)
        {
            // ----- Same as the properties button.
            if (ActKeywordProperties.Enabled == true)
                ActKeywordProperties.PerformClick();
        }

        private void ActAuthorAdd_Click(object sender, EventArgs e)
        {
            // ----- Let the user add a new author.
            long authorID;
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            string authorText;
            long nameTypeID = -1L;
            string typeText;
            int sequenceID;
            ListViewItem newItem;

            // ----- The record must be saved first.
            if (ActiveID == -1L)
            {
                // ----- Confirm with the user.
                if (MessageBox.Show("The item must be saved to the database before authors " +
                        "or names can be added. Would you like to save the record now?", ProgramTitle,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
                    return;

                // ----- Verify and save the data.
                if (ValidateFormData() == false)
                    return;
                if (SaveFormData() == false)
                    return;
            }

            // ----- Prompt the user to add a author.
            authorID = (new AuthorAddLocate()).PromptUser(ref nameTypeID);
            if (authorID == -1L)
                return;

            // ----- Jump out early if the record already exists.
            sqlText = "SELECT COUNT(*) FROM ItemAuthor " +
                "WHERE ItemID = @RecordID AND AuthorID = @AuthID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            sqlRun.AddLong("AuthID", authorID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                    return;
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.ActSubjectAdd_Click.Check", ex);
                return;
            }
            this.Cursor = Cursors.WaitCursor;

            // ----- Get the next sequence number.
            sequenceID = 1;
            sqlText = "SELECT MAX(Sequence) AS TheMax " +
                "FROM ItemAuthor WHERE ItemID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("NamedItem.ActSubjectAdd_Click.Sequence", ex);
                return;
            }
            if (dbInfo.Rows.Count > 0)
            {
                if (Convert.IsDBNull(dbInfo.Rows[0] ["TheMax"]) == false)
                    sequenceID = DBGetInteger(dbInfo.Rows[0] ["TheMax"]) + 1;
            }
            dbInfo.Dispose();

            // ----- Add the author to the database record.
            sqlText = "INSERT INTO ItemAuthor (ItemID, AuthorID, Sequence, AuthorType) " +
                "VALUES (@NewItem, @NewAuthor, @NewSequence, @NewType)";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("NewItem", ActiveID);
            sqlRun.AddLong("NewAuthor", authorID);
            sqlRun.AddInteger("NewSequence", sequenceID);
            sqlRun.AddLong("NewType", nameTypeID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("NamedItem.ActSubjectAdd_Click.Add", ex);
                return;
            }
            ActCancel.Text = "Close";
            SessionSaved = true;

            // ----- Get the text of the name type.
            sqlText = "SELECT FullName FROM CodeAuthorType WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", nameTypeID);
            try
            {
                typeText = DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("NamedItem.ActSubjectAdd_Click.Type", ex);
                return;
            }

            // ----- Get the author name.
            sqlText = "SELECT * FROM Author WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", authorID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("NamedItem.ActSubjectAdd_Click.Author", ex);
                return;
            }
            authorText = General.FormatAuthorName(dbInfo.Rows[0]);
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Add this item to the display.
            newItem = AuthorsList.Items.Add(authorText);
            newItem.SubItems.Add(typeText);
            newItem.Tag = authorID.ToString() + "," + nameTypeID.ToString();
            UpdateAuthorButtons();
            this.Cursor = Cursors.Default;
        }

        private void ActAuthorDelete_Click(object sender, EventArgs e)
        {
            // ----- Allow the user to delete an author entry.
            string sqlText;
            DataCommand sqlRun;

            // ----- Make sure an item is selected.
            if (AuthorsList.SelectedItems.Count == 0)
                return;

            // ----- Confirm with the user.
            if (MessageBox.Show("Really disassociate the author '" +
                    AuthorsList.SelectedItems[0].Text + "' from the library item?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                    DialogResult.Yes)
                return;

            // ----- Remove the selection from the database.
            sqlText = "DELETE FROM ItemAuthor WHERE ItemID = @RecordID " +
                "AND AuthorID = @AuthID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            sqlRun.AddLong("AuthID", ((string)AuthorsList.SelectedItems[0].Tag).GetSubStr(",", 1));
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.ActAuthorDelete_Click", ex);
                return;
            }
            ActCancel.Text = "Close";
            SessionSaved = true;

            // ----- Remove the item from the display.
            AuthorsList.Items.Remove(AuthorsList.SelectedItems[0]);
            UpdateAuthorButtons();
        }

        private void ActAuthorMoveUp_Click(object sender, EventArgs e)
        {
            // ----- Move the selected author up in priority.
            if (AuthorsList.SelectedItems.Count == 0)
                return;
            if (AuthorsList.SelectedItems[0].Index == 0)
                return;

            SwitchTwoAuthors(AuthorsList.SelectedItems[0].Index,
                AuthorsList.SelectedItems[0].Index - 1);
        }

        private void ActAuthorMoveDown_Click(object sender, EventArgs e)
        {
            // ----- Move the selected author down in priority.
            if (AuthorsList.SelectedItems.Count == 0)
                return;
            if (AuthorsList.SelectedItems[0].Index == (AuthorsList.Items.Count - 1))
                return;

            SwitchTwoAuthors(AuthorsList.SelectedItems[0].Index,
                AuthorsList.SelectedItems[0].Index + 1);
        }

        private void SwitchTwoAuthors(int activeIndex, int inactiveIndex)
        {
            // ----- Given the indexes of two items in the AuthorsList control,
            //       switch them in the database and on-screen.
            long activeAuthor;
            long inactiveAuthor;
            int activeSequence;
            int inactiveSequence;
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            ListViewItem activeItem;
            ListViewItem inactiveItem;
            string holdText;
            LibraryTransaction transInfo = null;

            // ----- Get the author IDs from the two items.
            activeItem = AuthorsList.Items[activeIndex];
            inactiveItem = AuthorsList.Items[inactiveIndex];
            activeAuthor = Convert.ToInt64(((string)(activeItem.Tag)).GetSubStr(",", 1));
            inactiveAuthor = Convert.ToInt64(((string)(inactiveItem.Tag)).GetSubStr(",", 1));

            // ----- Get the sequence numbers of the two records.
            activeSequence = -1;
            inactiveSequence = -1;
            sqlText = "SELECT AuthorID, Sequence FROM ItemAuthor " +
                "WHERE ItemID = @RecordID AND AuthorID IN (@Author1, @Author2)";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            sqlRun.AddLong("Author1", activeAuthor);
            sqlRun.AddLong("Author2", inactiveAuthor);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.SwitchTwoAuthors.Sequence", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                if (DBGetLong(dbRow["AuthorID"]) == activeAuthor)
                    activeSequence = DBGetInteger(dbRow["Sequence"]);
                else if (DBGetLong(dbRow["AuthorID"]) == inactiveAuthor)
                    inactiveSequence = DBGetInteger(dbRow["Sequence"]);
            }
            dbInfo.Dispose();
            dbInfo = null;
            if ((activeSequence == -1) | (inactiveSequence == -1))
            {
                MessageBox.Show("Error occurred while trying to access the author or name records.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Update the database records.
            this.Cursor = Cursors.WaitCursor;
            try
            {
                transInfo = TransactionBegin();

                sqlText = "UPDATE ItemAuthor SET Sequence = @NewSequence " +
                    "WHERE ItemID = @RecordID AND AuthorID = @AuthID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddInteger("NewSequence", inactiveSequence);
                sqlRun.AddLong("RecordID", ActiveID);
                sqlRun.AddLong("AuthID", activeAuthor);
                ExecuteSQL(sqlRun, transInfo);

                sqlRun = new DataCommand(sqlText);
                sqlRun.AddInteger("NewSequence", activeSequence);
                sqlRun.AddLong("RecordID", ActiveID);
                sqlRun.AddLong("AuthID", inactiveAuthor);
                ExecuteSQL(sqlRun, transInfo);

                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                if (transInfo != null)
                    TransactionRollback(transInfo);
                GeneralError("NamedItem.SwitchTwoAuthors.Swap", ex);
                return;
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }

            ActCancel.Text = "Close";
            SessionSaved = true;

            // ----- Reverse the elements on the display.
            holdText = activeItem.Text;
            activeItem.Text = inactiveItem.Text;
            inactiveItem.Text = holdText;

            holdText = activeItem.SubItems[1].Text;
            activeItem.SubItems[1].Text = inactiveItem.SubItems[1].Text;
            inactiveItem.SubItems[1].Text = holdText;

            holdText = Convert.ToString(activeItem.Tag);
            activeItem.Tag = inactiveItem.Tag;
            inactiveItem.Tag = holdText;

            // ----- Highlight the correct item.
            AuthorsList.SelectedItems.Clear();
            inactiveItem.Selected = true;
            UpdateAuthorButtons();
        }

        private void ActAuthorProperties_Click(object sender, EventArgs e)
        {
            // ----- Allow the use to change the name type.
            string sqlText;
            DataCommand sqlRun;
            long newNameType;
            long oldNameType;
            long authorID;
            string holdText;

            // ----- Check for invalid data.
            if (AuthorsList.SelectedItems.Count == 0)
                return;

            // ----- Get the IDs of the author and author type.
            holdText = (string)AuthorsList.SelectedItems[0].Tag;
            authorID = Convert.ToInt64(holdText.GetSubStr(",", 1));
            oldNameType = Convert.ToInt64(holdText.GetSubStr(",", 2));

            // ----- Prompt the user.
            newNameType = (new ItemAuthorEdit()).EditRecord(ActiveID, authorID,
                BuildItemName(), AuthorsList.SelectedItems[0].Text, oldNameType);
            if (newNameType == -1L)
                return;
            ActCancel.Text = "Close";
            SessionSaved = true;
            if (newNameType == oldNameType)
                return;

            // ----- Update the internal value of the display item.
            AuthorsList.SelectedItems[0].Tag = authorID + "," + newNameType;

            // ----- Get the text for the new name type and update the display.
            sqlText = "SELECT FullName FROM CodeAuthorType WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", newNameType);
            try
            {
                AuthorsList.SelectedItems[0].SubItems[1].Text =
                    DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.ActAuthorProperties_Click", ex);
            }
        }

        private void UpdateAuthorButtons()
        {
            // ----- Update the author panel buttons based on selection.
            if (AuthorsList.SelectedItems.Count == 0)
            {
                // ----- No selection.
                ActAuthorDelete.Enabled = false;
                ActAuthorProperties.Enabled = false;
                ActAuthorMoveUp.Enabled = false;
                ActAuthorMoveDown.Enabled = false;
            }
            else
            {
                // ----- Selection.
                ActAuthorDelete.Enabled = true;
                ActAuthorProperties.Enabled = true;
                ActAuthorMoveUp.Enabled = (AuthorsList.SelectedItems[0].Index > 0);
                ActAuthorMoveDown.Enabled =
                    (AuthorsList.SelectedItems[0].Index < (AuthorsList.Items.Count - 1));
            }
        }

        private void AuthorsList_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Refresh the buttons.
            UpdateAuthorButtons();
        }

        private void AuthorsList_Click(object sender, EventArgs e)
        {
            // ----- Refresh the buttons.
            UpdateAuthorButtons();
        }

        private void ActCopyAdd_Click(object sender, EventArgs e)
        {
            // ----- Let the user add a new author.
            long copyID;
            string sqlText;
            DataCommand sqlRun;
            string barcode;
            DataTable dbInfo;
            DataRow oneRow;
            string copyNumber;
            ListViewItem newItem;

            // ----- The record must be saved first.
            if (ActiveID == -1L)
            {
                // ----- Confirm with the user.
                if (MessageBox.Show("The item must be saved to the database before copies " +
                        "can be added. Would you like to save the record now?", ProgramTitle,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
                    return;

                // ----- Verify and save the data.
                if (ValidateFormData() == false)
                    return;
                if (SaveFormData() == false)
                    return;
            }

            // ----- Prompt the user to add a subject.
            copyID = (new ItemCopy()).AddCopy(ActiveID, BuildItemName());
            if (copyID == -1L)
                return;
            ActCancel.Text = "Close";
            SessionSaved = true;

            // ----- Get some information about the copy.
            sqlText = "SELECT CopyNumber, Barcode FROM ItemCopy WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", copyID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.ActCopyAdd_Click", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            copyNumber = DBGetInteger(oneRow["CopyNumber"]).ToString();
            barcode = DBGetText(oneRow["Barcode"]);
            dbInfo.Dispose();
            dbInfo = null;
            if (barcode.Length == 0)
                barcode = "None";

            // ----- Add this item to the display.
            newItem = CopyList.Items.Add(copyNumber);
            newItem.SubItems.Add(GetCopyDisposition(copyID));
            newItem.SubItems.Add(barcode);
            newItem.Tag = copyID.ToString();
            UpdateCopyButtons();
        }

        private string BuildItemName()
        {
            // ----- Build the text of the item name used by some subordinate
            //       forms.
            string itemName;

            // ----- The name includes title, subtitle, edition, and media type.
            itemName = RecordTitle.Text;
            if (RecordSubtitle.Text.Trim().Length != 0)
                itemName += ": " + RecordSubtitle.Text;
            if (RecordEdition.Text.Trim().Length != 0)
                itemName += ", " + RecordEdition.Text;
            if ((long)(ListItemData)RecordMediaType.SelectedItem != -1L)
                itemName += " [" + RecordMediaType.Text + "]";
            return itemName;
        }

        private void ActCopyEdit_Click(object sender, EventArgs e)
        {
            // ----- Let the user edit an existing author.
            long copyID;
            string sqlText;
            DataCommand sqlRun;
            string barcode;
            DataTable dbInfo;
            DataRow oneRow;
            string copyNumber;
            ListViewItem newItem;

            // ----- Check for invalid data.
            if (CopyList.SelectedItems.Count == 0)
                return;
            newItem = CopyList.SelectedItems[0];
            copyID = Convert.ToInt64(newItem.Tag);

            // ----- Prompt the user to add a subject.
            copyID = (new ItemCopy()).EditCopy(ActiveID, copyID, BuildItemName());
            if (copyID == -1L)
                return;
            ActCancel.Text = "Close";
            SessionSaved = true;

            // ----- Get some information about the copy.
            sqlText = "SELECT CopyNumber, Barcode FROM ItemCopy WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", copyID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("NamedItem.ActCopyEdit_Click", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            copyNumber = DBGetInteger(oneRow["CopyNumber"]).ToString();
            barcode = DBGetText(oneRow["Barcode"]);
            dbInfo.Dispose();
            dbInfo = null;
            if (barcode.Length == 0)
                barcode = "None";

            // ----- Update this item in the display.
            newItem.SubItems[0].Text = copyNumber;
            newItem.SubItems[1].Text = GetCopyDisposition(copyID);
            newItem.SubItems[2].Text = barcode;
            UpdateCopyButtons();
        }

        private void ActCopyDelete_Click(object sender, EventArgs e)
        {
            // ----- Allow the user to delete an item copy.
            string sqlText;
            DataCommand sqlRun;
            string holdText;
            long copyID;
            LibraryTransaction transInfo = null;

            // ----- Make sure an item is selected.
            if (CopyList.SelectedItems.Count == 0)
                return;
            copyID = Convert.ToInt64(CopyList.SelectedItems[0].Tag);

            // ----- Make sure this item is not checked out.
            holdText = CopyList.SelectedItems[0].SubItems[1].Text;
            if ((holdText == "Checked Out") | (holdText == "Overdue"))
            {
                MessageBox.Show("You cannot delete this item because it is still checked out.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Confirm with the user.
            if (MessageBox.Show("Deleting an item copy from the library is a permanent " +
                    "process; it cannot be undone without re-entering the item." +
                    Environment.NewLine + Environment.NewLine +
                    "Also, any patron history associated with this " +
                    "item copy will be lost. This includes any unpaid fines associated " +
                    "with the item copy." + Environment.NewLine + Environment.NewLine +
                    "Modifying the item copy and setting it to 'Not Available' is " +
                    "often a better choice." + Environment.NewLine + Environment.NewLine +
                    "Really delete copy '" + CopyList.SelectedItems[0].Text +
                    "' from the database?", ProgramTitle, MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question) != DialogResult.Yes)
                return;

            // ----- Remove the selection from the database.
            try
            {
                transInfo = TransactionBegin();

                sqlText = "DELETE FROM PatronPayment WHERE PatronCopy IN " +
                    "(SELECT ID FROM PatronCopy WHERE ItemCopy = @RecordID)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", copyID);
                ExecuteSQL(sqlRun, transInfo);

                sqlText = "DELETE FROM PatronCopy WHERE ItemCopy = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", copyID);
                ExecuteSQL(sqlRun, transInfo);

                sqlText = "DELETE FROM ItemCopy WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", copyID);
                ExecuteSQL(sqlRun, transInfo);

                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                if (transInfo != null)
                    TransactionRollback(transInfo);
                GeneralError("NamedItem.ActCopyDelete_Click", ex);
                return;
            }

            ActCancel.Text = "Close";
            SessionSaved = true;

            // ----- Remove the item from the display.
            CopyList.Items.Remove(CopyList.SelectedItems[0]);
            UpdateCopyButtons();
        }

        private void UpdateCopyButtons()
        {
            // ----- Update the author panel buttons based on selection.
            if (CopyList.SelectedItems.Count == 0)
            {
                // ----- No selection.
                ActCopyEdit.Enabled = false;
                ActCopyDelete.Enabled = false;
            }
            else
            {
                // ----- Selection.
                ActCopyEdit.Enabled = true;
                ActCopyDelete.Enabled = true;
            }
        }

        private void CopyList_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Refresh the buttons.
            UpdateCopyButtons();
        }

        private void CopyList_Click(object sender, EventArgs e)
        {
            // ----- Refresh the buttons.
            UpdateCopyButtons();
        }

        private void AuthorsList_DoubleClick(object sender, EventArgs e)
        {
            // ----- Same as a click on the edit button.
            if (ActAuthorProperties.Enabled == true)
                ActAuthorProperties.PerformClick();
        }

        private void CopyList_DoubleClick(object sender, EventArgs e)
        {
            // ----- Same as a click on the edit button.
            if (ActCopyEdit.Enabled == true)
                ActCopyEdit.PerformClick();
        }

        private void RecordTitle_TextChanged(object sender, EventArgs e)
        {
            // ----- Update the large display title when the title changes.
            if (RecordTitle.Text.Trim().Length == 0)
                MainTitle.Text = "Untitled";
            else
                MainTitle.Text = RecordTitle.Text.Trim();
        }
    }
}
