﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class SystemValue : Library.BaseCodeForm
    {
        private long ActiveID;
        private bool StartingOver;

        public SystemValue()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            ActiveID = -1L;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManageValues];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManageValues];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManageValues];
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the records.
            string sqlText;
            DataCommand sqlRun;

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete the system value?",
                    ProgramTitle, MessageBoxButtons.YesNo,
                    MessageBoxIcon.Question) != DialogResult.Yes)
                return false;

            // ----- Delete the record.
            sqlText = "DELETE FROM SystemValue WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("SystemValue.DeleteRecord", ex);
                return false;
            }
            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record.
            ActiveID = recordID;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items.
            string sqlText;
            DataTable dbInfo;

            // ----- Clear any existing values.
            destList.Items.Clear();
            exceededMatches = false;

            // ----- Retrieve the values from the database.
            sqlText = "SELECT * FROM SystemValue ORDER BY ValueName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("SystemValue.FillListWithRecords", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                destList.Items.Add(new ListItemData(
                    DBGetText(dbRow["ValueName"]), DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;

            // ----- Retrieve the value from the database.
            sqlText = "SELECT ValueName FROM SystemValue WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                return DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("SystemValue.FormatRecordName", ex);
                return "Error";
            }
        }

        public override string GetEditDescription()
        {
            // ----- Return the list form display title.
            return "Use these codes to configure extended system features. These values " +
                "are defined and managed by the system administrator.";
        }

        public override string GetEditTitle()
        {
            // ----- Return the list form display title.
            return "System Values";
        }

        private void PrepareFormFields()
        {
            // ----- Fill in the data.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;

            // ----- Clear the existing data.
            StartingOver = true;
            RecordFullName.Text = "";
            RecordData.Text = "";
            if (ActiveID == -1L)
                return;

            // ----- Load in the values stored in the database.
            sqlText = "SELECT * FROM SystemValue WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("SystemValue.PrepareFormFields", ex);
                return;
            }
            if (dbInfo.Rows.Count > 0)
            {
                oneRow = dbInfo.Rows[0];
                RecordFullName.Text = DBGetText(oneRow["ValueName"]);
                RecordData.Text = DBGetText(oneRow["ValueData"]);
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        private bool SaveFormData()
        {
            // ----- Save changes to the database.
            string sqlText;
            DataCommand sqlRun;
            long newID;

            this.Cursor = Cursors.WaitCursor;

            // ----- Save the data.
            if (ActiveID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO SystemValue (ValueName, ValueData) " +
                    "OUTPUT INSERTED.ID VALUES (@NewName, @NewValue)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                sqlRun.AddText("NewValue", RecordData.Text.Trim());
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    GeneralError("SystemValue.SaveFormData.Add", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
                ActiveID = newID;
            }
            else
            {
                // ----- Update the existing entry.
                sqlText = "UPDATE SystemValue SET " +
                    "ValueName = @NewName, ValueData = @NewValue " +
                    "WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                sqlRun.AddText("NewValue", RecordData.Text.Trim());
                sqlRun.AddLong("RecordID", ActiveID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("SystemValue.SaveFormData.Update", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }

            // ----- Success.
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            string sqlText;
            DataCommand sqlRun;

            // ----- Name is a required field.
            if (RecordFullName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the system value name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordFullName.Focus();
                return false;
            }

            // ----- Make sure the name is not already in use.
            sqlText = "SELECT COUNT(*) FROM SystemValue " +
                "WHERE UPPER(ValueName) = @TestName";
            if (ActiveID != -1L)
                sqlText += " AND ID <> @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestName", RecordFullName.Text.Trim().ToUpper());
            if (ActiveID != -1L)
                sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    // ----- Duplicate entry.
                    MessageBox.Show("A record already exists with the name '" +
                        RecordFullName.Text.Trim() + "'.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordFullName.Focus();
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("SystemValue.ValidateFormData", ex);
                return false;
            }

            // ----- Success.
            return true;
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void SystemValue_Activated(object sender, EventArgs e)
        {
            // ----- Return the focus to the main field.
            if (StartingOver)
                RecordFullName.Focus();
            StartingOver = false;
        }

        private void RecordFullName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordFullName.SelectAll();
        }

        private void RecordData_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordData.SelectAll();
        }
    }
}
