﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Collections.Generic;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    internal static class TicketPrinting
    {
        public static void PrintCheckoutTicket(long patronID, ListBox checkedOutItems)
        {
            // ----- Print out a ticket of what the patron checked out. The supplied
            //       ListBox control contains objects of type CheckedOutItem.
            int ticketWidth;
            System.Text.StringBuilder ticketText;
            decimal patronFines;
            CheckedOutItem itemDetail;
            string holdText;

            // ----- Ignore if there is nothing to print.
            if ((patronID == -1L) | (checkedOutItems.Items.Count == 0))
                return;

            // ----- Get the width of the ticket.
            ticketWidth = Properties.Settings.Default.ReceiptWidth;
            if (ticketWidth <= 0)
                ticketWidth = 40;

            // ----- Build the heading.
            ticketText = GetTicketHeader(patronID, ticketWidth);
            if (ticketText == null)
                return;

            // ----- Process each checked-out item.
            for (int counter = 0; counter < checkedOutItems.Items.Count; counter++)
            {
                // ----- Extract the detail from the list.
                itemDetail = (CheckedOutItem)(checkedOutItems.Items[counter]);

                // ----- Add the item name.
                holdText = itemDetail.ItemTitle;
                if (holdText.Length > ticketWidth)
                    holdText = holdText.Substring(0, ticketWidth).TrimEnd();
                ticketText.AppendLine(holdText);

                // ----- Add the barcode number and due date.
                ticketText.AppendLine(LeftAndRightText(itemDetail.Barcode,
                    "Due: " + itemDetail.DueDate.ToString("MMM d, yyyy"), ticketWidth));
                ticketText.AppendLine();
            }

            // ----- If there are fines due, print them here.
            patronFines = CalculatePatronFines(patronID);
            if (patronFines > 0M)
            {
                ticketText.AppendLine("Fines Due: " + patronFines.ToString("C"));
                ticketText.AppendLine();
            }

            // ----- Add the bottom display text.
            ticketText.Append(GetTicketFooter(ticketWidth));

            // ----- Send the ticket to the printer.
            try
            {
                RawPrinterHelper.SendStringToPrinter(
                    Properties.Settings.Default.ReceiptPrinter,
                    ticketText.ToString(), "Checkout Receipt");
            }
            catch (Exception ex)
            {
                GeneralError("TicketPrinting.PrintCheckoutTicket", ex);
            }
        }

        public static void PrintBalanceTicket(long patronID, ListBox finesList)
        {
            // ----- Print out a ticket of unpaid balances. The supplied ListBox
            //       control contains objects of type PatronDetialItem.
            int ticketWidth;
            System.Text.StringBuilder ticketText;
            string holdText;
            PatronDetailItem itemDetail;
            int included = 0;

            // ----- Ignore if there is nothing to print.
            if ((patronID == -1L) | (finesList.Items.Count == 0))
                return;

            // ----- Get the width of the ticket.
            ticketWidth = Properties.Settings.Default.ReceiptWidth;
            if (ticketWidth <= 0)
                ticketWidth = 40;

            // ----- Build the heading.
            ticketText = GetTicketHeader(patronID, ticketWidth);
            if (ticketText == null)
                return;

            // ----- Process each checked-out item.
            for (int counter = 0; counter < finesList.Items.Count; counter++)
            {
                // ----- Extract the detail from the list.
                itemDetail = (PatronDetailItem)(finesList.Items[counter]);

                // ----- Skip entries with a zero balance.
                if (itemDetail.BalanceDue == 0M)
                    continue;
                included += 1;

                // ----- Add the item name.
                holdText = itemDetail.TitleText;
                if (holdText.Length > ticketWidth)
                    holdText = holdText.Substring(0, ticketWidth).TrimEnd();
                ticketText.AppendLine(holdText);

                // ----- Print the remaining balance.
                ticketText.AppendLine(LeftAndRightText("Balance Due:",
                    itemDetail.BalanceDue.ToString("C"), ticketWidth));
                ticketText.AppendLine();
            }

            // ----- Warn if there is nothing to print.
            if (included == 0)
            {
                MessageBox.Show("There are no unpaid balances to print.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            // ----- If there are fines due, print them here.
            ticketText.AppendLine("Fines Due: " +
                CalculatePatronFines(patronID).ToString("C"));
            ticketText.AppendLine();

            // ----- Add the bottom display text.
            ticketText.Append(GetTicketFooter(ticketWidth));

            // ----- Send the ticket to the printer.
            try
            {
                RawPrinterHelper.SendStringToPrinter(
                    Properties.Settings.Default.ReceiptPrinter,
                    ticketText.ToString(), "Balance Receipt");
            }
            catch (Exception ex)
            {
                GeneralError("TicketPrinting.PrintBalanceTicket", ex);
            }
        }

        public static void PrintPaymentTicket(long patronID, List<PaymentItem> paymentList)
        {
            // ----- Print out a ticket of unpaid balances. The supplied ListBox
            //       control contains objects of type PatronDetialItem.
            int ticketWidth;
            System.Text.StringBuilder ticketText;
            string holdText;
            PaymentItem itemDetail;
            int included = 0;

            // ----- Ignore if there is nothing to print.
            if ((patronID == -1L) | (paymentList.Count == 0))
                return;

            // ----- Get the width of the ticket.
            ticketWidth = Properties.Settings.Default.ReceiptWidth;
            if (ticketWidth <= 0)
                ticketWidth = 40;

            // ----- Build the heading.
            ticketText = GetTicketHeader(patronID, ticketWidth);
            if (ticketText == null)
                return;

            // ----- Process each checked-out item.
            for (int counter = 0; counter < paymentList.Count; counter++)
            {
                // ----- Extract the detail from the list.
                itemDetail = paymentList[counter];

                // ----- Ignore items with no payment amount.
                if (itemDetail.FeesPaid == 0M)
                    continue;
                included += 1;

                // ----- Add the item name.
                holdText = itemDetail.ItemTitle;
                if (holdText.Length > ticketWidth)
                    holdText = holdText.Substring(0, ticketWidth).TrimEnd();
                ticketText.AppendLine(holdText);

                // ----- Show the payment amount.
                ticketText.AppendLine(LeftAndRightText("Payments:",
                    itemDetail.FeesPaid.ToString("C"), ticketWidth));

                // ----- Print the remaining balance.
                ticketText.AppendLine(LeftAndRightText("Balance Due:",
                    itemDetail.BalanceDue.ToString("C"), ticketWidth));
                ticketText.AppendLine();
            }
            if (included == 0)
                return;

            // ----- If there are fines due, print them here.
            ticketText.AppendLine("Fines Due: " +
                CalculatePatronFines(patronID).ToString("C"));
            ticketText.AppendLine();

            // ----- Add the bottom display text.
            ticketText.Append(GetTicketFooter(ticketWidth));

            // ----- Send the ticket to the printer.
            try
            {
                RawPrinterHelper.SendStringToPrinter(
                    Properties.Settings.Default.ReceiptPrinter,
                    ticketText.ToString(), "Balance Receipt");
            }
            catch (Exception ex)
            {
                GeneralError("TicketPrinting.PrintPaymentTicket", ex);
            }
        }

        private static System.Text.StringBuilder GetTicketHeader(long patronID, int ticketWidth)
        {
            // ----- Print out a ticket of what the patron checked out. The supplied
            //       ListBox control contains objects of type CheckedOutItem.
            System.Text.StringBuilder ticketText;
            string holdText;
            string sqlText;
            DataCommand sqlRun;

            // ----- Check for an active printer.
            if (Properties.Settings.Default.UseReceipts == false)
                return null;

            // ----- Get the printer path.
            if (Properties.Settings.Default.ReceiptPrinter == null)
                return null;
            if (Properties.Settings.Default.ReceiptPrinter.Trim().Length == 0)
                return null;

            // ----- Create the storage area for the content.
            ticketText = new System.Text.StringBuilder();

            // ----- Start with the heading.
            holdText = GetSystemValue("TicketHeading").Trim();
            if (holdText.Length != 0)
            {
                for (int counter = 1; counter <= CountSubStr(holdText, "|") + 1; counter++)
                {
                    // ----- Center each line of the heading.
                    ticketText.AppendLine(CenterText(holdText.GetSubStr("|", counter), ticketWidth));
                }
            }

            // ----- Add today's date.
            ticketText.AppendLine(CenterText(DateTime.Today.ToString("MMMM d, yyyy"), ticketWidth));
            ticketText.AppendLine();

            // ----- Add the patron name.
            sqlText = "SELECT FirstName + ' ' + LastName FROM Patron " +
                "WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", patronID);
            try
            {
                ticketText.AppendLine(CenterText("Patron: " +
                    DBGetText(ExecuteSQLReturn(sqlRun)), ticketWidth));
            }
            catch (Exception ex)
            {
                GeneralError("TicketPrinting.GetTicketHeader", ex);
            }
            ticketText.AppendLine();

            // ----- That's the full header.
            return ticketText;
        }

        private static string GetTicketFooter(int ticketWidth)
        {
            // ----- Build the content for the end of the ticket.
            string ticketText = "";
            string holdText;

            // ----- Add the bottom display text.
            holdText = GetSystemValue("TicketFooting").Trim();
            if (holdText.Length != 0)
            {
                for (int counter = 1; counter <= CountSubStr(holdText, "|") + 1; counter++)
                {
                    // ----- Center each line of the heading.
                    ticketText += CenterText(holdText.GetSubStr("|", counter),
                        ticketWidth) + Environment.NewLine;
                }
            }

            // ----- Include any final characters.
            ticketText += GetPrinterPostlude();
            return ticketText;
        }

        private static string GetPrinterPostlude()
        {
            // ----- Retrieve the postlude and process any escape sequences.
            string origText;
            string oneChar;
            string resultText;
            int position;
            int hexCode;

            // ----- Retrieve the raw postlude.
            origText = Properties.Settings.Default.ReceiptPostlude;
            if (origText == null)
                return "";
            if (origText.Trim().Length == 0)
                return "";

            // ----- Scan through the text and process any backslash characters.
            resultText = "";
            position = 0;
            while (position < origText.Length)
            {
                // ----- Peek at the next character.
                oneChar = origText.Substring(position, 1);
                if (oneChar == "\\")
                {
                    // ----- Handle the escape character.
                    if (position == (origText.Length - 1))
                    {
                        // ----- Already at the end.
                        oneChar = "";
                    }
                    else
                    {
                        oneChar = origText.Substring(position + 1, 1);
                    }
                    switch (oneChar.ToUpper())
                    {
                        case "E":
                            // ----- Add the escape character.
                            resultText += (char)Keys.Escape;
                            position += 2;
                            break;
                        case "N":
                            // ----- Add the newline character.
                            resultText += (char)Keys.LineFeed;
                            position += 2;
                            break;
                        case "R":
                            // ----- Add the carriage return character.
                            resultText += (char)Keys.Return;
                            position += 2;
                            break;
                        case "X":
                            // ----- Add in an ASCII value identified by the next
                            //       two hex digits.
                            try
                            {
                                oneChar = origText.Substring(position + 2, 2);
                                hexCode = Convert.ToInt32(oneChar, 16);
                                resultText += (char)hexCode;
                            }
                            catch
                            {
                                // ----- Ignore this character.
                            }
                            finally
                            {
                                position += 4;
                            }
                            break;
                        case "":
                            // ----- Already at the end. Just skip the backslash.
                            position += 1;
                            break;
                        default:
                            // ----- Simply add this character to the final output.
                            //       oneChar may be the "\" character.
                            resultText += oneChar;
                            position += 2;
                            break;
                    }
                }
                else
                {
                    // ----- Simply add this character to the final output.
                    resultText += oneChar;
                    position += 1;
                }
            }

            // ----- Return the final text.
            return resultText;
        }
    }
}
