﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class CheckLookup : Form
    {
        private bool UseLC;
        private int SecondItemRow;
        private bool CheckInMode;

        public CheckLookup()
        {
            InitializeComponent();
        }

        private class CheckItemDetail
        {
            // ----- Used instead of ListItemData, with extra fields.
            public string TitleText = "";
            public string AuthorName = "";
            public bool MultiAuthor = false;
            public string MediaType = "";
            public string CallNumber = "-----";
            public string Barcode = "-----";
            public int CopyNumber = -1;
            public long ItemCopyID = -1L;
        }

        public long CheckItemByTitle(bool checkIn, string searchText)
        {
            // ----- Prompt the user for a title item. Return the item copy
            //       ID on success, or -1 if no item is selected.
            CheckItemDetail itemDetail;

            // ----- Determine if Dewey or LC is used, and other initialization.
            UseLC = GetBoolean(GetSystemValue("UseLC"));

            // ----- Perform the first search.
            if (PerformLookup(checkIn, searchText) == false)
            {
                MessageBox.Show("Your search resulted in no matches.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return -1L;
            }
            CheckInMode = checkIn;

            // ----- Show the results to the user.
            if (this.ShowDialog() == DialogResult.Cancel)
                return -1L;

            // ----- Extract the copy ID number and return it.
            itemDetail = (CheckItemDetail)MatchingItems.SelectedItem;
            return itemDetail.ItemCopyID;
        }

        private bool PerformLookup(bool asCheckIn, string searchText)
        {
            // ----- Perform a check-in or check-out title search. Return True
            //       if matches were found.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            string useSearch;
            int matches;
            long itemID;
            CheckItemDetail matchItem = null;

            // ----- Prepare the title.
            useSearch = searchText.Trim();
            if (useSearch.Contains("*") == false)
                useSearch += "*";
            useSearch = useSearch.Replace("*", "%");

            // ----- Perform the search.
            sqlText = "SELECT AU.LastName, AU.FirstName, AU.MiddleName, AU.Suffix, " +
                "AU.BirthYear, AU.DeathYear, NI.Title, NI.SubTitle, NI.Dewey, " +
                "NI.LC, CMT.FullName AS MediaName, IC.ID, IC.Barcode, IC.CopyNumber " +
                "FROM ItemCopy AS IC " +
                "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " +
                "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " +
                "LEFT JOIN ItemAuthor AS IA ON NI.ID = IA.ItemID " +
                "LEFT JOIN Author AS AU ON IA.AuthorID = AU.ID " +
                "WHERE NI.Title LIKE @TestTitle";
            if (asCheckIn)
                sqlText += " AND IC.ID IN";
            else
                sqlText += " AND IC.ID NOT IN";
            sqlText += " (SELECT ItemCopy FROM PatronCopy WHERE Returned = 0)";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestTitle", useSearch);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("ItemLookup.PerformLookup", ex);
                return false;
            }
            if (dbInfo.Rows.Count == 0)
            {
                // ----- No matches.
                return false;
            }

            // ----- Prepare for item matches.
            MatchingItems.Items.Clear();
            matches = 0;
            itemID = -1L;
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                if (itemID != DBGetLong(dbRow["ID"]))
                {
                    // ----- Create a new matching item entry.
                    matchItem = new CheckItemDetail();
                    matchItem.TitleText = DBGetText(dbRow["Title"]);
                    if (Convert.IsDBNull(dbRow["Subtitle"]) == false)
                        matchItem.TitleText += ": " + DBGetText(dbRow["Subtitle"]);
                    matchItem.AuthorName = dbRow.FormatAuthorName();
                    matchItem.MediaType = DBGetText(dbRow["MediaName"]);
                    if (UseLC == true)
                    {
                        if (Convert.IsDBNull(dbRow["LC"]) == false)
                            matchItem.CallNumber = DBGetText(dbRow["LC"]).GetSubStr("/", 1);
                    }
                    else
                    {
                        if (Convert.IsDBNull(dbRow["Dewey"]) == false)
                            matchItem.CallNumber = DBGetText(dbRow["Dewey"]).GetSubStr("/", 1);
                    }
                    if (Convert.IsDBNull(dbRow["Barcode"]) == false)
                        matchItem.Barcode = DBGetText(dbRow["Barcode"]);
                    if (Convert.IsDBNull(dbRow["CopyNumber"]) == false)
                        matchItem.CopyNumber = DBGetInteger(dbRow["CopyNumber"]);
                    matchItem.ItemCopyID = DBGetLong(dbRow["ID"]);

                    // ----- Ready to display the item.
                    MatchingItems.Items.Add(matchItem);

                    // ----- Limit the search to SearchMatchLimit matches.
                    matches += 1;
                    if (matches >= SearchMatchLimit)
                    {
                        if (dbInfo.Rows.Count > SearchMatchLimit)
                            matches += 1;
                        break;
                    }
                }
                else
                {
                    // ----- We found another match for the previous item, caused
                    //       by the presence of another author record. Modify the
                    //       last item by adding "et. al." to the display.
                    if (matchItem.MultiAuthor == false)
                    {
                        matchItem.AuthorName += ", et. al.";
                        matchItem.MultiAuthor = true;
                    }
                }
            }
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Display the results.
            if (matches == 1)
                LabelItemsCount.Text = "1 matching item found.";
            else if (matches > SearchMatchLimit)
                LabelItemsCount.Text =
                    $"Limited to first {SearchMatchLimit} matching items.";
            else
                LabelItemsCount.Text = $"{matches} matching items found.";
            return true;
        }

        private void MatchingItems_DrawItem(object sender, DrawItemEventArgs e)
        {
            // ----- Draw the matching items on two lines.
            Brush useBrush;
            string copyBarcodeText;
            Font boldFont;
            CheckItemDetail itemDetail;
            StringFormat ellipsesText;

            // ----- Draw the background of the item.
            if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
                useBrush = SystemBrushes.HighlightText;
            else
                useBrush = SystemBrushes.WindowText;
            e.DrawBackground();

            // ----- Make a bold version of the font.
            boldFont = new Font(e.Font, FontStyle.Bold);

            // ----- Extract the details from the list.
            itemDetail = (CheckItemDetail)MatchingItems.Items[e.Index];

            // ----- Format the display strings.
            if (itemDetail.CopyNumber == -1)
                copyBarcodeText = "??";
            else
                copyBarcodeText = itemDetail.CopyNumber.ToString();
            copyBarcodeText += " / " + itemDetail.Barcode;
            ellipsesText = new StringFormat();
            ellipsesText.Trimming = StringTrimming.EllipsisCharacter;

            // ----- Draw the text of the item.
            e.Graphics.DrawString(itemDetail.TitleText, boldFont, useBrush,
                new Rectangle(0, e.Bounds.Top, ItemColBarcode.Left -
                MatchingItems.Left - 5, boldFont.Height), ellipsesText);
            e.Graphics.DrawString(copyBarcodeText, e.Font, useBrush,
                new Rectangle(ItemColBarcode.Left - ItemColTitle.Left, e.Bounds.Top,
                ItemColEnd.Left - ItemColBarcode.Left, e.Font.Height), ellipsesText);
            e.Graphics.DrawString(itemDetail.AuthorName, e.Font, useBrush,
                new Rectangle(ItemColAuthor.Left - ItemColTitle.Left, e.Bounds.Top + SecondItemRow,
                ItemColCall.Left - ItemColAuthor.Left - 8, e.Font.Height), ellipsesText);
            e.Graphics.DrawString(itemDetail.CallNumber, e.Font, useBrush,
                new Rectangle(ItemColCall.Left - ItemColTitle.Left, e.Bounds.Top + SecondItemRow,
                ItemColEnd.Left - ItemColType.Left - 5, e.Font.Height), ellipsesText);
            e.Graphics.DrawString(itemDetail.MediaType, e.Font, useBrush,
                new Rectangle(ItemColType.Left - ItemColTitle.Left, e.Bounds.Top + SecondItemRow,
                ItemColType.Left - ItemColCall.Left - 8, e.Font.Height), ellipsesText);

            // ----- If the ListBox has focus, draw a focus rectangle.
            e.DrawFocusRectangle();
            boldFont.Dispose();
        }

        private void CheckLookup_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- F1 shows online help.
            if (e.KeyCode == Keys.F1)
            {
                if (CheckInMode == true)
                    OnlineHelp(this, "CheckLookup_In.htm");
                else
                    OnlineHelp(this, "CheckLookup_Out.htm");
            }
        }

        private void CheckLookup_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            Graphics formGraphics;

            // ----- Set the default height of items in the matching items listbox.
            formGraphics = this.CreateGraphics();
            MatchingItems.ItemHeight = (int)formGraphics.MeasureString(
                "A" + Environment.NewLine + "g", MatchingItems.Font).Height + 3;
            SecondItemRow = (int)formGraphics.MeasureString(
                "Ag", MatchingItems.Font).Height + 1;
            formGraphics.Dispose();

            // ----- Adjust the form title.
            if (CheckInMode == true)
                this.Text = "Check-In By Title";
            else
                this.Text = "Check-Out By Title";
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- Make sure a list item is selected.
            if (MatchingItems.SelectedIndex == -1)
            {
                MessageBox.Show("Please select an item from the list.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Continue with CheckItemByTitle()
            this.DialogResult = DialogResult.OK;
        }

        private void MatchingItems_DoubleClick(object sender, EventArgs e)
        {
            // ----- Same as a click on the OK button.
            ActOK.PerformClick();
        }
    }
}
