﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class PatronPayment : Form
    {
        private long ActivePatronCopyID;
        private int OneLineHeight;
        private int TwoLineHeight;
        private int SecondItemRow;
        private List<PaymentItem> PaymentsOnly;

        public PatronPayment()
        {
            InitializeComponent();
        }

        private enum EventEntryType: int
        {
            NotDefined,
            PatronPayment,
            FineAdded,
            FineDismissal,
            RefundToPatron,
            OverdueFines
        }

        private class EventHistoryItem
        {
            public long PaymentID;
            public DateTime EntryDate;
            public decimal PaymentAmount;
            public string Comments;
            public EventEntryType EntryType;
        }

        public void ManagePayments(long patronCopyID, List<PaymentItem> sessionPayments)
        {
            // ----- Manage the payments for an item.
            ActivePatronCopyID = patronCopyID;
            PaymentsOnly = sessionPayments;
            this.ShowDialog();
        }

        private void PatronPayment_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- F1 shows online help.
            if (e.KeyCode == Keys.F1)
                OnlineHelp(this, "PatronPayment.htm");
        }

        private void PatronPayment_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            decimal balanceDue;
            decimal originalFine;
            Graphics canvas;
            EventHistoryItem historyItem;

            // ----- Get the height of the history list lines.
            canvas = EventHistory.CreateGraphics();
            OneLineHeight = EventHistory.ItemHeight;
            TwoLineHeight = Convert.ToInt32(canvas.MeasureString(
                "A" + Environment.NewLine + "g", EventHistory.Font).Height) + 3;
            SecondItemRow = Convert.ToInt32(canvas.MeasureString(
                "Ag", EventHistory.Font).Height) + 1;
            canvas.Dispose();

            // ----- Retrieve the summary details for this patron/item entry.
            sqlText = "SELECT PA.FirstName, PA.LastName, NI.Title, PC.Fine, " +
                "PC.Paid FROM PatronCopy AS PC INNER JOIN Patron AS PA ON " +
                "PC.Patron = PA.ID INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " +
                "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " +
                "WHERE PC.ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActivePatronCopyID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("PatronPayment.PatronPayment_Load.Summary", ex);
                return;
            }
            oneRow = dbInfo.Rows[0];
            RecordPatron.Text = DBGetText(oneRow["FirstName"]) + " " +
                DBGetText(oneRow["LastName"]);
            RecordItem.Text = DBGetText(oneRow["Title"]);
            originalFine = DBGetDecimal(oneRow["Fine"]);
            RecordFine.Text = originalFine.ToString("C");
            RecordPayments.Text = DBGetDecimal(oneRow["Paid"]).ToString("C");
            balanceDue = originalFine - DBGetDecimal(oneRow["Paid"]);
            RecordBalance.Text = balanceDue.ToString("C");
            dbInfo.Dispose();

            // ----- Load in the history for this item.
            EventHistory.Items.Clear();
            sqlText = "SELECT * FROM PatronPayment " +
                "WHERE PatronCopy = @RecordID ORDER BY EntryDate";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActivePatronCopyID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("PatronPayment.PatronPayment_Load.History", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Build the history detail entry.
                historyItem = new EventHistoryItem();
                historyItem.PaymentID = DBGetLong(dbRow["ID"]);
                historyItem.EntryDate = DBGetDate(dbRow["EntryDate"]);
                historyItem.PaymentAmount = DBGetDecimal(dbRow["Amount"]);
                historyItem.Comments = DBGetText(dbRow["Comment"]);
                switch (DBGetText(dbRow["EntryType"]).ToUpper())
                {
                    // ----- The database stores letter codes.
                    case "D":
                        historyItem.EntryType = EventEntryType.FineDismissal;
                        originalFine += DBGetDecimal(dbRow["Amount"]);
                        break;
                    case "F":
                        historyItem.EntryType = EventEntryType.FineAdded;
                        originalFine -= DBGetDecimal(dbRow["Amount"]);
                        break;
                    case "P":
                        historyItem.EntryType = EventEntryType.PatronPayment;
                        break;
                    case "R":
                        historyItem.EntryType = EventEntryType.RefundToPatron;
                        break;
                }
                EventHistory.Items.Add(historyItem);
            }
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Add in the original fine if needed.
            if (originalFine > 0M)
            {
                historyItem = new EventHistoryItem();
                historyItem.PaymentID = -1L;
                historyItem.EntryDate = DateTime.MinValue;
                historyItem.PaymentAmount = originalFine;
                historyItem.Comments = "";
                historyItem.EntryType = EventEntryType.OverdueFines;
                EventHistory.Items.Insert(0, historyItem);
            }
        }

        private bool SaveEventData()
        {
            // ----- Add a new payment entry. Return True on success.
            EventEntryType entryType;
            string entryCode;
            decimal fineAmount;
            decimal paidAmount;
            decimal enteredAmount;
            string sqlText;
            DataCommand sqlRun;
            long newID;
            bool foundPayment;
            PaymentItem onePayment;
            EventHistoryItem historyItem;
            LibraryTransaction transInfo = null;

            // ----- Determine the type of entry.
            entryType = EventEntryType.NotDefined;
            if (OptionPayment.Checked == true)
            {
                entryType = EventEntryType.PatronPayment;
                entryCode = "P";
            }
            else if (OptionFine.Checked == true)
            {
                entryType = EventEntryType.FineAdded;
                entryCode = "F";
            }
            else if (OptionReduce.Checked == true)
            {
                entryType = EventEntryType.FineDismissal;
                entryCode = "D";
            }
            else
            {
                // ----- Refund entry (OptionRefund.Checked = True).
                entryType = EventEntryType.RefundToPatron;
                entryCode = "R";
            }

            // ----- Determine the new totals.
            fineAmount = decimal.Parse(RecordFine.Text,
                NumberStyles.AllowCurrencySymbol | NumberStyles.Number);
            paidAmount = decimal.Parse(RecordPayments.Text,
                NumberStyles.AllowCurrencySymbol | NumberStyles.Number);
            enteredAmount = decimal.Parse(RecordAmount.Text,
                NumberStyles.AllowCurrencySymbol | NumberStyles.Number);
            switch (entryType)
            {
                case EventEntryType.PatronPayment:
                    paidAmount += enteredAmount;
                    break;
                case EventEntryType.FineAdded:
                    fineAmount += enteredAmount;
                    break;
                case EventEntryType.FineDismissal:
                    fineAmount -= enteredAmount;
                    break;
                case EventEntryType.RefundToPatron:
                    paidAmount -= enteredAmount;
                    break;
            }
            this.Cursor = Cursors.WaitCursor;

            // ----- Add the entry to the database.
            try
            {
                transInfo = TransactionBegin();

                sqlText = "INSERT INTO PatronPayment (PatronCopy, EntryDate, EntryType, " +
                    "Amount, Comment, UserID) OUTPUT INSERTED.ID VALUES (" +
                    "@NewCopy, GETDATE(), @NewType, @NewAmount, @NewComment, @NewUser)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("NewCopy", ActivePatronCopyID);
                sqlRun.AddText("NewType", entryCode);
                sqlRun.AddDecimal("NewAmount", enteredAmount);
                sqlRun.AddText("NewComment", RecordComment.Text.Trim());
                sqlRun.AddLong("NewUser", LoggedInUserID);
                newID = DBGetLong(ExecuteSQLReturn(sqlRun, transInfo));

                sqlText = "UPDATE PatronCopy SET Fine = @NewFine, " +
                    "Paid = @NewPaid WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddDecimal("NewFine", fineAmount);
                sqlRun.AddDecimal("NewPaid", paidAmount);
                sqlRun.AddLong("RecordID", ActivePatronCopyID);
                ExecuteSQL(sqlRun, transInfo);

                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                if (transInfo != null)
                    TransactionRollback(transInfo);
                this.Cursor = Cursors.Default;
                GeneralError("PatronPayment.SaveEventData", ex);
                return false;
            }

            // ----- Add an item to the entry list.
            historyItem = new EventHistoryItem();
            historyItem.PaymentID = newID;
            historyItem.EntryDate = DateTime.Today;
            historyItem.PaymentAmount = Convert.ToDecimal(RecordAmount.Text);
            historyItem.Comments = RecordComment.Text.Trim();
            historyItem.EntryType = entryType;
            EventHistory.Items.Add(historyItem);

            // ----- Update the payment in the payments list.
            foundPayment = false;
            foreach (PaymentItem scanPayment in PaymentsOnly)
            {
                if (scanPayment.PatronCopyID == ActivePatronCopyID)
                {
                    // ----- Update an existing payment.
                    foundPayment = true;
                    if (entryType == EventEntryType.PatronPayment)
                        scanPayment.FeesPaid += paidAmount;
                    scanPayment.BalanceDue = fineAmount - paidAmount;
                    break;
                }
            }
            if ((foundPayment == false) & (entryType == EventEntryType.PatronPayment))
            {
                // ----- Add a new payment.
                onePayment = new PaymentItem();
                onePayment.PatronCopyID = ActivePatronCopyID;
                onePayment.ItemTitle = RecordItem.Text;
                onePayment.FeesPaid = paidAmount;
                onePayment.BalanceDue = fineAmount - paidAmount;
                PaymentsOnly.Add(onePayment);
            }

            // ----- Update the on-screen values.
            RecordFine.Text = fineAmount.ToString("C");
            RecordPayments.Text = paidAmount.ToString("C");
            RecordBalance.Text = (fineAmount - paidAmount).ToString("C");

            // ----- Success.
            this.Cursor = Cursors.Default;
            return true;
        }

        private bool VerifyEventData()
        {
            // ----- The user is trying to add a new event. Make sure all of the
            //       fields are valid. Return True on success.
            decimal holdValue;

            // ----- An entry type is required.
            if ((OptionPayment.Checked == false) & (OptionFine.Checked == false) &
                (OptionReduce.Checked == false) & (OptionRefund.Checked == false))
            {
                MessageBox.Show("Please indicate the type of payment entry.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            // ----- Some entry types require a comment.
            if (RecordComment.Text.Trim().Length == 0)
            {
                if ((OptionFine.Checked == true) | (OptionRefund.Checked == true))
                {
                    MessageBox.Show("Comment is required for this type of payment entry.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
            }

            // ----- Check for a valid amount.
            if (decimal.TryParse(RecordAmount.Text, out holdValue) == false)
            {
                MessageBox.Show("Please supply a valid amount.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordAmount.Focus();
                return false;
            }
            if ((holdValue <= 0M) | (holdValue >= 1000M))
            {
                MessageBox.Show("Amount must be between $0.01 and $999.99.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordAmount.Focus();
                return false;
            }

            // ----- Success.
            return true;
        }

        private void ActAdd_Click(object sender, EventArgs e)
        {
            // ----- Add a new payment entry.
            if (VerifyEventData() == false)
                return;
            if (SaveEventData() == false)
                return;

            // ----- Prepare for a new entry.
            RecordAmount.Text = "";
            RecordComment.Text = "";
            OptionPayment.Checked = false;
            OptionFine.Checked = false;
            OptionReduce.Checked = false;
            OptionRefund.Checked = false;
        }

        private void EventHistory_MeasureItem(object sender, MeasureItemEventArgs e)
        {
            // ----- Is the active item one or two lines high?
            EventHistoryItem historyItem;

            historyItem = (EventHistoryItem)(EventHistory.Items[e.Index]);
            if (historyItem.Comments.Length == 0)
            {
                // ----- One line.
                e.ItemHeight = OneLineHeight;
            }
            else
            {
                // ----- Two lines.
                e.ItemHeight = TwoLineHeight;
            }
        }

        private void EventHistory_DrawItem(object sender, DrawItemEventArgs e)
        {
            // ----- Draw the matching items on two lines.
            Brush useBrush;
            EventHistoryItem historyItem;
            string dateText;
            string amountText;
            string reasonText;
            string commentText;
            Font italicFont;

            // ----- Draw the background of the item.
            if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
                useBrush = SystemBrushes.HighlightText;
            else
                useBrush = SystemBrushes.WindowText;
            e.DrawBackground();

            // ----- Make an italic version of the font.
            italicFont = new Font(e.Font, FontStyle.Italic);

            // ----- Get the record that contains the display details.
            historyItem = (EventHistoryItem)(EventHistory.Items[e.Index]);

            // ----- Format the display components.
            if (historyItem.EntryDate == DateTime.MinValue)
                dateText = "--/--/----";
            else
                dateText = historyItem.EntryDate.ToString("M/d/yyyy");
            amountText = historyItem.PaymentAmount.ToString("C");
            switch (historyItem.EntryType)
            {
                case EventEntryType.PatronPayment:
                    reasonText = "Payment by patron";
                    break;
                case EventEntryType.FineAdded:
                    reasonText = "Fine added";
                    break;
                case EventEntryType.FineDismissal:
                    reasonText = "Dismissal of fine";
                    break;
                case EventEntryType.RefundToPatron:
                    reasonText = "Refund to patron";
                    break;
                case EventEntryType.OverdueFines:
                    reasonText = "Overdue fines";
                    break;
                default:
                    reasonText = "Undefined entry";
                    break;
            }
            if (historyItem.Comments.Length == 0)
                commentText = "";
            else
                commentText = ">> " + historyItem.Comments;

            // ----- Draw the text of the item.
            e.Graphics.DrawString(dateText, e.Font, useBrush, 0, e.Bounds.Top);
            e.Graphics.DrawString(amountText, e.Font, useBrush,
                LabelHistoryCol2.Left - EventHistory.Left + 2, e.Bounds.Top);
            e.Graphics.DrawString(reasonText, e.Font, useBrush,
                LabelHistoryCol3.Left - EventHistory.Left + 2, e.Bounds.Top);
            if (commentText.Length != 0)
                e.Graphics.DrawString(commentText, italicFont, useBrush,
                    LabelHistoryCol3.Left - EventHistory.Left + 2,
                    e.Bounds.Top + SecondItemRow);

            // ----- If the ListBox has focus, draw a focus rectangle.
            e.DrawFocusRectangle();
            italicFont.Dispose();
        }

        private void RecordAmount_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordAmount.SelectAll();
        }

        private void RecordComment_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordComment.SelectAll();
        }
    }
}
