﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Windows.Forms;
using System.Xml.Linq;
using static Library.General;

namespace Library
{
    public partial class ReportSelect : Form
    {
        public ReportSelect()
        {
            InitializeComponent();
        }

        private void LoadReportGroup(XElement groupNode, int indentLevel)
        {
            // ----- Add the groups and items at this level,
            //       and recurse as needed.
            XElement detailNode;
            ReportItem reportEntry;
            int holdValue;

            // ----- Process each item or group.
            foreach (XElement scanNode in groupNode.Nodes())
            {
                // ----- Build a content item for the list.
                reportEntry = new ReportItem();
                reportEntry.Indent = indentLevel;

                // ----- Get the display name.
                detailNode = (XElement)scanNode.Elements("displayText");
                if (detailNode == null)
                    continue;
                reportEntry.DisplayText = detailNode.Value;

                if (scanNode.Name == "reportGroup")
                {
                    // ----- Start a new display group.
                    reportEntry.ItemType = ReportItemEnum.GroupLabel;
                    AllReports.Items.Add(reportEntry);

                    // ----- Recurse to child items.
                    LoadReportGroup(scanNode, indentLevel + 1);
                }
                else if (scanNode.Name == "reportItem")
                {
                    // ----- This is an item. Record its location.
                    detailNode = (XElement)scanNode.Elements("reportPath");
                    if (detailNode != null)
                        reportEntry.ReportPath = detailNode.Value;

                    // ----- Get any command-line arguments.
                    detailNode = (XElement)scanNode.Elements("reportArgs");
                    if (detailNode != null)
                        reportEntry.ReportArgs = detailNode.Value;

                    // ----- Get any item-specific flags.
                    detailNode = (XElement)scanNode.Elements("reportFlags");
                    if (detailNode != null)
                    {
                        // ---- "U" adds "-u loginid" to the command.
                        if ((detailNode.Value.ToUpper().Contains("U") == true) &
                                (LoggedInUserName.Length != 0))
                            reportEntry.ReportArgs = (reportEntry.ReportArgs +
                                " -u " + LoggedInUserName).Trim();
                    }

                    // ----- Store the full description.
                    detailNode = (XElement)scanNode.Elements("description");
                    if (detailNode != null)
                        reportEntry.Description = detailNode.Value;

                    // ----- So, what type of entry is it?
                    if (scanNode.Attributes("type").ToString() == "built-in")
                    {
                        // ----- Built-in program. Check for valid ID.
                        if (int.TryParse(reportEntry.ReportPath, out holdValue) == false)
                            holdValue = 0;
                        if ((holdValue < 1) | (holdValue > (int)ReportItemEnum.BuiltInStatistics))
                            continue;
                        reportEntry.ItemType = (ReportItemEnum)Convert.ToInt32(reportEntry.ReportPath);
                        if (reportEntry.Description.Length == 0)
                            reportEntry.Description = GetBuiltinReportDescription(reportEntry.ItemType);
                        AllReports.Items.Add(reportEntry);
                    }
                    else if (scanNode.Attributes("type").ToString() == "program")
                    {
                        // ----- EXE program-based report.
                        if (reportEntry.ReportPath.Length == 0)
                            continue;
                        reportEntry.ItemType = ReportItemEnum.ExeProgram;
                        AllReports.Items.Add(reportEntry);
                    }
                    else if (scanNode.Attributes("type").ToString() == "url")
                    {
                        // ----- URL-based report.
                        if (reportEntry.ReportPath.Length == 0)
                            continue;
                        reportEntry.ItemType = ReportItemEnum.UrlProgram;
                        AllReports.Items.Add(reportEntry);
                    }
                }
            }
        }

        private void RefreshReportList()
        {
            // ----- Load in the list of available reports.
            string configFile;
            XDocument configData;
            ReportItem reportEntry;

            // ----- Clear the existing list.
            AllReports.Items.Clear();

            // ----- Get the location of the configuration file.
            configFile = Properties.Settings.Default.ReportConfig + "";

            // ----- Load the configuration file.
            if (configFile.Length != 0)
            {
                if (System.IO.File.Exists(configFile))
                {
                    try
                    {
                        // ----- Load in the file.
                        configData = XDocument.Load(configFile);

                        // ----- Process the configuration file.
                        LoadReportGroup((XElement)configData.FirstNode, 0);
                    }
                    catch (Exception ex)
                    {
                        GeneralError("ReportSelect.RefreshReportList", ex);
                    }
                }
            }

            // ----- If the configuration file resulted in no reports
            //       appearing in the list, add the default reports.
            if (AllReports.Items.Count == 0)
            {
                for (int counter = 1; counter <= Convert.ToInt32(
                    ReportItemEnum.BuiltInStatistics); counter++)
                {
                    // ----- Build the report entry.
                    reportEntry = new ReportItem();
                    reportEntry.Indent = 0;
                    reportEntry.ItemType = (ReportItemEnum)counter;
                    switch (reportEntry.ItemType)
                    {
                        case ReportItemEnum.BuiltInCheckedOut:
                            reportEntry.DisplayText = "Items Checked Out";
                            break;
                        case ReportItemEnum.BuiltInOverdue:
                            reportEntry.DisplayText = "Items Overdue";
                            break;
                        case ReportItemEnum.BuiltInMissing:
                            reportEntry.DisplayText = "Items Missing";
                            break;
                        case ReportItemEnum.BuiltInFinesOwed:
                            reportEntry.DisplayText = "Patron Fines Owed";
                            break;
                        case ReportItemEnum.BuiltInStatistics:
                            reportEntry.DisplayText = "Database Statistics";
                            break;
                    }
                    reportEntry.Description = GetBuiltinReportDescription(reportEntry.ItemType);

                    // ----- Add the report entry to the list.
                    AllReports.Items.Add(reportEntry);
                }
            }
        }

        private void AllReports_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Display a description of the report, if available.
            ReportItem reportEntry;

            // ----- Clear any previous description.
            FullDescription.Text = "No report selected.";
            if (AllReports.SelectedIndex != -1)
            {
                // ----- Locate the content and display it.
                reportEntry = (ReportItem)AllReports.SelectedItem;
                FullDescription.Text = reportEntry.Description;
            }
        }

        private void ReportSelect_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- F1 shows online help.
            if (e.KeyCode == Keys.F1)
                OnlineHelp(this, "ReportSelect.htm");
        }

        private void ReportSelect_Load(object sender, EventArgs e)
        {
            // ----- Display the list of reports.
            RefreshReportList();
        }

        private void ActRun_Click(object sender, EventArgs e)
        {
            // ----- Run the selected report.
            ReportItem reportEntry;

            // ----- Make sure a report is selected.
            if (AllReports.SelectedIndex == -1)
            {
                MessageBox.Show("Please select a report from the list.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Different code for each type of entry.
            reportEntry = (ReportItem)AllReports.SelectedItem;
            this.Cursor = Cursors.WaitCursor;
            switch (reportEntry.ItemType)
            {
                case ReportItemEnum.GroupLabel:
                    // ----- No report for group entries.
                    MessageBox.Show("Please select a report from the list.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    break;
                case ReportItemEnum.BuiltInCheckedOut:
                    // ----- Items Checked Out
                    BasicReportCheckedOut();
                    break;
                case ReportItemEnum.BuiltInOverdue:
                    // ----- Items Overdue
                    BasicReportOverdue();
                    break;
                case ReportItemEnum.BuiltInMissing:
                    // ----- Items Missing
                    BasicReportMissing();
                    break;
                case ReportItemEnum.BuiltInFinesOwed:
                    // ----- Fines Owed by Patrons
                    BasicReportFines();
                    break;
                case ReportItemEnum.BuiltInStatistics:
                    // ----- Library Database Statistics
                    BasicReportStatistics();
                    break;
                case ReportItemEnum.ExeProgram:
                    // ----- Start a program.
                    try
                    {
                        Process.Start("\"" + reportEntry.ReportPath +
                            "\" " + reportEntry.ReportArgs);
                    }
                    catch (Exception ex)
                    {
                        GeneralError("ReportSelect.ActRun_Click.Exe", ex);
                    }
                    break;
                case ReportItemEnum.UrlProgram:
                    // ----- Start a URL.
                    Process.Start(reportEntry.ReportPath);
                    break;
            }
            this.Cursor = Cursors.Default;
        }

        private void BasicReportCheckedOut()
        {
            // ----- Run built-in report #1: Items checked out report.
            string sqlText;
            DataTable reportData;
            ReportBuiltinViewer reportForm;

            // ----- Retrieve the data as a dataset.
            sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " +
                "PA.Barcode AS PatronBarcode, " +
                "PC.DueDate, IC.CopyNumber, IC.Barcode AS ItemBarcode, " +
                "NI.Title, CMT.FullName AS MediaName " +
                "FROM Patron AS PA " +
                "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " +
                "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " +
                "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " +
                "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " +
                "WHERE PC.Returned = 0 " +
                "AND PC.Missing = 0 " +
                "AND IC.Missing = 0 " +
                "ORDER BY NI.Title, IC.CopyNumber, PA.LastName, PA.FirstName";
            try
            {
                reportData = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("ReportSelect.BasicReportCheckedOut.Retrieve", ex);
                return;
            }

            // ----- Check for no data.
            if (reportData.Rows.Count == 0)
            {
                reportData.Dispose();
                MessageBox.Show("No items are checked out.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Send the data to the report.
            reportForm = new ReportBuiltinViewer();
            try
            {
                reportForm.StartReport("Library.ReportCheckedOut.rdlc",
                    "Library_ReportSchemaPatronItems", reportData);
            }
            catch (Exception ex)
            {
                GeneralError("ReportSelect.BasicReportCheckedOut.Start", ex);
            }
        }

        private void BasicReportOverdue()
        {
            // ----- Run built-in report #2: Items overdue report.
            string sqlText;
            DataTable reportData;
            ReportBuiltinViewer reportForm;

            // ----- Retrieve the data as a dataset.
            sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " +
                "PA.Barcode AS PatronBarcode, " +
                "PC.DueDate, IC.CopyNumber, IC.Barcode AS ItemBarcode, " +
                "NI.Title, CMT.FullName AS MediaName " +
                "FROM Patron AS PA " +
                "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " +
                "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " +
                "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " +
                "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " +
                "WHERE PC.Returned = 0 " +
                "AND PC.Missing = 0 " +
                "AND IC.Missing = 0 " +
                "AND PC.DueDate < GETDATE() " +
                "ORDER BY NI.Title, IC.CopyNumber, PA.LastName, PA.FirstName";
            try
            {
                reportData = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("ReportSelect.BasicReportOverdue.Retrieve", ex);
                return;
            }

            // ----- Check for no data.
            if (reportData.Rows.Count == 0)
            {
                reportData.Dispose();
                MessageBox.Show("No items are overdue.", ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Send the data to the report.
            reportForm = new ReportBuiltinViewer();
            try
            {
                reportForm.StartReport("Library.ReportOverdue.rdlc",
                    "Library_ReportSchemaPatronItems", reportData);
            }
            catch (Exception ex)
            {
                GeneralError("ReportSelect.BasicReportOverdue.Start", ex);
            }
        }

        private void BasicReportMissing()
        {
            // ----- Run built-in report #3: Items missing report.
            string sqlText;
            DataTable reportData;
            ReportBuiltinViewer reportForm;

            // ----- Retrieve the data as a dataset.
            sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " +
                "PA.Barcode AS PatronBarcode, " +
                "PC.DueDate, IC.CopyNumber, IC.Barcode AS ItemBarcode, " +
                "NI.Title, CMT.FullName AS MediaName " +
                "FROM Patron AS PA " +
                "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " +
                "INNER JOIN ItemCopy AS IC ON PC.ItemCopy = IC.ID " +
                "INNER JOIN NamedItem AS NI ON IC.ItemID = NI.ID " +
                "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " +
                "WHERE PC.Missing = 1 " +
                "OR IC.Missing = 1 " +
                "ORDER BY NI.Title, IC.CopyNumber, PA.LastName, PA.FirstName";
            try
            {
                reportData = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("ReportSelect.BasicReportMissing.Retrieve", ex);
                return;
            }

            // ----- Check for no data.
            if (reportData.Rows.Count == 0)
            {
                reportData.Dispose();
                MessageBox.Show("No items are missing.", ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Send the data to the report.
            reportForm = new ReportBuiltinViewer();
            try
            {
                reportForm.StartReport("Library.ReportMissing.rdlc",
                    "Library_ReportSchemaPatronItems", reportData);
            }
            catch (Exception ex)
            {
                GeneralError("ReportSelect.BasicReportMissing.Start", ex);
            }
        }

        private void BasicReportFines()
        {
            // ----- Run built-in report #4: Fines owed by patrons report.
            string sqlText;
            DataTable reportData;
            ReportBuiltinViewer reportForm;

            // ----- Retrieve the data as a dataset.
            sqlText = "SELECT PA.LastName + ', ' + PA.FirstName AS PatronName, " +
                "PA.Barcode AS PatronBarcode, " +
                "SUM(PC.Fine - PC.Paid) AS FinesDue " +
                "FROM Patron AS PA " +
                "INNER JOIN PatronCopy AS PC ON PA.ID = PC.Patron " +
                "GROUP BY PA.LastName + ', ' + PA.FirstName, PA.Barcode " +
                "HAVING SUM(PC.Fine - PC.Paid) > 0 " +
                "ORDER BY PatronName";
            try
            {
                reportData = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("ReportSelect.BasicReportFines.Retrieve", ex);
                return;
            }

            // ----- Check for no data.
            if (reportData.Rows.Count == 0)
            {
                reportData.Dispose();
                MessageBox.Show("There are no patrons with unpaid fines.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Send the data to the report.
            reportForm = new ReportBuiltinViewer();
            try
            {
                reportForm.StartReport("Library.ReportPatronFines.rdlc",
                    "Library_ReportSchemaPatronFines", reportData);
            }
            catch (Exception ex)
            {
                GeneralError("ReportSelect.BasicReportFines.Start", ex);
            }
        }

        private void BasicReportStatistics()
        {
            // ----- Run built-in report #5: Library database statistics report.
            string sqlText;
            List<ReportSchemaStatistics> reportData;
            ReportSchemaStatistics oneEntry;
            ReportBuiltinViewer reportForm;
            int resultValue;
            const string tableSets = "Author,Publisher,Subject," +
                "NamedItem,ItemCopy,Patron";
            const string tableTitles = "Authors,Publishers,Subject Headings," +
                "Items,Item Copies,Patrons";

            // ----- Build the report data. It's all counts from different tables.
            reportData = new List<ReportSchemaStatistics>();
            for (int counter = 1; counter <= CountSubStr(tableSets, ",") + 1; counter++)
            {
                // ----- Process one table.
                sqlText = "SELECT COUNT(*) FROM " +
                    tableSets.GetSubStr(",", counter);
                try
                {
                    resultValue = DBGetInteger(ExecuteSQLReturn(sqlText));
                }
                catch (Exception ex)
                {
                    GeneralError("ReportSelect.BasicReportStatistics." +
                        tableSets.GetSubStr(",", counter), ex);
                    return;
                }

                // ----- Add it to the report data.
                oneEntry = new ReportSchemaStatistics();
                oneEntry.EntryName = tableTitles.GetSubStr(",", counter);
                oneEntry.EntryValue = resultValue.ToString();
                reportData.Add(oneEntry);
            }

            // ----- Send the data to the report.
            reportForm = new ReportBuiltinViewer();
            try
            {
                reportForm.StartReport("Library.ReportStatistics.rdlc",
                    "Library_ReportSchemaStatistics", reportData);
            }
            catch (Exception ex)
            {
                GeneralError("ReportSelect.BasicReportStatistics.Start", ex);
            }
        }

        private string GetBuiltinReportDescription(ReportItemEnum whichReport)
        {
            // ----- Return a predefined description for the built-in reports.
            switch (whichReport)
            {
                case ReportItemEnum.BuiltInCheckedOut:
                    return "Displays all items currently checked out, sorted by name.";
                case ReportItemEnum.BuiltInOverdue:
                    return "Displays all overdue items, sorted by name.";
                case ReportItemEnum.BuiltInMissing:
                    return "Displays all missing items, sorted by name.";
                case ReportItemEnum.BuiltInFinesOwed:
                    return "Displays all unpaid fines owed by patrons, " + "sorted by patron name.";
                case ReportItemEnum.BuiltInStatistics:
                    return "Displays some record counts from the Library database.";
                default:
                    return "There is no description for this report.";
            }
        }
    }
}
