﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class Author : Library.BaseCodeForm
    {
        private long ActiveID;
        private bool StartingOver;

        public Author()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            ActiveID = -1L;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public long AddRecord(string lastText, string firstText)
        {
            // ----- Add a new record, but use a starting value previously entered
            //       by the user.
            ActiveID = -1L;
            PrepareFormFields();
            RecordLastName.Text = lastText;
            RecordFirstName.Text = firstText;
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManageAuthors];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManageAuthors];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManageAuthors];
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the records.
            string sqlText;
            DataCommand sqlRun;

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete the author?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                    DialogResult.Yes)
                return false;

            // ----- Make sure this record is not in use.
            sqlText = "SELECT COUNT(*) FROM ItemAuthor WHERE AuthorID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    MessageBox.Show("You cannot delete this record because it is in use.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("Author.DeleteRecord.Check", ex);
                return false;
            }

            // ----- Delete the record.
            sqlText = "DELETE FROM Author WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Author.DeleteRecord.Delete", ex);
                return false;
            }
            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record.
            ActiveID = recordID;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items. Prompt the
            //       user first.
            destList.Items.Clear();
            SearchForRecord(destList, ref exceededMatches);
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            string result;

            // ----- Initialize.
            result = "";

            // ----- Retrieve the value from the database.
            sqlText = "SELECT * FROM Author WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Author.FormatRecordName", ex);
                return "Error";
            }
            if (dbInfo.Rows.Count > 0)
            {
                // ----- Format the display name.
                result = dbInfo.Rows[0].FormatAuthorName();
            }
            dbInfo.Dispose();
            dbInfo = null;
            return result;
        }

        public override string GetEditDescription()
        {
            // ----- Return the list form display title.
            return "Use this form to enter authors, editors, performers, and " +
                "others that are primarily responsible for the content of an item " +
                "in the library.";
        }

        public override string GetEditTitle()
        {
            // ----- Return the list form display title.
            return "Authors and Names";
        }

        private void PrepareFormFields()
        {
            // ----- Fill in the data.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow dbRow;

            // ----- Clear the existing data.
            StartingOver = true;
            RecordLastName.Text = "";
            RecordFirstName.Text = "";
            RecordMiddleName.Text = "";
            RecordSuffix.Text = "";
            RecordBirthYear.Text = "";
            RecordDeathYear.Text = "";
            RecordComments.Text = "";
            ShowMatches.Text = "      None";
            ShowMatches.ImageIndex = MatchNone;
            LabelMatches.Visible = true;
            ShowMatches.Visible = true;
            if (ActiveID == -1L)
                return;

            // ----- Load in the values stored in the database.
            sqlText = "SELECT * FROM Author WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Author.PrepareFormFields", ex);
                return;
            }
            if (dbInfo.Rows.Count > 0)
            {
                dbRow = dbInfo.Rows[0];
                RecordLastName.Text = DBGetText(dbRow["LastName"]);
                RecordFirstName.Text = DBGetText(dbRow["FirstName"]);
                RecordMiddleName.Text = DBGetText(dbRow["MiddleName"]);
                RecordSuffix.Text = DBGetText(dbRow["Suffix"]);
                if (Convert.IsDBNull(dbRow["BirthYear"]) == false)
                    RecordBirthYear.Text = DBGetInteger(dbRow["BirthYear"]).ToString();
                if (Convert.IsDBNull(dbRow["DeathYear"]) == false)
                    RecordDeathYear.Text = DBGetInteger(dbRow["DeathYear"]).ToString();
                RecordComments.Text = DBGetText(dbRow["Comments"]);
            }
            dbInfo.Dispose();
            dbInfo = null;
            LabelMatches.Visible = false;
            ShowMatches.Visible = false;
        }

        private void RefreshMatchingAuthors()
        {
            // ----- See if there are name matches for this author.
            string sqlText;
            DataCommand sqlRun;
            int matchCount;
            string messageText;

            // ----- Only performed for new records.
            if (ActiveID != -1L)
                return;

            // ----- Reset the display.
            ShowMatches.Text = "      None";
            ShowMatches.ImageIndex = MatchNone;

            // ----- Ignore if we have insufficient data.
            if (RecordLastName.Text.Trim().Length == 0)
                return;

            // ----- Retrieve a count of matching records.
            sqlText = "SELECT COUNT(*) AS TheCount FROM Author " +
                "WHERE LastName LIKE @TestLast";
            if (RecordFirstName.Text.Trim().Length != 0)
                sqlText += " AND FirstName LIKE @TestFirst";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestLast", RecordLastName.Text.Trim());
            if (RecordFirstName.Text.Trim().Length != 0)
                sqlRun.AddText("TestFirst",
                    RecordFirstName.Text.Trim().Substring(0, 1) + "%");
            try
            {
                matchCount = DBGetInteger(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("Author.RefreshMatchingAuthors", ex);
                return;
            }
            if (matchCount > 0)
            {
                // ----- There are matches.
                if (matchCount == 1)
                    messageText = "1 match";
                else
                    messageText = matchCount.ToString() + " matches";
                messageText += ". Click for details.";
                ShowMatches.Text = "      " + messageText;
                ShowMatches.ImageIndex = MatchPresent;
            }
        }

        public override void SearchForRecord(ListBox destList, ref bool exceededMatches)
        {
            // ----- Search for authors using user-specified criteria.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            string userLimit;
            string limitFirst;
            string limitLast;
            bool hasWhere;
            int resultCount;

            // ----- Prompt the user for the limited author name.
            exceededMatches = false;
            userLimit = (new AuthorLimit()).PromptUser();
            if (userLimit.Length == 0)
                return;

            // ----- Use the limits to help prepare the search text.
            limitLast = userLimit.GetSubStr(",", 1).Trim();
            limitFirst = userLimit.GetSubStr(",", 2).Trim();
            if ((limitLast + limitFirst).Length == 0)
                return;
            if (limitLast.Contains("*") == false)
                limitLast += "*";
            if (limitFirst.Contains("*") == false)
                limitFirst += "*";
            limitLast = limitLast.Replace("*", "%");
            limitFirst = limitFirst.Replace("*", "%");

            // ----- Clear any existing values.
            destList.Items.Clear();

            // ----- Prepare the full SQL statement based on the user's limits.
            hasWhere = false;
            sqlText = "SELECT * FROM Author";
            if (limitLast.Length != 0)
            {
                sqlText += " WHERE LastName LIKE @TestLast";
                hasWhere = true;
            }
            if (limitFirst.Length != 0)
            {
                if (hasWhere)
                    sqlText += " AND";
                else
                    sqlText += " WHERE";
                sqlText += " FirstName LIKE @TestFirst";
            }
            sqlText += " ORDER BY LastName, FirstName";

            // ----- Retrieve and process the values from the database.
            sqlRun = new DataCommand(sqlText);
            if (limitLast.Length != 0)
                sqlRun.AddText("TestLast", limitLast);
            if (limitFirst.Length != 0)
                sqlRun.AddText("TestFirst", limitFirst);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Author.SearchForRecord", ex);
                return;
            }

            resultCount = 0;
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add this item.
                resultCount += 1;
                if (resultCount > SearchMatchLimit)
                {
                    // ----- Too many matches found.
                    exceededMatches = true;
                    break;
                }
                destList.Items.Add(new ListItemData(
                    dbRow.FormatAuthorName(), DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override bool UsesSearch()
        {
            // ----- Author records are searchable.
            return true;
        }

        private bool SaveFormData()
        {
            // ----- Save the record to the database.
            string sqlText;
            DataCommand sqlRun;
            long newID;

            this.Cursor = Cursors.WaitCursor;

            // ----- Save the data.
            if (ActiveID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO Author (LastName, FirstName, MiddleName, " +
                    "Suffix, BirthYear, DeathYear, Comments) OUTPUT INSERTED.ID " +
                    "VALUES (@NewLast, @NewFirst, @NewMiddle, @NewSuffix, " +
                    "@NewBirth, @NewDeath, @NewComments)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewLast", RecordLastName.Text.Trim());
                sqlRun.AddText("NewFirst", RecordFirstName.Text.Trim());
                sqlRun.AddText("NewMiddle", RecordMiddleName.Text.Trim());
                sqlRun.AddText("NewSuffix", RecordSuffix.Text.Trim());
                sqlRun.AddInteger("NewBirth", RecordBirthYear.Text);
                sqlRun.AddInteger("NewDeath", RecordDeathYear.Text);
                sqlRun.AddText("NewComments", RecordComments.Text.Trim());
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    GeneralError("Author.SaveFormData.Add", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
                ActiveID = newID;
            }
            else
            {
                // ----- Update the existing entry.
                sqlText = "UPDATE Author SET LastName = @NewLast, " +
                    "FirstName = @NewFirst, MiddleName = @NewMiddle, " +
                    "Suffix = @NewSuffix, BirthYear = @NewBirth, " +
                    "DeathYear = @NewDeath, Comments = @NewComments " +
                    "WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewLast", RecordLastName.Text.Trim());
                sqlRun.AddText("NewFirst", RecordFirstName.Text.Trim());
                sqlRun.AddText("NewMiddle", RecordMiddleName.Text.Trim());
                sqlRun.AddText("NewSuffix", RecordSuffix.Text.Trim());
                sqlRun.AddInteger("NewBirth", RecordBirthYear.Text);
                sqlRun.AddInteger("NewDeath", RecordDeathYear.Text);
                sqlRun.AddText("NewComments", RecordComments.Text.Trim());
                sqlRun.AddLong("RecordID", ActiveID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("Author.SaveFormData.Update", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }

            // ----- Success.
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Make sure the user supplied valid data. Return True on success.

            // ----- Last name is a required field.
            if (RecordLastName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the last name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordLastName.Focus();
                return false;
            }

            // ----- Birth year, if supplied, must be valid.
            if (RecordBirthYear.Text.Trim().Length != 0)
            {
                if (CheckInt(RecordBirthYear.Text) == false)
                {
                    MessageBox.Show("Year of Birth, if supplied, must be a valid integer.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordBirthYear.Focus();
                    return false;
                }
            }

            // ----- Death year, if supplied, must be valid.
            if (RecordDeathYear.Text.Trim().Length != 0)
            {
                if (CheckInt(RecordDeathYear.Text) == false)
                {
                    MessageBox.Show("Year of Death, if supplied, must be a valid integer.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordDeathYear.Focus();
                    return false;
                }
            }

            // ----- Check the matching authors, just in case.
            if (ActiveID == -1L)
            {
                RefreshMatchingAuthors();
                if (ShowMatches.ImageIndex == MatchPresent)
                {
                    if (MessageBox.Show("The name you supplied is similar to one or more names " +
                            "already in the database. Do you still want to save this record?",
                            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                            DialogResult.Yes)
                        return false;
                }
            }

            // ----- Success.
            return true;
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void Author_Activated(object sender, EventArgs e)
        {
            // ----- Return the focus to the first field.
            if (StartingOver)
                RecordLastName.Focus();
            StartingOver = false;
        }

        private void RecordLastName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordLastName.SelectAll();
        }

        private void RecordFirstName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordFirstName.SelectAll();
        }

        private void RecordMiddleName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordMiddleName.SelectAll();
        }

        private void RecordSuffix_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordSuffix.SelectAll();
        }

        private void RecordBirthYear_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordBirthYear.SelectAll();
        }

        private void RecordDeathYear_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordDeathYear.SelectAll();
        }

        private void RecordBirthYear_KeyPress(object sender, KeyPressEventArgs e)
        {
            // ----- Only allow digits, hyphens, and backspaces.
            if ((e.KeyChar.ToString() == "\b") | (Char.IsDigit(e.KeyChar)) |
                    (e.KeyChar == '-'))
                return;
            e.Handled = true;
        }

        private void RecordDeathYear_KeyPress(object sender, KeyPressEventArgs e)
        {
            // ----- Only allow digits, hyphens, and backspaces.
            if ((e.KeyChar.ToString() == "\b") | (Char.IsDigit(e.KeyChar)) |
                    (e.KeyChar == '-'))
                return;
            e.Handled = true;
        }

        private void RecordLastName_Leave(object sender, EventArgs e)
        {
            // ----- Update the matching display.
            RefreshMatchingAuthors();
        }

        private void RecordFirstName_Leave(object sender, EventArgs e)
        {
            // ----- Update the matching display.
            RefreshMatchingAuthors();
        }

        private void ShowMatches_Click(object sender, EventArgs e)
        {
            // ----- The user wants more information on the matches.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            string nameList;
            string message;
            int matches;
            int counter;

            // ----- Only performed for new records.
            if (ActiveID != -1L)
                return;

            // ----- Ignore if there are no matches.
            if (ShowMatches.ImageIndex == MatchNone)
            {
                MessageBox.Show("No matches.", ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            // ----- Retrieve a count of matching records.
            if (int.TryParse(ShowMatches.Text, out matches) == false)
                matches = 0;

            // ----- Retrieve the matching records.
            sqlText = "SELECT * FROM Author WHERE LastName LIKE @TestLast";
            if (RecordFirstName.Text.Trim().Length != 0)
                sqlText += " AND FirstName LIKE @TestFirst";
            sqlText += " ORDER BY LastName, FirstName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestLast", RecordLastName.Text.Trim());
            if (RecordFirstName.Text.Trim().Length != 0)
                sqlRun.AddText("TestFirst",
                    RecordFirstName.Text.Trim().Substring(0, 1) + "%");
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Author.ShowMatches_Click", ex);
                return;
            }

            counter = 0;
            nameList = "";
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add this author.
                nameList += Environment.NewLine + "\t" + dbRow.FormatAuthorName();

                // ----- Limit to ten matches.
                counter += 1;
                if (counter >= 10)
                    break;
            }
            dbInfo.Dispose();
            dbInfo = null;

            // ----- Format and display the message.
            message = "The database contains " + matches.ToString() + " match";
            if (matches > 1)
                message += "es";
            message += " to the name:" + Environment.NewLine +
                Environment.NewLine + "\t" + RecordLastName.Text.Trim();
            if (RecordFirstName.Text.Trim().Length != 0)
                message += ", " + RecordFirstName.Text.Trim();
            if (matches > 10)
                message += Environment.NewLine + Environment.NewLine +
                    "The first ten matches are:";
            else
                message += Environment.NewLine + Environment.NewLine +
                    "The matches are:";
            message += Environment.NewLine + nameList;
            MessageBox.Show(message, ProgramTitle,
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void LabelMatches_Click(object sender, EventArgs e)
        {
            // ----- Same as a click on the related data label.
            ShowMatches_Click(sender, e);
        }

        private void Author_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- F1 shows online help.
            if (e.KeyCode == Keys.F1)
                OnlineHelp(this, "Author.htm");
        }
    }
}
