﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Reflection;
using System.Security.Cryptography;
using System.Text;
using System.Windows.Forms;

namespace Library
{
    internal static class General
    {
        // ----- Public constants.
        public const string ProgramTitle = "The Library Project";
        public const string NotAuthorizedMessage = "You are not authorized to perform this task.";
        public const int UseDBVersion = 1;

        // ----- Constants for the MatchingImages image list.
        public const int MatchPresent = 0;
        public const int MatchNone = 1;

        public enum LookupMethods: int
        {
            ByTitle = 1,
            ByAuthor = 2,
            BySubject = 3,
            ByKeywordAny = 4,
            ByKeywordAll = 5,
            ByPublisher = 6,
            BySeries = 7,
            BySeriesID = 8,
            ByBarcode = 9,
            ByDatabaseID = 10,
            ByAuthorID = 11,
            BySubjectID = 12,
            ByPublisherID = 13
        }

        // ----- Security values.
        public enum LibrarySecurity: int
        {
            ManageAuthors = 1,
            ManageAuthorTypes = 2,
            ManageCopyStatus = 3,
            ManageMediaTypes = 4,
            ManageSeries = 5,
            ManageGroups = 6,
            ManageItems = 7,
            ManagePatrons = 8,
            ManagePublishers = 9,
            ManageValues = 10,
            ManageUsers = 11,
            ProcessFees = 12,
            ManageLocations = 13,
            CheckOutItems = 14,
            CheckInItems = 15,
            AdminFeatures = 16,
            DailyProcessing = 17,
            RunReports = 18,
            PatronOverride = 19,
            ManageBarcodeTemplates = 20,
            ManageHolidays = 21,
            ManagePatronGroups = 22,
            ViewAdminPatronMessages = 23
        }
        public const LibrarySecurity MaxLibrarySecurity = LibrarySecurity.ViewAdminPatronMessages;

        // ----- Global variables.
        public static long LoggedInUserID;
        public static string LoggedInUserName;
        public static long LoggedInGroupID;
        public static bool[] SecurityProfile = new bool[(int)MaxLibrarySecurity + 1];
        public static long DefaultItemLocation;
        public static int SearchMatchLimit;
        public static string MainHelpFile;
        public static string MainAdminHelpFile;
        public static int FineGraceDays;
        public static List<HolidaySet> AllHolidays;

        private static string ActiveConnectionString = "";

        public static FontStyle BuildFontStyle(string textStyle)
        {
            // ----- Given a string with the possible letters "BIUK", convert it to a
            //       font style object with Bold, Italic, Underline, or Strikeout.
            //       See the DBFontStyle function for the reverse of this function.
            FontStyle resultStyle = new FontStyle();

            // ----- Process the string.
            resultStyle = FontStyle.Regular;
            if (textStyle.ToUpper().Contains("B") == true)
                resultStyle = resultStyle | FontStyle.Bold;
            if (textStyle.ToUpper().Contains("I") == true)
                resultStyle = resultStyle | FontStyle.Italic;
            if (textStyle.ToUpper().Contains("U") == true)
                resultStyle = resultStyle | FontStyle.Underline;
            if (textStyle.ToUpper().Contains("K") == true)
                resultStyle = resultStyle | FontStyle.Strikeout;
            return resultStyle;
        }

        public static string CenterText(string origText, int textWidth)
        {
            // ----- Center a piece of text in a field width. If the text is too
            //       wide, truncate it.
            string resultText;

            resultText = origText.Trim();
            if (resultText.Length >= textWidth)
            {
                // ----- Truncate as needed.
                return origText.Substring(0, textWidth).Trim();
            }
            else
            {
                // ----- Start with extra spaces.
                return new string(' ',
                    (textWidth - origText.Length) / 2) + resultText;
            }
        }

        public static bool ConfirmDefaultLocation()
        {
            // ----- Return true if a default location is defined.
            string locationID;

            // ----- Obtain the default location if needed.
            if (DefaultItemLocation == -1L)
            {
                locationID = GetSystemValue("DefaultLocation");
                if (locationID.Length == 0)
                    locationID = "-1";
                if (long.TryParse(locationID, out DefaultItemLocation) == false)
                    DefaultItemLocation = -1L;
            }

            // ----- Return the status result.
            if (DefaultItemLocation == -1L)
            {
                MessageBox.Show("You must first define the default item location.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
            else
            {
                return true;
            }
        }

        public static bool CheckDateTime(string content)
        {
            // ----- Return True if a string contains a valid DateTime.
            DateTime result;

            return DateTime.TryParse(content, out result);
        }

        public static bool CheckDecimal(string content)
        {
            // ----- Return True if a string contains a valid decimal.
            decimal result;

            return decimal.TryParse(content, out result);
        }

        public static bool CheckInt(string content)
        {
            // ----- Return True if a string contains a valid int.
            int result;

            return int.TryParse(content, out result);
        }

        public static bool CheckLong(string content)
        {
            // ----- Return True if a string contains a valid long.
            long result;

            return long.TryParse(content, out result);
        }

        public static SqlConnection ConnectDatabase()
        {
            // ----- Connect to the database. Throw exception on failure.
            SqlConnection libraryDB;
            bool configChanged;

            // ----- Initialize.
            configChanged = false;

            // ----- Obtain the connection string.
            if (ActiveConnectionString.Length == 0)
            {
                if ((Properties.Settings.Default.DBConnection + "").Trim().Length == 0)
                {
                    // ----- Inform the user about the need to configure the database.
                    if (MessageBox.Show("This copy of the application has not been configured to " +
                            "connect to the library database. If you know the database settings, " +
                            "you can configure it now. Would you like to proceed?", ProgramTitle,
                            MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
                        return null;

                    // ----- Prompt for the new connection details.
                    ActiveConnectionString = (new LocateDatabase()).PromptUser();
                    if (ActiveConnectionString.Length == 0)
                        return null;
                    configChanged = true;
                }
                else
                {
                    ActiveConnectionString = Properties.Settings.Default.DBConnection;
                }
            }

    TryConnectingAgain:
            // ----- Attempt to open the database.
            try
            {
                libraryDB = new SqlConnection(ActiveConnectionString);
                libraryDB.Open();
            }
            catch
            {
                // ----- Perhaps it is just a configuration issue.
                if (MessageBox.Show("The connection to the database failed, possibly due to " +
                        "invalid configuration settings. Would you like to change the " +
                        "database configuration at this time?", ProgramTitle, MessageBoxButtons.YesNo,
                        MessageBoxIcon.Question) != DialogResult.Yes)
                    throw;

                // ----- Prompt for new details.
                ActiveConnectionString = (new LocateDatabase()).PromptUser();
                if (ActiveConnectionString.Length == 0)
                    throw;
                configChanged = true;
                goto TryConnectingAgain;
            }

            // ----- Save the updated configuration if needed.
            if (configChanged == true)
                Properties.Settings.Default.DBConnection = ActiveConnectionString;

            // ----- Success.
            return libraryDB;
        }

        public static decimal ConvertPageUnits(char fromUnit, char toUnit, decimal baseSize)
        {
            // ----- Convert a unit from one size to another. The units are:
            //       I = Inches
            //       C = Centimeters
            //       P = Points
            //       T = Twips
            //       H = 1/100 of an inch
            decimal workSize = 0M;
            char unitType;

            // ----- First, convert everything to twips.
            unitType = char.ToUpper(fromUnit);
            if (unitType == 'C')
                workSize = baseSize * (1440.0M / 2.54M);
            else if (unitType == 'H')
                workSize = baseSize * (1440.0M / 100.0M);
            else if (unitType == 'I')
                workSize = baseSize * 1440.0M;
            else if (unitType == 'P')
                workSize = baseSize * (1440.0M / 72.0M);
            else if (unitType == 'T')
                workSize = baseSize;
            else
                return baseSize;

            // ----- Now convert it to the requested units.
            unitType = char.ToUpper(toUnit);
            if (unitType == 'C')
                return workSize / (1440.0M / 2.54M);
            else if (unitType == 'H')
                return workSize / (1440.0M / 100.0M);
            else if (unitType == 'I')
                return workSize / 1440.0M;
            else if (unitType == 'P')
                return workSize / (1440.0M / 72.0M);
            else if (unitType == 'T')
                return workSize;
            else
                return baseSize;
        }

        public static int CountSubStr(string mainText, string subText)
        {
            // ----- Return a count of the number of times that a subText occurs in
            //       a string (mainText).
            int totalTimes;
            int startPos;
            int foundPos;

            if ((mainText == null) || (mainText.Length == 0))
                return 0;
            if ((subText == null) || (subText.Length == 0))
                return 0;
            totalTimes = 0;
            startPos = 0;

            // ----- Keep searching until we don't find it no more!
            do
            {
                // ----- Search for the subText.
                foundPos = mainText.IndexOf(subText, startPos);
                if (foundPos == -1)
                    break;
                totalTimes += 1;

                // ----- Move to just after the occurrence.
                startPos = foundPos + subText.Length;
            } while (true);

            // ----- Return the count.
            return totalTimes;
        }

        public static DataTable CreateDataTable(DataCommand theAction)
        {
            // ----- Given a SQL statement with parameters, return a data table.
            SqlCommand dbCommand;
            SqlDataAdapter dbAdapter;
            DataTable dbTable;
            SqlConnection localConnection = null;

            // ----- Connect to the database, if not yet done. Throw exception on failure.
            if (theAction.Connection == null)
                localConnection = ConnectDatabase();

            // ----- Try to run the statement. Throw exception on failure.
            try
            {
                dbCommand = theAction.GetCommandObject();
                if (localConnection != null)
                    dbCommand.Connection = localConnection;
                dbAdapter = new SqlDataAdapter(dbCommand);
                dbTable = new DataTable();
                dbAdapter.Fill(dbTable);
            }
            catch
            {
                // ----- Reissue error after Finally clause.
                throw;
            }
            finally
            {
                // ----- Disconnect if needed.
                if (localConnection != null)
                    localConnection.Close();
            }
            return dbTable;
        }

        public static DataTable CreateDataTable(DataCommand theAction, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement with parameters, return a data table.
            SqlCommand dbCommand;
            SqlDataAdapter dbAdapter;
            DataTable dbTable;

            // ----- Try to run the statement. Throw exception on failure.
            dbCommand = theAction.GetCommandObject();
            dbCommand.Connection = transInfo.Connection;
            dbCommand.Transaction = transInfo.Transaction;
            dbAdapter = new SqlDataAdapter(dbCommand);
            dbTable = new DataTable();
            dbAdapter.Fill(dbTable);
            return dbTable;
        }

        public static DataTable CreateDataTable(string sqlText)
        {
            // ----- Given a SQL statement, return a data table.
            SqlCommand dbCommand;
            SqlDataAdapter dbAdapter;
            DataTable dbTable;

            // ----- Connect to the database. Throw exception on failure.
            using (SqlConnection libraryDB = ConnectDatabase())
            {
                // ----- Try to run the statement. Throw exception on failure.
                dbCommand = new SqlCommand(sqlText, libraryDB);
                dbAdapter = new SqlDataAdapter(dbCommand);
                dbTable = new DataTable();
                dbAdapter.Fill(dbTable);
            }
            return dbTable;
        }

        public static DataTable CreateDataTable(string sqlText, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement and a transaction, return a data table.
            SqlCommand dbCommand;
            SqlDataAdapter dbAdapter;
            DataTable dbTable;

            // ----- Try to run the statement. Throw exception on failure.
            dbCommand = new SqlCommand(sqlText, transInfo.Connection);
            dbCommand.Transaction = transInfo.Transaction;
            dbAdapter = new SqlDataAdapter(dbCommand);
            dbTable = new DataTable();
            dbAdapter.Fill(dbTable);
            return dbTable;
        }

        public static string DBFontStyle(Font useFont)
        {
            // ----- Given a font, extract to a string the possible letters "BIUK"
            //      (bold, italic, underline, strikeout). See the BuildFontStyle
            //      function for the reverse of this function.
            string resultStyle;

            // ----- Process the string.
            resultStyle = "";
            if (useFont.Bold == true)
                resultStyle += "B";
            if (useFont.Italic == true)
                resultStyle += "I";
            if (useFont.Underline == true)
                resultStyle += "U";
            if (useFont.Strikeout == true)
                resultStyle += "K";
            return resultStyle;
        }

        public static string DBComboParam(ComboBox whichField)
        {
            // ----- Prepare the value of a ComboBox for use in the database.
            ListItemData listItem = null;

            // ----- Get the stored data for this item.
            if (whichField.SelectedItem == null)
                return "";
            listItem = (ListItemData)whichField.SelectedItem;

            // ----- An item value of "-1" has special meaning: unselected.
            if (listItem.ItemData == -1L)
                return "";
            else
                return listItem.ItemData.ToString();
        }

        public static bool DBGetBool(object dataField)
        {
            // ----- Return the Boolean equivalent of an optional database field.
            return Convert.ToBoolean(DBGetInteger(dataField));
        }

        public static DateTime DBGetDate(object dataField)
        {
            // ----- Return the date equivalent of an optional database field.
            DateTime holdDate;

            if (dataField == null)
                return DateTime.MinValue;
            else if (Convert.IsDBNull(dataField) == true)
                return DateTime.MinValue;
            else if (DateTime.TryParse(dataField.ToString(), out holdDate) == false)
                return DateTime.MinValue;
            else
                return holdDate;
        }

        public static decimal DBGetDecimal(object dataField)
        {
            // ----- Return the decimal equivalent of an optional database field.
            if (Convert.IsDBNull(dataField) == true)
                return 0M;
            else
                return Convert.ToDecimal(dataField);
        }

        public static int DBGetInteger(object dataField)
        {
            // ----- Return the integer equivalent of an optional database field.
            if (Convert.IsDBNull(dataField) == true)
                return 0;
            else
                return Convert.ToInt32(dataField);
        }

        public static long DBGetLong(object dataField, long useDefault = 0L)
        {
            // ----- Return the long equivalent of an optional database field.
            if (dataField == null)
                return useDefault;
            else if (Convert.IsDBNull(dataField) == true)
                return useDefault;
            else
                return Convert.ToInt64(dataField);
        }

        public static string DBGetText(object dataField, string defaultText = "")
        {
            // ----- Return the text equivalent of an optional database field.
            if (Convert.IsDBNull(dataField) == true)
                return defaultText;
            else
                return Convert.ToString(dataField);
        }

        public static string DigitsOnly(string origText)
        {
            // ----- Return only the digits found in a string.
            string destText;

            // ----- Examine each character.
            destText = "";
            for (int counter = 0; counter < origText.Length; counter++)
            {
                if (char.IsDigit(origText[counter]) == true)
                    destText += origText[counter];
            }
            return destText;
        }

        public static string EncryptPassword(string loginID, string passwordText)
        {
            // ----- Given a username and a password, encrypt the password so that it
            //       is not easy to decrypt. There is no limit on password length since
            //       it is going to be hashed anyway.
            HMACSHA1 hashingFunction;
            byte[] secretKey;
            byte[] hashValue;
            string result = "";

            // ----- Prepare the secret key. Force it to uppercase for consistency,
            //       and then stuff it in a byte array.
            secretKey = (new UnicodeEncoding()).GetBytes(loginID.ToUpper());

            // ----- Create the hashing component using Managed SHA-1.
            hashingFunction = new HMACSHA1(secretKey, true);

            // ----- Calculate the hash value. One simple line of code.
            hashValue = hashingFunction.ComputeHash((new UnicodeEncoding()).GetBytes(passwordText));

            // ----- The hash value is ready, but I like things in plain text when
            //       possible. Let's convert it to a long hex string.
            for (int counter = 0; counter < hashValue.Length; counter++)
                result += Convert.ToString(hashValue[counter], 16).ToUpper();

            // ----- Stored passwords are limited to 20 characters.
            if (result.Length > 20)
                result = result.Substring(0, 20);
            return result;
        }

        public static void ExecuteSQL(DataCommand theAction)
        {
            // ----- Given a SQL statement with parameters, run it.
            SqlCommand dbCommand;
            SqlConnection localConnection = null;

            // ----- Connect to the database, if not yet done. Throw exception on failure.
            if (theAction.Connection == null)
                localConnection = ConnectDatabase();

            // ----- Try to run the statement. Throw exception on failure.
            try
            {
                dbCommand = theAction.GetCommandObject();
                if (localConnection != null)
                    dbCommand.Connection = localConnection;
                dbCommand.ExecuteNonQuery();
            }
            catch
            {
                // ----- Reissue error after Finally clause.
                throw;
            }
            finally
            {
                // ----- Disconnect if needed.
                if (localConnection != null)
                    localConnection.Close();
            }
        }

        public static void ExecuteSQL(DataCommand theAction, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement with parameters, run it.
            SqlCommand dbCommand;

            // ----- Try to run the statement. Throw exception on failure.
            dbCommand = theAction.GetCommandObject();
            dbCommand.Connection = transInfo.Connection;
            dbCommand.Transaction = transInfo.Transaction;
            dbCommand.ExecuteNonQuery();
        }

        public static void ExecuteSQL(string sqlText)
        {
            // ----- Given a SQL statement, run it.
            SqlCommand dbCommand;

            // ----- Connect to the database. Throw exception on failure.
            using (SqlConnection libraryDB = ConnectDatabase())
            {
                // ----- Try to run the statement. Throw exception on failure.
                dbCommand = new SqlCommand(sqlText, libraryDB);
                dbCommand.ExecuteNonQuery();
            }
        }

        public static void ExecuteSQL(string sqlText, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement, run it.
            SqlCommand dbCommand;

            // ----- Connect to the database. Throw exception on failure.
            dbCommand = new SqlCommand(sqlText, transInfo.Connection);
            dbCommand.Transaction = transInfo.Transaction;
            dbCommand.ExecuteNonQuery();
        }

        public static object ExecuteSQLReturn(DataCommand theAction)
        {
            // ----- Given a SQL statement with parameters, run it, and return a result.
            SqlCommand dbCommand;
            SqlConnection localConnection = null;

            // ----- Connect to the database, if not yet done. Throw exception on failure.
            if (theAction.Connection == null)
                localConnection = ConnectDatabase();

            // ----- Try to run the statement. Throw exception on failure.
            try
            {
                dbCommand = theAction.GetCommandObject();
                if (localConnection != null)
                    dbCommand.Connection = localConnection;
                return dbCommand.ExecuteScalar();
            }
            catch
            {
                // ----- Reissue error after Finally clause.
                throw;
            }
            finally
            {
                // ----- Disconnect if needed.
                if (localConnection != null)
                    localConnection.Close();
            }
        }

        public static object ExecuteSQLReturn(DataCommand theAction, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement with parameters, run it, and return a result.
            SqlCommand dbCommand;

            // ----- Try to run the statement. Throw exception on failure.
            dbCommand = theAction.GetCommandObject();
            dbCommand.Connection = transInfo.Connection;
            dbCommand.Transaction = transInfo.Transaction;
            return dbCommand.ExecuteScalar();
        }

        public static object ExecuteSQLReturn(string sqlText)
        {
            // ----- Given a SQL statement, run it, and return a result.
            SqlCommand dbCommand;

            // ----- Connect to the database. Throw exception on failure.
            using (SqlConnection libraryDB = ConnectDatabase())
            {
                // ----- Try to run the statement. Throw exception on failure.
                dbCommand = new SqlCommand(sqlText, libraryDB);
                return dbCommand.ExecuteScalar();
            }
        }

        public static object ExecuteSQLReturn(string sqlText, LibraryTransaction transInfo)
        {
            // ----- Given a SQL statement, run it, and return a result.
            SqlCommand dbCommand;

            // ----- Connect to the database. Throw exception on failure.
            dbCommand = new SqlCommand(sqlText, transInfo.Connection);
            dbCommand.Transaction = transInfo.Transaction;
            return dbCommand.ExecuteScalar();
        }

        public static string FitTextToWidth(string origText, int pixelWidth,
            Graphics canvas, Font useFont)
        {
            // ----- Given a text string, make sure it fits in the specified
            //       pixel width. Truncate and add an ellipsis if needed.
            string newText;

            newText = origText;
            if (canvas.MeasureString(newText, useFont).Width > pixelWidth)
            {
                while (canvas.MeasureString(newText + "...", useFont).Width > pixelWidth)
                {
                    newText = newText.Substring(0, newText.Length - 1);
                    if (newText.Length == 0)
                        break;
                }
                if (newText.Length != 0)
                    newText += "...";
            }
            return newText;
        }

        public static string FormatAuthorName(this DataRow dbRow)
        {
            // ----- Given an author record, return the formatted name.
            string authorName;

            // ----- Format the name.
            authorName = DBGetText(dbRow["LastName"]);
            if (Convert.IsDBNull(dbRow["FirstName"]) == false)
            {
                authorName += ", " + DBGetText(dbRow["FirstName"]);
                if (Convert.IsDBNull(dbRow["MiddleName"]) == false)
                    authorName += " " + DBGetText(dbRow["MiddleName"]);
            }
            if (Convert.IsDBNull(dbRow["Suffix"]) == false)
                authorName += ", " + DBGetText(dbRow["Suffix"]);

            // ----- Add in the birth and death years.
            if ((Convert.IsDBNull(dbRow["BirthYear"]) == false) |
                (Convert.IsDBNull(dbRow["DeathYear"]) == false))
            {
                authorName += " (";
                if (Convert.IsDBNull(dbRow["BirthYear"]) == true)
                    authorName += "????";
                else
                {
                    authorName += Math.Abs(DBGetInteger(dbRow["BirthYear"])).ToString();
                    if (DBGetInteger(dbRow["BirthYear"]) < 0)
                        authorName += "BC";
                }
                authorName += "-";
                if (Convert.IsDBNull(dbRow["DeathYear"]) == false)
                {
                    authorName += Math.Abs(DBGetInteger(dbRow["DeathYear"])).ToString();
                    if (DBGetInteger(dbRow["DeathYear"]) < 0)
                        authorName += "BC";
                }
                authorName += ")";
            }

            // ----- Finished.
            return authorName;
        }

        public static string FormatAuthorName(this QAuthor author)
        {
            // ----- Given an author record, return the formatted name.
            string authorName;

            // ----- Format the name.
            authorName = author.LastName + "";
            if (author.FirstName != null)
            {
                authorName += ", " + author.FirstName;
                if (author.MiddleName != null)
                    authorName += " " + author.MiddleName;
            }
            if (author.Suffix != null)
                authorName += ", " + author.Suffix;

            // ----- Add in the birth and death years.
            if ((author.BirthYear != null) | (author.DeathYear != null))
            {
                authorName += " (";
                if (author.BirthYear == null)
                    authorName += "????";
                else
                {
                    authorName += Math.Abs(author.BirthYear.Value).ToString();
                    if (author.BirthYear < 0)
                        authorName += "BC";
                }
                authorName += "-";
                if (author.DeathYear != null)
                {
                    authorName += Math.Abs(author.DeathYear.Value).ToString();
                    if (author.DeathYear < 0)
                        authorName += "BC";
                }
                authorName += ")";
            }

            // ----- Finished.
            return authorName;
        }

        public static void GeneralError(string routineName, Exception theError)
        {
            // ----- Report an error to the user.
            MessageBox.Show("The following error occurred at location '" + routineName + "':" +
                Environment.NewLine + Environment.NewLine + theError.Message,
                ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        public static string GetBarcodeFont()
        {
            // ----- Retrieve the barcode font to use.
            string barcodeFontName;

            barcodeFontName = GetSystemValue("BarcodeFont").Trim();
            if (barcodeFontName.Length == 0)
            {
                MessageBox.Show("The font used to print barcodes has not been identified. " +
                    "You must identify the barcode font through the Application " +
                    "Maintenance features before editing label templates.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            return barcodeFontName;
        }

        public static bool GetBoolean(string content)
        {
            // ----- Extract a boolean value from a string. Return False if
            //       the string contains no legitimate value.
            int holdInt;
            bool holdBool;

            if (int.TryParse(content, out holdInt) == true)
                return Convert.ToBoolean(holdInt);
            else if (bool.TryParse(content, out holdBool) == true)
                return holdBool;
            else
                return false;
        }

        public static string GetCopyDisposition(long copyID)
        {
            // ----- Given the ID of an item copy, return it's current status.
            //       The available results are:
            //         New Item Copy      Error or missing record
            //         Checked In         Copy is checked in and available
            //         Checked Out        Copy is checked out by a patron
            //         Overdue            Copy is checked out and late
            //         Missing            Copy has been reported missing
            //         Reference          Copy is a reference copy
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            string response;

            // ----- Retrieve information about the book.
            sqlText = "SELECT IC.Reference, IC.Missing, PC.Missing AS PatronMissing, " +
                "PC.DueDate, PC.Patron FROM ItemCopy AS IC LEFT JOIN PatronCopy AS PC " +
                "ON IC.ID = PC.ItemCopy WHERE IC.ID = @RecordID AND PC.Returned = 0";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", copyID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("GetCopyDisposition.Info", ex);
                return "New Item Copy";
            }
            if (dbInfo.Rows.Count == 0)
            {
                // ----- Perhaps the database, due to the JOIN clause, could not return
                //       the base record of a checked-in book.
                dbInfo.Dispose();
                sqlText = "SELECT Reference, Missing, NULL AS PatronMissing, " +
                    "NULL AS DueDate, NULL AS Patron FROM ItemCopy WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", copyID);
                try
                {
                    dbInfo = CreateDataTable(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("GetCopyDisposition.Info2", ex);
                    return "New Item Copy";
                }
                if (dbInfo.Rows.Count == 0)
                {
                    // ----- What happened to the record?
                    dbInfo.Dispose();
                    dbInfo = null;
                    return "New Item Copy";
                }
            }

            // ----- We found the record. Check its details.
            oneRow = dbInfo.Rows[0];
            if (Convert.IsDBNull(oneRow["Patron"]) == false)
            {
                // ----- This item is checked out.
                if ((DBGetBool(oneRow["Missing"]) == true) |
                        (DBGetBool(oneRow["PatronMissing"]) == true))
                    response = "Missing";
                else if (DBGetDate(oneRow["DueDate"]) < DateTime.Today)
                    response = "Overdue";
                else
                    response = "Checked Out";
            }
            else
            {
                // ----- This book is currently checked in.
                if (DBGetBool(oneRow["Missing"]) == true)
                    response = "Missing";
                else if (DBGetBool(oneRow["Reference"]) == true)
                    response = "Reference";
                else
                    response = "Checked In";
            }
            dbInfo.Dispose();
            dbInfo = null;
            return response;
        }

        public static string GetSubStr(this string origString, string delim, int whichField)
        {
            // ----- Extracts a delimited string from another larger string.
            string[] stringParts = null;

            // ----- Handle some errors.
            if (whichField < 0)
                return "";
            if ((origString == null) || (origString.Length == 0))
                return "";
            if ((delim == null) || (delim.Length == 0))
                return "";

            // ----- Break the string up into delimited parts.
            stringParts = origString.Split(new[] {delim}, StringSplitOptions.None);

            // ----- See if the part we want exists and return it.
            if (whichField > stringParts.Length)
                return "";
            else
                return stringParts[whichField - 1];
        }

        public static string GetSystemValue(string valueName)
        {
            // ----- Return the data portion of a system value name-data pair.
            string sqlText;
            DataCommand sqlRun;

            // ----- Retrieve the value.
            sqlText = "SELECT ValueData FROM SystemValue " +
                "WHERE UPPER(ValueName) = @TestName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestName", valueName.ToUpper());
            try
            {
                return DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("GetSystemValue", ex);
                return "";
            }
        }

        public static void InitializeSystem()
        {
            // ----- Initialize global variables here.  

            // ----- Ensure we have the latest settings.
            if (Properties.Settings.Default.SettingsUpgraded == false)
            {
                Properties.Settings.Default.Upgrade();
                Properties.Settings.Default.SettingsUpgraded = true;
            }

            // ----- Clear security-related values.
            LoggedInUserID = -1L;
            LoggedInUserName = "";
            LoggedInGroupID = -1L;
            DefaultItemLocation = -1L;
            SearchMatchLimit = 250;
            FineGraceDays = -1;
            AllHolidays = new List<HolidaySet>();

            for (int counter = 1; counter <= (int)MaxLibrarySecurity; counter++)
                SecurityProfile[counter] = false;

            // ----- Locate the online help files.
            MainHelpFile = Properties.Settings.Default.HelpFile + "";
            MainAdminHelpFile = Properties.Settings.Default.HelpFileAdmin + "";
        }

        public static bool IsHolidayDate(DateTime whatDate)
        {
            // ----- See if the given date is a holiday.
            foreach (HolidaySet oneHoliday in AllHolidays)
            {
                if (oneHoliday.IsHoliday(whatDate))
                    return true;
            }

            // ----- Not a holiday.
            return false;
        }

        public static string LeftAndRightText(string origLeft, string origRight, int textWidth)
        {
            // ----- Build a string of the indicated width that has both left-justified
            //       and right-justified text.
            string leftText;
            string rightText;

            leftText = origLeft.Trim();
            rightText = origRight.Trim();
            if ((leftText + rightText).Length >= textWidth)
                return (leftText + rightText).Substring(0, textWidth).Trim();
            else
                return leftText + new string(' ', textWidth -
                    (leftText + rightText).Length) + rightText;
        }

        public static void LoadDatabaseSettings()
        {
            // ----- Get some system-level values from database storage.
            string holdText;

            // ----- Get the default location.
            holdText = GetSystemValue("DefaultLocation");
            if (holdText.Length == 0)
                holdText = "-1";
            if (long.TryParse(holdText, out DefaultItemLocation) == false)
                DefaultItemLocation = -1L;

            // ----- Get the maximum number of search matches.
            holdText = GetSystemValue("SearchLimit");
            if (holdText.Length == 0)
                holdText = "-1";
            if (int.TryParse(holdText, out SearchMatchLimit) == false)
                SearchMatchLimit = -1;

            // ----- Get the number of days to wait before charging fines.
            holdText = GetSystemValue("FineGrace");
            if (holdText.Length == 0)
                holdText = "-1";
            if (int.TryParse(holdText, out FineGraceDays) == false)
                FineGraceDays = -1;
        }

        public static void RefreshHolidays()
        {
            // ----- Load in the list of holidays.
            string sqlText;
            DataTable dbInfo;
            HolidaySet newHoliday;

            // ----- Clear the current list of holidays.
            AllHolidays.Clear();

            // ----- Get the holidays from the database.
            sqlText = "SELECT * FROM Holiday";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("RefreshHolidays", ex);
                return;
            }
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                newHoliday = new HolidaySet(
                    DBGetText(dbRow["EntryType"]), DBGetText(dbRow["EntryDetail"]));
                AllHolidays.Add(newHoliday);
            }
            dbInfo.Dispose();
        }

        public static void ReprocessSecuritySet()
        {
            // ----- Reload in the security set for the current user. If no user
            //       is logged in, clear all settings.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo = null;

            // ----- Clear out the existing items.
            for (int counter = 1; counter <= (int)MaxLibrarySecurity; counter++)
                SecurityProfile[counter] = false;

            // ----- Exit if there is no user logged in.
            if ((LoggedInUserID == -1L) | (LoggedInGroupID == -1L))
                return;

            try
            {
                // ----- Load in the security elements for this user.
                sqlText = "SELECT ActivityID FROM GroupActivity " +
                    "WHERE GroupID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", LoggedInGroupID);
                dbInfo = CreateDataTable(sqlRun);
                foreach (DataRow dbRow in dbInfo.Rows)
                    SecurityProfile[DBGetLong(dbRow["ActivityID"])] = true;
            }
            catch (Exception ex)
            {
                // ----- Some database-related error.
                GeneralError("ReprocessSecuritySet", ex);

                // ----- Un-login the administrator through recursion.
                LoggedInUserID = -1L;
                LoggedInGroupID = -1L;
                ReprocessSecuritySet();
            }
            finally
            {
                if (dbInfo != null)
                    dbInfo.Dispose();
                dbInfo = null;
            }
        }

        public static void SetSystemValue(string valueName, string valueData)
        {
            // ----- Update a record in the SystemValue table.
            string sqlText;
            DataCommand sqlRun;

            try
            {
                // ----- See if the entry already exists.
                sqlText = "SELECT COUNT(*) FROM SystemValue " +
                    "WHERE UPPER(ValueName) = @TestName";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("TestName", valueName.ToUpper());
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    // ----- Value already exists.
                    sqlText = "UPDATE SystemValue SET ValueData = @NewData " +
                        "WHERE UPPER(ValueName) = @TestName";
                    sqlRun = new DataCommand(sqlText);
                    sqlRun.AddText("NewData", valueData);
                    sqlRun.AddText("TestName", valueName.ToUpper());
                }
                else
                {
                    // ----- Need to create value.
                    sqlText = "INSERT INTO SystemValue (ValueName, ValueData) " +
                        "VALUES (@NewName, @NewData)";
                    sqlRun = new DataCommand(sqlText);
                    sqlRun.AddText("NewName", valueName);
                    sqlRun.AddText("NewData", valueData);
                }

                // ----- Update the value.
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("SetSystemValue", ex);
            }
        }

        public static LibraryTransaction TransactionBegin()
        {
            // ----- Create a new database transaction. Throw exception on failure.
            LibraryTransaction transInfo;

            // ----- Connect to the database.
            transInfo = new LibraryTransaction();
            transInfo.Connection = ConnectDatabase();
            transInfo.Transaction = transInfo.Connection.BeginTransaction();
            return transInfo;
        }

        public static void TransactionCommit(LibraryTransaction transInfo)
        {
            // ----- Commit the transaction.
            if (transInfo == null)
                return;
            try
            {
                if (transInfo.Transaction != null)
                    transInfo.Transaction.Commit();
            }
            catch
            {
                // ----- Reissue error after Finally clause.
                throw;
            }
            finally
            {
                transInfo.Transaction = null;
                if (transInfo.Connection != null)
                    transInfo.Connection.Close();
                transInfo.Connection = null;
            }
        }

        public static void TransactionRollback(LibraryTransaction transInfo)
        {
            // ----- Rollback the transaction.
            if (transInfo == null)
                return;
            try
            {
                if (transInfo.Transaction != null)
                    transInfo.Transaction.Rollback();
            }
            catch
            {
                // ----- Reissue error after Finally clause.
                throw;
            }
            finally
            {
                transInfo.Transaction = null;
                if (transInfo.Connection != null)
                    transInfo.Connection.Close();
                transInfo.Connection = null;
            }
        }
    }

    public class LibraryTransaction
    {
        public SqlConnection Connection;
        public SqlTransaction Transaction;

        public void Commit()
        {
            // ----- Manage the transaction commit.
            Exception saveError = null;

            // ----- Commit the transaction.
            try
            {
                if (this.Transaction != null)
                    this.Transaction.Commit();
            }
            catch (Exception ex)
            {
                // ----- Save error for later.
                saveError = ex;
            }

            // ----- Close the connection.
            try
            {
                if (this.Connection != null)
                    this.Connection.Close();
            }
            catch
            {
                // ----- Ignore any error.
            }

            // ----- Return error if it exists.
            if (saveError != null)
                throw saveError;
        }

        public void Rollback()
        {
            // ----- Manage the transaction rollback.
            Exception saveError = null;

            // ----- Rollback the transaction.
            try
            {
                if (this.Transaction != null)
                    this.Transaction.Rollback();
            }
            catch (Exception ex)
            {
                // ----- Save error for later.
                saveError = ex;
            }

            // ----- Close the connection.
            try
            {
                if (this.Connection != null)
                    this.Connection.Close();
            }
            catch
            {
                // ----- Ignore any error.
            }

            // ----- Return error if it exists.
            if (saveError != null)
                throw saveError;
        }
    }

    public class DataCommand
    {
        public enum CommandDataType: int
        {
            AsBoolean,
            AsDate,
            AsDateTime,
            AsLong,
            AsInteger,
            AsDecimal,
            AsText
        }

        public class CommandParameter
        {
            public string ParameterName = "";
            public object SourceData;
            public CommandDataType SourceType;
            public CommandDataType TargetType;
        }

        public SqlConnection Connection = null;
        public string CommandText = "";
        public Dictionary<string, CommandParameter> Parameters =
            new Dictionary<string, CommandParameter>();
        public bool IsStoredProcedure = false;

        public DataCommand()
        {
            // ----- Default constructor.
            this.Connection = null;
            this.CommandText = "";
        }

        public DataCommand(string sqlText)
        {
            // ----- Command-only constructor.
            this.Connection = null;
            this.CommandText = sqlText;
            if (sqlText == null)
                this.CommandText = "";
        }

        public DataCommand(string sqlText, SqlConnection connectDB)
        {
            // ----- Full constructor.
            this.Connection = connectDB;
            this.CommandText = sqlText;
            if (sqlText == null)
                this.CommandText = "";
        }

        public SqlCommand GetCommandObject()
        {
            // ----- Build a command object that includes all of the appropriate
            //       parameters.
            SqlCommand result = new SqlCommand();
            SqlParameter oneParam;
            string holdText;
            DateTime holdDate;

            // ----- Fill in the basics.
            result.Connection = this.Connection;
            result.CommandText = this.CommandText;
            if (this.IsStoredProcedure == true)
                result.CommandType = CommandType.StoredProcedure;
            else
                result.CommandType = CommandType.Text;

            foreach (CommandParameter scanParam in this.Parameters.Values)
            {
                // ----- Build a parameter object for the command.
                oneParam = null;
                switch (scanParam.TargetType)
                {
                    case CommandDataType.AsBoolean:
                        // ----- Source will always be Boolean. Convert to Integer.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.Bit);
                        oneParam.Value = Convert.ToBoolean(scanParam.SourceData) ? 1 : 0;
                        break;
                    case CommandDataType.AsDate:
                    case CommandDataType.AsDateTime:
                        // ----- Source will be date or string.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.DateTime);
                        if ((scanParam.SourceType == CommandDataType.AsDateTime) |
                            (scanParam.SourceType == CommandDataType.AsDate))
                        {
                            // ----- Date source.
                            if (scanParam.TargetType == CommandDataType.AsDate)
                            {
                                // ----- Exclude the time.
                                oneParam.Value = Convert.ToDateTime(scanParam.SourceData).Date;
                            }
                            else
                            {
                                // ----- Include the time.
                                oneParam.Value = Convert.ToDateTime(scanParam.SourceData);
                            }
                        }
                        else
                        {
                            // ----- String source.
                            if (DateTime.TryParse(Convert.ToString(scanParam.SourceData),
                                out holdDate) == false)
                            {
                                // ----- Store NULL.
                                oneParam.Value = System.DBNull.Value;
                            }
                            else
                            {
                                // ----- True date.
                                if (scanParam.TargetType == CommandDataType.AsDate)
                                {
                                    // ----- Exclude the time.
                                    oneParam.Value = holdDate.Date;
                                }
                                else
                                {
                                    // ----- Include the time.
                                    oneParam.Value = holdDate;
                                }
                            }
                        }
                        break;
                    case CommandDataType.AsLong:
                        // ----- String or true Long value.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.BigInt);
                        if (scanParam.SourceType == CommandDataType.AsText)
                        {
                            holdText = Convert.ToString(scanParam.SourceData).Trim();
                            if (holdText.Length == 0)
                                oneParam.Value = System.DBNull.Value;
                            else
                                oneParam.Value = Convert.ToInt64(holdText);
                        }
                        else
                        {
                            oneParam.Value = Convert.ToInt64(scanParam.SourceData);
                        }
                        break;
                    case CommandDataType.AsInteger:
                        // ----- String or true Integer value.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.Int);
                        if (scanParam.SourceType == CommandDataType.AsText)
                        {
                            holdText = Convert.ToString(scanParam.SourceData).Trim();
                            if (holdText.Length == 0)
                                oneParam.Value = System.DBNull.Value;
                            else
                                oneParam.Value = Convert.ToInt32(holdText);
                        }
                        else
                        {
                            oneParam.Value = Convert.ToInt32(scanParam.SourceData);
                        }
                        break;
                    case CommandDataType.AsDecimal:
                        // ----- String or true Decimal value.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.Decimal);
                        if (scanParam.SourceType == CommandDataType.AsText)
                        {
                            holdText = Convert.ToString(scanParam.SourceData).Trim();
                            if (holdText.Length == 0)
                                oneParam.Value = System.DBNull.Value;
                            else
                                oneParam.Value = Convert.ToDecimal(holdText);
                        }
                        else
                        {
                            oneParam.Value = Convert.ToDecimal(scanParam.SourceData);
                        }
                        break;
                    case CommandDataType.AsText:
                        // ----- Char, Varchar, Text, etc.
                        oneParam = new SqlParameter(scanParam.ParameterName, SqlDbType.VarChar);
                        holdText = Convert.ToString(scanParam.SourceData);
                        if ((holdText == null) || (holdText.Trim().Length == 0))
                            oneParam.Value = System.DBNull.Value;
                        else
                            oneParam.Value = holdText;
                        break;
                }

                // ----- Store the parameter in the command.
                if (oneParam != null)
                    result.Parameters.Add(oneParam);
            }

            // ----- Finished.
            return result;
        }

        public void AddBool(string paramName, bool content)
        {
            // ----- Add a new Boolean parameter.
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsBoolean;
            newEntry.TargetType = CommandDataType.AsBoolean;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddDate(string paramName, string content = "")
        {
            // ----- Add a new date parameter. Stores NULL when content is "".
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsText;
            newEntry.TargetType = CommandDataType.AsDate;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddDate(string paramName, DateTime content)
        {
            // ----- Add a new date parameter.
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsDate;
            newEntry.TargetType = CommandDataType.AsDate;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddInteger(string paramName, string content = "")
        {
            // ----- Add a new integer parameter. Stores NULL when content is "".
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsText;
            newEntry.TargetType = CommandDataType.AsInteger;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddInteger(string paramName, int content)
        {
            // ----- Add a new integer parameter.
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsInteger;
            newEntry.TargetType = CommandDataType.AsInteger;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddID(string paramName, long content)
        {
            // ----- Same as AddLong, but a value of -1 is treated as NULL.
            if (content == -1L)
                this.AddLong(paramName);
            else
                this.AddLong(paramName, content);
        }

        public void AddLong(string paramName, string content = "")
        {
            // ----- Add a new long integer parameter. Stores NULL when content is "".
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsText;
            newEntry.TargetType = CommandDataType.AsLong;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddLong(string paramName, long content)
        {
            // ----- Add a new long integer parameter.
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsLong;
            newEntry.TargetType = CommandDataType.AsLong;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddDecimal(string paramName, string content = "")
        {
            // ----- Add a new long integer parameter. Stores NULL when content is "".
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsText;
            newEntry.TargetType = CommandDataType.AsDecimal;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddDecimal(string paramName, decimal content)
        {
            // ----- Add a new long integer parameter.
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsDecimal;
            newEntry.TargetType = CommandDataType.AsDecimal;
            this.Parameters.Add(paramName, newEntry);
        }

        public void AddText(string paramName, string content = "")
        {
            // ----- Add a new string parameter. Stores NULL when content is "".
            CommandParameter newEntry = new CommandParameter();

            newEntry.ParameterName = paramName;
            newEntry.SourceData = content;
            newEntry.SourceType = CommandDataType.AsText;
            newEntry.TargetType = CommandDataType.AsText;
            this.Parameters.Add(paramName, newEntry);
        }
    }
}
