﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class GroupName : Library.BaseCodeForm
    {
        private long ActiveID;
        private bool StartingOver;

        public GroupName()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            ActiveID = -1L;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManageGroups];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManageGroups];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManageGroups];
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the record.
            string sqlText;
            DataCommand sqlRun;
            LibraryTransaction transInfo = null;

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete the security group?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                    DialogResult.Yes)
                return false;

            // ----- Make sure this record is not in use.
            sqlText = "SELECT COUNT(*) FROM UserName WHERE GroupID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    MessageBox.Show("You cannot delete this record because it is in use.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("GroupName.DeleteRecord.Check", ex);
                return false;
            }

            // ----- Delete the record.
            try
            {
                transInfo = TransactionBegin();

                sqlText = "DELETE FROM GroupActivity WHERE GroupID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", recordID);
                ExecuteSQL(sqlRun, transInfo);

                sqlText = "DELETE FROM GroupName WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", recordID);
                ExecuteSQL(sqlRun, transInfo);

                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                if (transInfo != null)
                    TransactionRollback(transInfo);
                GeneralError("GroupName.DeleteRecord.Delete", ex);
                return false;
            }

            // ----- Success.
            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record.
            ActiveID = recordID;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items.
            string sqlText;
            DataTable dbInfo;

            // ----- Clear any existing values.
            destList.Items.Clear();
            exceededMatches = false;

            // ----- Retrieve the values from the database.
            sqlText = "SELECT * FROM GroupName ORDER BY FullName";
            try
            {
                dbInfo = CreateDataTable(sqlText);
            }
            catch (Exception ex)
            {
                GeneralError("GroupName.FillListWithRecords", ex);
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                destList.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;

            // ----- Retrieve the value from the database.
            sqlText = "SELECT FullName FROM GroupName WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                return DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("GroupName.FormatRecordName", ex);
                return "Error";
            }
        }

        public override string GetEditDescription()
        {
            // ----- Return the list form display title.
            return "Use these codes to establish the security groups into which " +
                "administrative users are placed. Each security group can receive " +
                "access to different parts of the system.";
        }

        public override string GetEditTitle()
        {
            // ----- Return the list form display title.
            return "Security Groups";
        }

        private void PrepareFormFields()
        {
            // ----- Fill in the data.
            string sqlText;
            DataTable dbInfo;
            DataCommand sqlRun;
            int matchPos;

            // ----- Load in the list of security groups.
            if (ActivityList.Items.Count == 0)
            {
                // ----- This is the first time. Load from the database.
                ActivityList.Items.Clear();
                sqlText = "SELECT * FROM Activity ORDER BY FullName";
                try
                {
                    dbInfo = CreateDataTable(sqlText);
                }
                catch (Exception ex)
                {
                    GeneralError("GroupName.PrepareFormFields.Retrieve", ex);
                    return;
                }
                foreach (DataRow dbRow in dbInfo.Rows)
                {
                    ActivityList.Items.Add(new ListItemData(
                        DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
                }
                dbInfo.Dispose();
                dbInfo = null;
            }
            else
            {
                // ----- Editing a subsequent code. Clear selections. Amazingly,
                //       there's no Clear method to do this.
                while (ActivityList.CheckedItems.Count > 0)
                    ActivityList.SetItemChecked(ActivityList.CheckedIndices[0], false);
                if (ActivityList.Items.Count > 0)
                    ActivityList.TopIndex = 0;
            }

            // ----- Clear the existing data.
            StartingOver = true;
            RecordFullName.Text = "";
            if (ActiveID == -1L)
                return;

            // ----- Get the name of this group.
            sqlText = "SELECT FullName FROM GroupName WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                RecordFullName.Text = DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("GroupName.PrepareFormFields.Name", ex);
                return;
            }

            // ----- Load in the existing security settings.
            sqlText = "SELECT ActivityID FROM GroupActivity WHERE GroupID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("GroupName.PrepareFormFields.Current", ex);
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
            {
                matchPos = ActivityList.Items.IndexOf(DBGetLong(dbRow["ActivityID"]));
                if (matchPos != -1)
                    ActivityList.SetItemChecked(matchPos, true);
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        private bool SaveFormData()
        {
            // ----- The user wants to save changes. Return True on success.
            string sqlText;
            DataCommand sqlRun;
            long newID = -1L;
            LibraryTransaction transInfo = null;

            // ----- Prepare to save the data.
            try
            {
                transInfo = TransactionBegin();
            }
            catch (Exception ex)
            {
                GeneralError("GroupName.SaveFormData.Trans", ex);
                return false;
            }
            this.Cursor = Cursors.WaitCursor;

            // ----- Save the data.
            if (ActiveID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO GroupName (FullName) " +
                    "OUTPUT INSERTED.ID VALUES (@NewName)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun, transInfo));
                }
                catch (Exception ex)
                {
                    TransactionRollback(transInfo);
                    this.Cursor = Cursors.Default;
                    GeneralError("GroupName.SaveFormData.Add", ex);
                    return false;
                }
            }
            else
            {
                // ----- Update the existing entry.
                newID = ActiveID;
                sqlText = "UPDATE GroupName SET FullName = @NewName " +
                    "WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                sqlRun.AddLong("RecordID", ActiveID);
                try
                {
                    ExecuteSQL(sqlRun, transInfo);
                }
                catch (Exception ex)
                {
                    TransactionRollback(transInfo);
                    this.Cursor = Cursors.Default;
                    GeneralError("GroupName.SaveFormData.Update", ex);
                    return false;
                }
            }

            // ----- Clear any existing security settings.
            sqlText = "DELETE FROM GroupActivity WHERE GroupID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", newID);
            try
            {
                ExecuteSQL(sqlRun, transInfo);
            }
            catch (Exception ex)
            {
                TransactionRollback(transInfo);
                this.Cursor = Cursors.Default;
                GeneralError("GroupName.SaveFormData.Clear", ex);
                return false;
            }

            // ----- Save the selected security settings.
            foreach (ListItemData itemChecked in ActivityList.CheckedItems)
            {
                sqlText = "INSERT INTO GroupActivity (GroupID, ActivityID) " +
                    "VALUES (@NewGroup, @NewActivity)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("NewGroup", newID);
                sqlRun.AddLong("NewActivity", itemChecked.ItemData);
                try
                {
                    ExecuteSQL(sqlRun, transInfo);
                }
                catch (Exception ex)
                {
                    TransactionRollback(transInfo);
                    this.Cursor = Cursors.Default;
                    GeneralError("GroupName.SaveFormData.Activity", ex);
                    return false;
                }
            }

            // ----- Complete all changes.
            try
            {
                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                TransactionRollback(transInfo);
                this.Cursor = Cursors.Default;
                GeneralError("GroupName.SaveFormData.Commit", ex);
                return false;
            }

            // ----- This change may affect this user.
            if (LoggedInGroupID == newID)
                ReprocessSecuritySet();

            // ----- Success.
            ActiveID = newID;
            this.Cursor = Cursors.Default;
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            string sqlText;
            DataCommand sqlRun;

            // ----- Name is a required field.
            if (RecordFullName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the security group name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordFullName.Focus();
                return false;
            }

            // ----- Make sure the name is not already in use.
            sqlText = "SELECT COUNT(*) FROM GroupName " +
                "WHERE UPPER(FullName) = @NewName";
            if (ActiveID != -1L)
                sqlText += " AND ID <> @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("NewName", RecordFullName.Text.Trim().ToUpper());
            if (ActiveID != -1L)
                sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    MessageBox.Show("A record already exists with the name '" +
                        RecordFullName.Text.Trim() + "'.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordFullName.Focus();
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("GroupName.ValidateFormData", ex);
                return false;
            }

            // ----- Success.
            return true;
        }

        private void GroupName_Activated(object sender, EventArgs e)
        {
            // ----- Return the focus to the main field.
            if (StartingOver)
                RecordFullName.Focus();
            StartingOver = false;
        }

        private void RecordFullName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordFullName.SelectAll();
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- Save changes and return to the summary form.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }
    }
}
