﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Data.Linq.SqlClient;
using System.Drawing;
using System.Linq;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class ItemLookup : Form
    {
        public ItemLookup()
        {
            InitializeComponent();
        }

        private class SingleItemLink
        {
            public LookupMethods LinkType;
            public long LinkID;
        }

        // ----- Class to track the history of the views in this form instance.
        private class ItemLookupHistory
        {
            public string HistoryDisplay;
            public LookupMethods LookupType;
            public string LookupData;
        }

        // ----- Class to track items in the MatchingItems list.
        private class MatchingItemData
        {
            public long ItemID; // NamedItem.ID
            public string Title;
            public string Subtitle;
            public string Author;
            public string MediaType;
            public string CallNumber;

            public override string ToString()
            {
                // ----- Build a simple display string.
                if ((Subtitle == null) || (Subtitle.Length == 0))
                    return Title + ", by " + Author;
                else
                    return Title + ": " + Subtitle + ", by " + Author;
            }
        }

        // ----- Class-level member variables.
        private bool UseLC;
        private Dictionary<long, SingleItemLink> ItemLinkSet;
        private Stack<ItemLookupHistory> LookupHistorySet;
        private long LimitByMedia;
        private long LimitByLocation;
        private int SecondItemRow;

        internal void InitiateSearch(LookupMethods searchType, string searchText,
            long limitMedia, long limitLocation)
        {
            // ----- Start a new item search.
            LimitByMedia = limitMedia;
            LimitByLocation = limitLocation;

            // ----- Determine if Dewey or LC is used, and other initialization.
            UseLC = GetBoolean(GetSystemValue("UseLC"));
            ItemLinkSet = new Dictionary<long, SingleItemLink>();
            LookupHistorySet = new Stack<ItemLookupHistory>();

            // ----- Perform the first search.
            if (PerformLookup(searchType, searchText, true) == false)
            {
                MessageBox.Show("Your search resulted in no matches.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                ItemLinkSet = null;
                LookupHistorySet = null;
                return;
            }

            // ----- Add a history item.
            AddLookupHistory(searchType, searchText);

            // ----- Show the results to the user.
            this.ShowDialog();
            ItemLinkSet = null;
            LookupHistorySet = null;
        }

        private string HTMLEncode(string origText)
        {
            // ----- Prepare some text for use in a web page.
            string result;

            result = origText.Replace("&", "&amp;");
            result = result.Replace("<", "&lt;");
            result = result.Replace(">", "&gt;");
            return result;
        }

        private void AddLookupHistory(LookupMethods searchType, string searchText)
        {
            // ----- Add an item to the lookup history.
            ItemLookupHistory newHistory;
            string displayText;

            // ----- Build the text for display in the new item.
            displayText = BuildDisplayText(searchType, searchText);

            // ----- Build the new history item.
            newHistory = new ItemLookupHistory();
            newHistory.LookupType = searchType;
            newHistory.LookupData = searchText;
            newHistory.HistoryDisplay = displayText;
            LookupHistorySet.Push(newHistory);

            // ----- Update the back button.
            RefreshBackButtons();
        }

        private string BuildDisplayText(LookupMethods searchType, string searchText)
        {
            // ----- Add an item to the lookup history.
            string displayText;
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;

            // ----- Build the text for display in the new item.
            switch (searchType)
            {
                case LookupMethods.ByTitle:
                    // ----- Search by title text.
                    displayText = "Titles matching '" + searchText + "'";
                    break;
                case LookupMethods.ByAuthor:
                    // ----- Search by author name text.
                    displayText = "Authors matching '" + searchText + "'";
                    break;
                case LookupMethods.BySubject:
                    // ----- Search by subject name text.
                    displayText = "Subjects matching '" + searchText + "'";
                    break;
                case LookupMethods.ByKeywordAny:
                    // ----- Search by keyword (any).
                    displayText = "Items with any keyword(s): " + searchText;
                    break;
                case LookupMethods.ByKeywordAll:
                    // ----- Search by keyword (all).
                    displayText = "Items with all keyword(s): " + searchText;
                    break;
                case LookupMethods.ByPublisher:
                    // ----- Search by publisher name text.
                    displayText = "Publishers matching '" + searchText + "'";
                    break;
                case LookupMethods.BySeries:
                    // ----- Search by series name text.
                    displayText = "Series matching '" + searchText + "'";
                    break;
                case LookupMethods.BySeriesID:
                    // ----- Items for a specific series.
                    sqlText = "SELECT FullName FROM CodeSeries WHERE ID = @RecordID";
                    sqlRun = new DataCommand(sqlText);
                    sqlRun.AddLong("RecordID", searchText);
                    try
                    {
                        displayText = "Items from series '" +
                            DBGetText(ExecuteSQLReturn(sqlRun)) + "'";
                    }
                    catch (Exception ex)
                    {
                        GeneralError("ItemLookup.BuildDisplayText.Series", ex);
                        return "Error";
                    }
                    break;
                case LookupMethods.ByBarcode:
                    // ----- Item with a copy of a specific barcode.
                    displayText = "Item with copy barcode '" + searchText + "'";
                    break;
                case LookupMethods.ByDatabaseID:
                    // ----- Item with a specific ID.
                    sqlText = "SELECT Title FROM NamedItem WHERE ID = @RecordID";
                    sqlRun = new DataCommand(sqlText);
                    sqlRun.AddLong("RecordID", searchText);
                    try
                    {
                        displayText = "Item with title '" +
                            DBGetText(ExecuteSQLReturn(sqlRun)) + "'";
                    }
                    catch (Exception ex)
                    {
                        GeneralError("ItemLookup.BuildDisplayText.Database", ex);
                        return "Error";
                    }
                    break;
                case LookupMethods.ByAuthorID:
                    // ----- Items for a specific author.
                    sqlText = "SELECT * FROM Author WHERE ID = @RecordID";
                    sqlRun = new DataCommand(sqlText);
                    sqlRun.AddLong("RecordID", searchText);
                    try
                    {
                        dbInfo = CreateDataTable(sqlRun);
                    }
                    catch (Exception ex)
                    {
                        GeneralError("ItemLookup.BuildDisplayText.Author", ex);
                        return "Error";
                    }
                    displayText = "Items by author/name '" +
                        General.FormatAuthorName(dbInfo.Rows[0]) + "'";
                    dbInfo.Dispose();
                    break;
                case LookupMethods.BySubjectID:
                    // ----- Items for a specific subject.
                    sqlText = "SELECT FullName FROM Subject WHERE ID = @RecordID";
                    sqlRun = new DataCommand(sqlText);
                    sqlRun.AddLong("RecordID", searchText);
                    try
                    {
                        displayText = "Items for subject '" +
                            DBGetText(ExecuteSQLReturn(sqlRun)) + "'";
                    }
                    catch (Exception ex)
                    {
                        GeneralError("ItemLookup.BuildDisplayText.Subject", ex);
                        return "Error";
                    }
                    break;
                case LookupMethods.ByPublisherID:
                    // ----- Items for a specific publisher.
                    sqlText = "SELECT FullName FROM Publisher WHERE ID = @RecordID";
                    sqlRun = new DataCommand(sqlText);
                    sqlRun.AddLong("RecordID", searchText);
                    try
                    {
                        displayText = "Items for publisher '" +
                            DBGetText(ExecuteSQLReturn(sqlRun)) + "'";
                    }
                    catch (Exception ex)
                    {
                        GeneralError("ItemLookup.BuildDisplayText.Publisher", ex);
                        return "Error";
                    }
                    break;
                default:
                    displayText = "Error";
                    break;
            }

            return displayText;
        }

        private bool PerformLookup(LookupMethods searchType, string searchText, bool breakOnFailure)
        {
            // ----- Perform a search based on the user's request.
            string resultType;
            int matches;
            string firstNamePattern;
            string lastNamePattern;
            string subjectPattern;
            string publishPattern;
            string seriesPattern;
            string titlePattern;
            string holdText;
            MatchingItemData matchEntry;
            LibraryDataContext libraryDC;
            IQueryable<long> idQuery;
            List<string> keywordSet;

            // ----- Connect to the LINQ engine.
            this.Cursor = Cursors.WaitCursor;
            libraryDC = new LibraryDataContext(Properties.Settings.Default.DBConnection);

            if ((LimitByMedia == -1L) & (LimitByLocation == -1L))
            {
                LabelSpecialLimit.Visible = false;
            }
            else if (LimitByMedia == -1L)
            {
                LabelSpecialLimit.Text = "Limited by location.";
                LabelSpecialLimit.Visible = true;
            }
            else if (LimitByLocation == -1L)
            {
                LabelSpecialLimit.Text = "Limited by media type.";
                LabelSpecialLimit.Visible = true;
            }
            else
            {
                LabelSpecialLimit.Text = "Limited by location and media type.";
                LabelSpecialLimit.Visible = true;
            }

            // ----- Different code for each lookup method.
            if (searchType == LookupMethods.ByAuthor)
            {
                // ----- Prepare the query for an author lookup.
                lastNamePattern = searchText.GetSubStr(",", 1).Trim();
                firstNamePattern = searchText.GetSubStr(",", 2).Trim();
                if ((lastNamePattern.Contains("*") == false) & (lastNamePattern.Length != 0))
                    lastNamePattern += "*";
                if ((firstNamePattern.Contains("*") == false) & (firstNamePattern.Length != 0))
                    firstNamePattern += "*";
                var authorQuery = from au in libraryDC.QAuthors
                                  select au;
                if (lastNamePattern.Length != 0)
                {
                    lastNamePattern = lastNamePattern.Replace("*", "%");
                    authorQuery = from au in authorQuery
                                  where SqlMethods.Like(au.LastName, lastNamePattern)
                                  select au;
                }
                if (firstNamePattern.Length != 0)
                {
                    firstNamePattern = firstNamePattern.Replace("*", "%");
                    authorQuery = from au in authorQuery
                                  where SqlMethods.Like(au.FirstName, firstNamePattern)
                                  select au;
                }
                authorQuery = from au in authorQuery
                              orderby au.LastName, au.FirstName
                              select au;
                resultType = "List";

                // ----- Run the query and store the results.
                matches = 0;
                MatchingGeneral.Items.Clear();
                foreach (var authorItem in authorQuery)
                {
                    // ----- Limit to the specified maximum records.
                    if (matches >= SearchMatchLimit)
                    {
                        matches += 1;
                        break;
                    }

                    // ----- Author list item.
                    MatchingGeneral.Items.Add(new ListItemData(
                        authorItem.FormatAuthorName(), authorItem.ID));
                    matches += 1;
                }
            }
            else if (searchType == LookupMethods.BySubject)
            {
                // ----- Prepare the query for a subject lookup.
                subjectPattern = searchText.Trim();
                if (subjectPattern.Contains("*") == false)
                    subjectPattern += "*";
                subjectPattern = subjectPattern.Replace("*", "%");
                var subjectQuery = from sb in libraryDC.QSubjects
                                   where SqlMethods.Like(sb.FullName, subjectPattern)
                                   orderby sb.FullName
                                   select sb;
                resultType = "List";

                // ----- Run the query and store the results.
                matches = 0;
                MatchingGeneral.Items.Clear();
                foreach (var subjectItem in subjectQuery)
                {
                    // ----- Limit to the specified maximum records.
                    if (matches >= SearchMatchLimit)
                    {
                        matches += 1;
                        break;
                    }

                    // ----- General list item.
                    MatchingGeneral.Items.Add(new ListItemData(
                        subjectItem.FullName, subjectItem.ID));
                    matches += 1;
                }
            }
            else if (searchType == LookupMethods.ByPublisher)
            {
                // ----- Prepare the query for a publisher lookup.
                publishPattern = searchText.Trim();
                if (publishPattern.Contains("*") == false)
                    publishPattern += "*";
                publishPattern = publishPattern.Replace("*", "%");
                var publisherQuery = from pb in libraryDC.QPublishers
                                     where SqlMethods.Like(pb.FullName, publishPattern)
                                     orderby pb.FullName
                                     select pb;
                resultType = "List";

                // ----- Run the query and store the results.
                matches = 0;
                MatchingGeneral.Items.Clear();
                foreach (var publishItem in publisherQuery)
                {
                    // ----- Limit to the specified maximum records.
                    if (matches >= SearchMatchLimit)
                    {
                        matches += 1;
                        break;
                    }

                    // ----- General list item.
                    MatchingGeneral.Items.Add(new ListItemData(
                        publishItem.FullName, publishItem.ID));
                    matches += 1;
                }
            }
            else if (searchType == LookupMethods.BySeries)
            {
                // ----- Prepare the query for a series lookup.
                seriesPattern = searchText.Trim();
                if (seriesPattern.Contains("*") == false)
                    seriesPattern += "*";
                seriesPattern = seriesPattern.Replace("*", "%");
                var seriesQuery = from se in libraryDC.QCodeSeries
                                  where SqlMethods.Like(se.FullName, seriesPattern)
                                  orderby se.FullName
                                  select se;
                resultType = "List";

                // ----- Run the query and store the results.
                matches = 0;
                MatchingGeneral.Items.Clear();
                foreach (var seriesItem in seriesQuery)
                {
                    // ----- Limit to the specified maximum records.
                    if (matches >= SearchMatchLimit)
                    {
                        matches += 1;
                        break;
                    }

                    // ----- General list item.
                    MatchingGeneral.Items.Add(new ListItemData(
                        seriesItem.FullName, seriesItem.ID));
                    matches += 1;
                }
            }
            else if ((searchType == LookupMethods.ByDatabaseID) |
                (searchType == LookupMethods.ByBarcode))
            {
                // ----- Lookup by specific ID value.
                if (searchType == LookupMethods.ByDatabaseID)
                {
                    // ----- Lookup by database ID of the Named Item.
                    idQuery = from ni in libraryDC.QNamedItems
                              where ni.ID == Convert.ToInt64(searchText)
                              select ni.ID;
                }
                else
                {
                    // ----- Lookup by the barcode of a copy.
                    idQuery = from ic in libraryDC.QItemCopies
                              where ic.Barcode == searchText.Trim()
                              select ic.ItemID;
                }
                resultType = "Single";

                // ----- Prepare for a single-item display.
                if (idQuery.Count() == 0)
                {
                    if (breakOnFailure == true)
                    {
                        // ----- This was the initial search, and it failed.
                        this.Cursor = Cursors.Default;
                        return false;
                    }
                    else
                    {
                        SingleItemDetail.Text = "Error";
                        ItemLinks.Items.Clear();
                    }
                    matches = 0;
                }
                else
                {
                    SingleItemDetail.DocumentText =
                        BuildHTMLAndLinks(idQuery.ToArray()[0]);
                    matches = 1;
                }
            }
            else
            {
                // ----- Some item-type lookup. Add in the media and
                //       location limits if needed.
                var itemQuery = from ni in libraryDC.QNamedItems
                                select ni;
                if (LimitByMedia != -1L)
                {
                    // ----- Limit to a specific media type.
                    itemQuery = from ni in itemQuery
                                where ni.MediaType == LimitByMedia
                                select ni;
                }
                if (LimitByLocation != -1L)
                {
                    // ----- Limit to a specific location for item copies.
                    itemQuery = from ni in itemQuery
                                let inLoc = (
                                    from ic in ni.QItemCopies
                                    where ic.Location == LimitByLocation
                                    select ic.ID)
                                where inLoc.Count() > 0
                                select ni;
                }

                // ----- Alter the query based on the lookup type.
                if (searchType == LookupMethods.ByTitle)
                {
                    // ----- Perform a title search.
                    titlePattern = searchText.Trim();
                    if (titlePattern.Contains("*") == false)
                        titlePattern += "*";
                    titlePattern = titlePattern.Replace("*", "%");
                    itemQuery = from ni in itemQuery
                                where SqlMethods.Like(ni.Title, titlePattern)
                                select ni;
                }
                else if (searchType == LookupMethods.ByKeywordAny)
                {
                    // ----- Perform a keyword (any) search.
                    keywordSet = new List<string>();
                    keywordSet.AddRange(searchText.ToUpper().Split(new[] {","}, StringSplitOptions.None));
                    itemQuery = from ni in itemQuery
                                let keySet = ((
                                        from ik in ni.QItemKeywords
                                        select ik.QKeyword.FullName.ToUpper())
                                    .Any(searchTerm => keywordSet.Contains(searchTerm)))
                                where keySet == true
                                select ni;
                }
                else if (searchType == LookupMethods.ByKeywordAll)
                {
                    // ----- Perform a keyword (all) search.
                    keywordSet = new List<string>();
                    keywordSet.AddRange(searchText.ToUpper().Split(
                        new[] {","}, StringSplitOptions.None));
                    itemQuery = from ni in itemQuery
                                let keySet = ((
                                        from ik in ni.QItemKeywords
                                        select ik.QKeyword.FullName.ToUpper())
                                        .Count(searchTerm => keywordSet.Contains(searchTerm)))
                                where keySet == keywordSet.Count
                                select ni;
                }
                else if (searchType == LookupMethods.BySeriesID)
                {
                    // ----- Return all items within a series.
                    itemQuery = from ni in itemQuery
                                where ni.Series == Convert.ToInt64(searchText)
                                select ni;
                }
                else if (searchType == LookupMethods.ByAuthorID)
                {
                    // ----- Return all items by an author.
                    itemQuery = from ni in itemQuery
                                let aiSet = (
                                    from au in ni.QItemAuthors
                                    where au.AuthorID == Convert.ToInt64(searchText)
                                    select au.AuthorID)
                                where aiSet.Count() > 0
                                select ni;
                }
                else if (searchType == LookupMethods.BySubjectID)
                {
                    // ----- Return all items on a certain subject.
                    itemQuery = from ni in itemQuery
                                let subset = (
                                    from sb in ni.QItemSubjects
                                    where sb.SubjectID == Convert.ToInt64(searchText)
                                    select sb.SubjectID)
                                where subset.Count() > 0
                                select ni;
                }
                else if (searchType == LookupMethods.ByPublisherID)
                {
                    // ----- Return all items by a publisher.
                    itemQuery = from ni in itemQuery
                                where ni.Publisher == Convert.ToInt64(searchText)
                                select ni;
                }
                resultType = "Item";

                // ----- All item queries sort by title.
                itemQuery = from ni in itemQuery
                            orderby ni.Title, ni.Subtitle
                            select ni;

                // ----- Prepare for item matches.
                MatchingItems.Items.Clear();
                matches = 0;
                foreach (var itemItem in itemQuery)
                {
                    // ----- Limit to the specified maximum records.
                    if (matches >= SearchMatchLimit)
                    {
                        matches += 1;
                        break;
                    }

                    // ----- Store a new display item.
                    matchEntry = new MatchingItemData();
                    matchEntry.ItemID = itemItem.ID;
                    matchEntry.Title = itemItem.Title;
                    matchEntry.Subtitle = itemItem.Subtitle;
                    if (itemItem.QItemAuthors.Count > 0)
                    {
                        // ----- Get the top author in preference order.
                        var topAuthor = from au in itemItem.QItemAuthors
                                        orderby au.Sequence
                                        select au.QAuthor;
                        matchEntry.Author = topAuthor.ElementAt(0).FormatAuthorName();
                        if (itemItem.QItemAuthors.Count > 1)
                            matchEntry.Author += ", et. al.";
                    }
                    matchEntry.MediaType = itemItem.QCodeMediaType.FullName;
                    holdText = "";
                    if (UseLC == true)
                    {
                        if (itemItem.LC != null)
                            holdText = itemItem.LC.GetSubStr("/", 1);
                    }
                    else
                    {
                        if (itemItem.Dewey != null)
                            holdText = itemItem.Dewey.GetSubStr("/", 1);
                    }
                    matchEntry.CallNumber = holdText;

                    // ----- Store the match in the list.
                    MatchingItems.Items.Add(matchEntry);
                    matches += 1;
                }
            }

            // ----- Error if nothing matched the initial search.
            if ((matches == 0) & (breakOnFailure == true))
            {
                this.Cursor = Cursors.Default;
                return false;
            }

            // ----- Update the display.
            PanelMatches.Visible = false;
            PanelItems.Visible = false;
            PanelOneItem.Visible = false;
            if (resultType == "Single")
            {
                LabelSingleItem.Text = BuildDisplayText(searchType, searchText);
                PanelOneItem.Visible = true;
                SingleItemDetail.Focus();
            }
            else if (resultType == "Item")
            {
                // ----- Display the results.
                LabelMatchingItems.Text = BuildDisplayText(searchType, searchText);
                if (matches == 1)
                    LabelItemsCount.Text = "1 matching item found.";
                else if (matches > SearchMatchLimit)
                    LabelItemsCount.Text = "Limited to first " +
                        SearchMatchLimit.ToString() + " matching items.";
                else
                    LabelItemsCount.Text = matches.ToString() + " matching items found.";
                ActItemLookup.Enabled = false;
                PanelItems.Visible = true;
            }
            else
            {
                // ----- Display the results.
                LabelMatchingGeneral.Text = BuildDisplayText(searchType, searchText);
                if (matches == 1)
                    LabelMatchCount.Text = "1 match found.";
                else if (matches > SearchMatchLimit)
                    LabelMatchCount.Text = "Limited to first " +
                        SearchMatchLimit.ToString() + " matches.";
                else
                    LabelMatchCount.Text = matches.ToString() + " matches found.";
                ActGeneralLookup.Enabled = false;
                PanelMatches.Visible = true;
            }

            // ----- Finished.
            libraryDC.Dispose();
            this.Cursor = Cursors.Default;
            return true;
        }

        private void RefreshBackButtons()
        {
            // ----- Refresh the display of the back button items.
            int counter;
            IEnumerator scanSet;
            ItemLookupHistory scanHistory;
            ToolStripMenuItem whichMenu;

            // ***** Insert Code Snippet #3 here.

            // ----- See if there is any history.
            if (LookupHistorySet.Count <= 1)
            {
                // ----- No extra history available.
                ActBack.Enabled = false;
                ActBackMenu.Enabled = false;
            }
            else
            {
                // ----- Build the history. Skip the first item as it is the
                //       one that is displayed.
                counter = 1;
                scanSet = LookupHistorySet.GetEnumerator();
                scanSet.MoveNext(); // Skip first entry.
                while (scanSet.MoveNext())
                {
                    scanHistory = (ItemLookupHistory)scanSet.Current;
                    switch (counter)
                    {
                        case 1:
                            whichMenu = BackMenu1;
                            break;
                        case 2:
                            whichMenu = BackMenu2;
                            break;
                        case 3:
                            whichMenu = BackMenu3;
                            break;
                        case 4:
                            whichMenu = BackMenu4;
                            break;
                        case 5:
                            whichMenu = BackMenu5;
                            break;
                        case 6:
                            whichMenu = BackMenu6;
                            break;
                        case 7:
                            whichMenu = BackMenu7;
                            break;
                        case 8:
                            whichMenu = BackMenu8;
                            break;
                        case 9:
                            whichMenu = BackMenu9;
                            break;
                        default:
                            whichMenu = BackMenu10;
                            break;
                    }

                    // ***** Remove the next code line, and then
                    // ***** Insert Code Snippet #2 here.
                    whichMenu.Text = scanHistory.HistoryDisplay;

                    whichMenu.Visible = true;
                    counter += 1;
                    if (counter > 10)
                        break;
                }

                // ----- Hide unused menu items.
                while (counter <= 10)
                {
                    switch (counter)
                    {
                        case 1:
                            whichMenu = BackMenu1;
                            break;
                        case 2:
                            whichMenu = BackMenu2;
                            break;
                        case 3:
                            whichMenu = BackMenu3;
                            break;
                        case 4:
                            whichMenu = BackMenu4;
                            break;
                        case 5:
                            whichMenu = BackMenu5;
                            break;
                        case 6:
                            whichMenu = BackMenu6;
                            break;
                        case 7:
                            whichMenu = BackMenu7;
                            break;
                        case 8:
                            whichMenu = BackMenu8;
                            break;
                        case 9:
                            whichMenu = BackMenu9;
                            break;
                        default:
                            whichMenu = BackMenu10;
                            break;
                    }
                    whichMenu.Visible = false;
                    counter += 1;
                }
                ActBack.Enabled = true;
                ActBackMenu.Enabled = true;
            }

            // ***** Insert Code Snippet #4 here.

        }

        private string BuildHTMLAndLinks(long itemID)
        {
            // ----- Given the ID of a library item, build the HTML text display
            //       that shows all of the relevant detail.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            System.Text.StringBuilder detailBody = new System.Text.StringBuilder();
            DataRow oneRow;
            string holdText;
            long entryID;
            int counter;
            SingleItemLink newLink;

            this.Cursor = Cursors.WaitCursor;

            // ----- Initialize.
            entryID = 0L;
            ItemLinkSet.Clear();
            ItemLinks.Items.Clear();

            // ----- The main HTML headers are stored in a text file resource.
            //       Add it to the body content now.
            detailBody.Append(Properties.Resources.ItemLookupBody);

            // ----- Add in the title.
            sqlText = "SELECT Title, Subtitle FROM NamedItem WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", itemID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("ItemLookup.BuildHTMLAndLinks.Title", ex);
                return "Error";
            }
            oneRow = dbInfo.Rows[0];
            detailBody.AppendLine("<h1>" + HTMLEncode(DBGetText(oneRow["Title"])) + "</h1>");

            // ----- Add in the subtitle if available.
            if (Convert.IsDBNull(oneRow["Subtitle"]) == false)
            {
                detailBody.AppendLine("<h3>" + HTMLEncode(DBGetText(oneRow["Subtitle"])) + "</h3>");
            }
            dbInfo.Dispose();

            // ----- Add in the authors and names.
            sqlText = "SELECT AU.*, IA.Sequence, CA.FullName FROM ItemAuthor AS IA " +
                "INNER JOIN Author AS AU ON IA.AuthorID = AU.ID INNER JOIN " +
                "CodeAuthorType AS CA ON IA.AuthorType = CA.ID " +
                "WHERE IA.ItemID = @RecordID ORDER BY IA.Sequence";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", itemID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("ItemLookup.BuildHTMLAndLinks.Authors", ex);
                return "Error";
            }
            if (dbInfo.Rows.Count > 0)
            {
                // ----- There are authors or names. Display a heading.
                detailBody.AppendLine("<h2>Authors and Names</h2>");
                foreach (DataRow dbRow in dbInfo.Rows)
                {
                    // ----- Add in this one author name.
                    holdText = dbRow.FormatAuthorName();
                    entryID += 1;
                    detailBody.AppendLine("<p><a href=\"library://" + entryID + "\">" +
                        HTMLEncode(holdText + " [" + DBGetText(dbRow["FullName"]) + "]") + "</a></p>");

                    // ----- Add in an author link.
                    newLink = new SingleItemLink();
                    newLink.LinkType = LookupMethods.ByAuthorID;
                    newLink.LinkID = DBGetLong(dbRow["ID"]);
                    ItemLinkSet.Add(entryID, newLink);
                    ItemLinks.Items.Add(new ListItemData("Author: " + holdText +
                        " [" + DBGetText(dbRow["FullName"]) + "]", entryID));
                }
            }
            dbInfo.Dispose();

            // ----- Retrieve most of the remaining fields.
            sqlText = "SELECT NI.Dewey, NI.Dewey, NI.LC, NI.Description, NI.Publisher, " +
                "PB.FullName AS PubName, NI.Edition, NI.Copyright, NI.ISxN, NI.Series, " +
                "CS.FullName AS SeriesName, CMT.FullName AS MediaName " +
                "FROM NamedItem AS NI LEFT JOIN Publisher AS PB ON NI.Publisher = PB.ID " +
                "LEFT JOIN CodeSeries AS CS ON NI.Series = CS.ID " +
                "INNER JOIN CodeMediaType AS CMT ON NI.MediaType = CMT.ID " +
                "WHERE NI.ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", itemID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("ItemLookup.BuildHTMLAndLinks.Fields", ex);
                return "Error";
            }
            oneRow = dbInfo.Rows[0];

            // ----- Add in the call number information.
            detailBody.AppendLine("<h2>Call Number</h2>");
            holdText = "No call number available.";
            if (UseLC)
            {
                // ----- Show the Library of Congress number.
                if (Convert.IsDBNull(oneRow["LC"]) == false)
                    holdText = DBGetText(oneRow["LC"]);
            }
            else
            {
                // ----- Show the Dewey Decimal number.
                if (Convert.IsDBNull(oneRow["Dewey"]) == false)
                    holdText = DBGetText(oneRow["Dewey"]);
            }
            for (counter = 1; counter <= CountSubStr(holdText, "/") + 1; counter++)
            {
                // ----- Add one line of the number.
                detailBody.AppendLine("<p>" +
                    HTMLEncode(holdText.GetSubStr("/", counter).Trim()) + "</p>");
            }

            // ----- Display the media type name.
            detailBody.AppendLine("<p><i>" +
                HTMLEncode(DBGetText(oneRow["MediaName"])) + "</i></p>");

            // ----- Add in the item description.
            if (Convert.IsDBNull(oneRow["Description"]) == false)
            {
                detailBody.AppendLine("<h2>Description</h2>");
                detailBody.AppendLine("<p>" +
                    HTMLEncode(DBGetText(oneRow["Description"])) + "</p>");
            }

            // ----- Add in the publishing information.
            counter = 0;
            if (Convert.IsDBNull(oneRow["PubName"]) == false)
                counter += 1;
            if (Convert.IsDBNull(oneRow["Edition"]) == false)
                counter += 1;
            if (Convert.IsDBNull(oneRow["Copyright"]) == false)
                counter += 1;
            if (Convert.IsDBNull(oneRow["ISxN"]) == false)
                counter += 1;
            if (counter > 0)
            {
                // ----- There is some publishing information to display.
                detailBody.AppendLine("<h2>Publishing Information</h2>");

                // ----- Add the publisher name, if available.
                if (Convert.IsDBNull(oneRow["PubName"]) == false)
                {
                    entryID += 1;
                    holdText = DBGetText(oneRow["PubName"]);
                    detailBody.AppendLine("<p><a href=\"library://" +
                        entryID + "\">" + HTMLEncode(holdText) + "</a></p>");

                    // ----- Add in a publisher link.
                    newLink = new SingleItemLink();
                    newLink.LinkType = LookupMethods.ByPublisherID;
                    newLink.LinkID = DBGetLong(oneRow["Publisher"]);
                    ItemLinkSet.Add(entryID, newLink);
                    ItemLinks.Items.Add(new ListItemData(
                        "Publisher: " + holdText, entryID));
                }

                // ----- Add the edition number, if available.
                if (Convert.IsDBNull(oneRow["Edition"]) == false)
                    detailBody.AppendLine("<p>Edition: " +
                        HTMLEncode(DBGetText(oneRow["Edition"])) + "</p>");

                // ----- Add the copyright year, if available.
                if (Convert.IsDBNull(oneRow["Copyright"]) == false)
                    detailBody.AppendLine("<p>Copyright: " +
                        DBGetInteger(oneRow["Copyright"]).ToString() + "</p>");

                // ----- Add the ISBN/ISSN number, if available.
                if (Convert.IsDBNull(oneRow["ISxN"]) == false)
                    detailBody.AppendLine("<p>ISBN / ISSN: " +
                        HTMLEncode(DBGetText(oneRow["ISxN"])) + "</p>");
            }

            // ----- Add in the series information.
            if (Convert.IsDBNull(oneRow["SeriesName"]) == false)
            {
                detailBody.AppendLine("<h2>Related Series</h2>");

                entryID += 1;
                holdText = DBGetText(oneRow["SeriesName"]);
                detailBody.AppendLine("<p><a href=\"library://" +
                    entryID + "\">" + HTMLEncode(holdText) + "</a></p>");

                // ----- Add in a series link.
                newLink = new SingleItemLink();
                newLink.LinkType = LookupMethods.BySeriesID;
                newLink.LinkID = DBGetLong(oneRow["Series"]);
                ItemLinkSet.Add(entryID, newLink);
                ItemLinks.Items.Add(new ListItemData(
                    "Series: " + holdText, entryID));
            }
            dbInfo.Dispose();

            // ----- Add in the related subjects.
            sqlText = "SELECT SB.* FROM ItemSubject AS IB INNER JOIN Subject AS SB " +
                "ON IB.SubjectID = SB.ID WHERE IB.ItemID = @RecordID " +
                "ORDER BY SB.FullName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", itemID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("ItemLookup.BuildHTMLAndLinks.Subjects", ex);
                return "Error";
            }
            if (dbInfo.Rows.Count > 0)
            {
                // ----- There are subjects. Display a heading.
                detailBody.AppendLine("<h2>Related Subjects</h2>");

                foreach (DataRow dbRow in dbInfo.Rows)
                {
                    // ----- Add in this one subject name.
                    entryID += 1;
                    holdText = DBGetText(dbRow["FullName"]);
                    detailBody.AppendLine("<p><a href=\"library://" +
                        entryID + "\">" + HTMLEncode(holdText) + "</a></p>");

                    // ----- Add in a subject link.
                    newLink = new SingleItemLink();
                    newLink.LinkType = LookupMethods.BySubjectID;
                    newLink.LinkID = DBGetLong(dbRow["ID"]);
                    ItemLinkSet.Add(entryID, newLink);
                    ItemLinks.Items.Add(new ListItemData(
                        "Subject: " + holdText, entryID));
                }
            }
            dbInfo.Dispose();

            // ----- See how many copies there are.
            sqlText = "SELECT COUNT(*) FROM ItemCopy WHERE ItemID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", itemID);
            try
            {
                counter = DBGetInteger(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("ItemLookup.BuildHTMLAndLinks.Copies", ex);
                return "Error";
            }

            // ----- Add in the copies information if needed.
            if (counter > 0)
            {
                // ----- Introduce the copies.
                if (counter == 1)
                    holdText = "1 copy";
                else
                    holdText = counter.ToString() + " copies";
                detailBody.AppendLine("<h2>Copies</h2>");
                detailBody.AppendLine("<p>" + holdText + "</p>");

                // ----- Build a table that lists the copies. Use a data table
                //       since the call to GetCopyDisposition() will open
                //       a separate data reader.
                detailBody.AppendLine("<table cellspacing=\"0\" cellpadding=\"3\">");
                detailBody.AppendLine("<tr><th>Copy</th><th>Location</th>" +
                    "<th>Status</th></tr>");
                sqlText = "SELECT IC.ID, IC.CopyNumber, CL.FullName FROM ItemCopy AS IC " +
                    "LEFT JOIN CodeLocation AS CL ON IC.Location = CL.ID " +
                    "WHERE IC.ItemID = @RecordID ORDER BY IC.CopyNumber";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", itemID);
                try
                {
                    dbInfo = CreateDataTable(sqlRun);
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    GeneralError("ItemLookup.BuildHTMLAndLinks.OneCopy", ex);
                    return "Error";
                }
                foreach (DataRow dbRow in dbInfo.Rows)
                {
                    // ----- Add the copy number.
                    detailBody.Append("<tr><td>" +
                        DBGetInteger(dbRow["CopyNumber"]).ToString() + "</td>");

                    // ----- Add the location.
                    if (Convert.IsDBNull(dbRow["FullName"]) == true)
                        holdText = "Not listed";
                    else
                        holdText = HTMLEncode(DBGetText(dbRow["FullName"]));
                    detailBody.Append("<td>" + holdText + "</td>");

                    // ----- Add the disposition.
                    detailBody.AppendLine("<td>" + HTMLEncode(
                        GetCopyDisposition(DBGetLong(dbRow["ID"]))) + "</td></tr>");
                }
                dbInfo.Dispose();
                dbInfo = null;
                detailBody.AppendLine("</table>");
            }

            // ----- Finished.
            detailBody.AppendLine("</body>");
            detailBody.AppendLine("</html>");
            return detailBody.ToString();
        }

        private void MatchingGeneral_DoubleClick(object sender, EventArgs e)
        {
            // ----- Same as a click on the Lookup button.
            ActGeneralLookup.PerformClick();
        }

        private void ActGeneralLookup_Click(object sender, EventArgs e)
        {
            // ----- Look up the items associated with the selected match.
            LookupMethods baseType;
            LookupMethods searchType = LookupMethods.ByTitle;
            long itemID;

            // ----- Ignore if no match is selected.
            if (MatchingGeneral.SelectedIndex == -1)
                return;
            itemID = (long)(ListItemData)MatchingGeneral.SelectedItem;

            // ----- Determine the type of search.
            baseType = LookupHistorySet.Peek().LookupType;
            switch (baseType)
            {
                case LookupMethods.ByAuthor:
                    searchType = LookupMethods.ByAuthorID;
                    break;
                case LookupMethods.BySubject:
                    searchType = LookupMethods.BySubjectID;
                    break;
                case LookupMethods.ByPublisher:
                    searchType = LookupMethods.ByPublisherID;
                    break;
                case LookupMethods.BySeries:
                    searchType = LookupMethods.BySeriesID;
                    break;
            }

            // ----- Perform the lookup.
            if (PerformLookup(searchType, itemID.ToString(), false) == false)
                return;

            // ----- Store the history.
            AddLookupHistory(searchType, itemID.ToString());
        }

        private void MatchingItems_DoubleClick(object sender, EventArgs e)
        {
            // ----- Same as the lookup button.
            ActItemLookup.PerformClick();
        }

        private void ActItemLookup_Click(object sender, EventArgs e)
        {
            // ----- Look up the item with the selected ID.
            long itemID;

            // ----- Ignore if no match is selected.
            if (MatchingItems.SelectedIndex == -1)
                return;
            itemID = ((MatchingItemData)MatchingItems.SelectedItem).ItemID;

            // ----- Perform the lookup.
            if (PerformLookup(LookupMethods.ByDatabaseID, itemID.ToString(), false) == false)
                return;

            // ----- Store the history.
            AddLookupHistory(LookupMethods.ByDatabaseID, itemID.ToString());
        }

        private void SingleItemDetail_Navigating(object sender, WebBrowserNavigatingEventArgs e)
        {
            // ----- Follow the clicked link.
            if (e.Url.Scheme == "library")
                FollowItemLink(Convert.ToInt64(e.Url.Host));
        }

        private void BackMenuItems_Click(object sender, EventArgs e)
        {
            // ----- One of the back menu items was clicked.
            long whichItem;
            ItemLookupHistory scanHistory;

            // ----- Determine the clicked item.
            whichItem = Convert.ToInt64(DigitsOnly(((ToolStripMenuItem)sender).Name));
            if (whichItem >= LookupHistorySet.Count)
                return;

            // ----- Get rid of the in-between items.
            for (long counter = 1L; counter <= whichItem; counter++)
                LookupHistorySet.Pop();

            // ----- Perform a lookup as requested.
            scanHistory = LookupHistorySet.Peek();
            if (PerformLookup(scanHistory.LookupType,
                    scanHistory.LookupData, false) == false)
                return;
            RefreshBackButtons();
        }

        private void ActBackMenu_Click(object sender, EventArgs e)
        {
            // ----- Show the popup menu for back items.
            BackMenu.Show(ActBackMenu, new Point(-ActBack.Width, ActBackMenu.Height));
        }

        private void ActBack_Click(object sender, EventArgs e)
        {
            // ----- Same as a click on the first menu item.
            if (LookupHistorySet.Count > 1)
                BackMenu1.PerformClick();
        }

        private void MatchingGeneral_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Activate the lookup button if needed.
            ActGeneralLookup.Enabled = (MatchingGeneral.SelectedIndex != -1);
        }

        private void MatchingGeneral_Click(object sender, EventArgs e)
        {
            // ----- Activate the lookup button if needed.
            ActGeneralLookup.Enabled = (MatchingGeneral.SelectedIndex != -1);
        }

        private void MatchingItems_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Activate the lookup button if needed.
            ActItemLookup.Enabled = (MatchingItems.SelectedIndex != -1);
        }

        private void MatchingItems_Click(object sender, EventArgs e)
        {
            // ----- Activate the lookup button if needed.
            ActItemLookup.Enabled = (MatchingItems.SelectedIndex != -1);
        }

        private void ActItemLinks_Click(object sender, EventArgs e)
        {
            // ----- Jump to the link indicated in the drop-down list.
            long entryLink;

            // ----- Ignore if no link is selected.
            if (ItemLinks.SelectedIndex == -1)
                return;
            entryLink = (long)(ListItemData)ItemLinks.SelectedItem;
            FollowItemLink(entryLink);
        }

        private void FollowItemLink(long entryID)
        {
            // ----- Given a character position in the single item text panel,
            //       follow the link indicated by that item.
            SingleItemLink scanLink;

            // ----- Access the link.
            scanLink = ItemLinkSet[entryID];
            if (scanLink == null)
                return;

            // ----- Perform a lookup as requested.
            if (PerformLookup(scanLink.LinkType,
                    scanLink.LinkID.ToString(), false) == false)
                return;

            // ----- Store the history.
            AddLookupHistory(scanLink.LinkType, scanLink.LinkID.ToString());
        }

        private void ItemLookup_KeyPress(object sender, KeyPressEventArgs e)
        {
            // ----- Treat the backspace key like a click on the Back button.
            if (e.KeyChar == (char)(Keys.Back))
            {
                if (ActBack.Enabled == true)
                    ActBack.PerformClick();
            }
        }

        private void ItemLookup_Load(object sender, EventArgs e)
        {
            // ***** Insert Code Snippet #5 here.

        }

        private void MatchingItems_DrawItem(object sender, DrawItemEventArgs e)
        {
            // ***** Insert Code Snippet #6 here.

        }
    }
}
