﻿// ----- The Library Project: License Generator
//       Start-to-Finish Visual Basic 2015
//       Copyright (c) 2015 by Tim Patrick

using System;
using System.Security.Cryptography;
using System.Windows;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using static LibraryLicensing.General;

namespace LibraryLicensing
{
    public partial class KeyLocationWindow : Window
    {
        public KeyLocationWindow()
        {
            InitializeComponent();
        }

        public bool GetKeyFolder(ref string folderPath)
        {
            // ----- Prompt the user for the key folder.
            KeyDirectory.Text = folderPath;
            RefreshFolderStatus(GetFolderStatus(folderPath));
            if (this.ShowDialog() == true)
            {
                // ----- Return the new user selection.
                folderPath = KeyDirectory.Text;
                return true;
            }
            else
                return false;
        }

        private void RefreshFolderStatus(KeyFolderStatus whichStatus)
        {
            // ----- Display a description of the current status.
            if (whichStatus == KeyFolderStatus.ValidFolder)
                StatusDetailColor.Fill = Brushes.Green;
            else
                StatusDetailColor.Fill = Brushes.Red;
            this.StatusDetailText.Content = GetFolderStatusText(whichStatus);
        }

        private void ActBrowse_Click(object sender, RoutedEventArgs e)
        {
            // ----- Let the user browse for the folder through a system dialog.
            System.Windows.Forms.FolderBrowserDialog browseKeyFolder =
                new System.Windows.Forms.FolderBrowserDialog();

            if (browseKeyFolder.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                KeyDirectory.Text = browseKeyFolder.SelectedPath;
                ActConfirm.RaiseEvent(new RoutedEventArgs(ButtonBase.ClickEvent));
            }
        }

        private void ActConfirm_Click(object sender, RoutedEventArgs e)
        {
            // ----- Update the status.
            RefreshFolderStatus(GetFolderStatus(KeyDirectory.Text));
        }

        private void ActGenerate_Click(object sender, RoutedEventArgs e)
        {
            // ----- Generate and save the keys.
            RSA twoPartKey;
            KeyFolderStatus currentStatus;
            string publicFile;
            string privateFile;

            // ----- Determine the directory and file status.
            currentStatus = GetFolderStatus(KeyDirectory.Text);
            if (currentStatus == KeyFolderStatus.NoFolder)
            {
                MessageBox.Show("Please supply a directory path.",
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                KeyDirectory.Focus();
                return;
            }

            // ----- Make sure the directory exists.
            if (currentStatus == KeyFolderStatus.InvalidFolder)
            {
                if (AttemptCreateFolder(KeyDirectory.Text) == false)
                    return;
                currentStatus = KeyFolderStatus.MissingBothFiles;
                RefreshFolderStatus(currentStatus);
            }

            // ----- Warn if either file exists.
            if (currentStatus == KeyFolderStatus.ValidFolder)
            {
                if (MessageBox.Show("One or both of the key files already exists. " +
                        "Do you really want to overwrite the existing file(s)? " +
                        "This action cannot be undone.", ProgramTitle,
                        MessageBoxButton.YesNo, MessageBoxImage.Question,
                        MessageBoxResult.No) != MessageBoxResult.Yes)
                    return;
            }

            // ----- Delete the existing files.
            try
            {
                publicFile = System.IO.Path.Combine(KeyDirectory.Text, PublicKeyFilename);
                privateFile = System.IO.Path.Combine(KeyDirectory.Text, PrivateKeyFilename);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error occurred while locating files:" +
                    Environment.NewLine + Environment.NewLine + ex.Message,
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }
            try
            {
                System.IO.File.Delete(publicFile);
                System.IO.File.Delete(privateFile);
            }
            catch
            {
                // ----- Ignore errors.
            }
            RefreshFolderStatus(KeyFolderStatus.MissingBothFiles);

            // ----- Generate the keys.
            twoPartKey = new RSACryptoServiceProvider();
            twoPartKey = RSA.Create();

            // ----- Save the public key.
            try
            {
                System.IO.File.WriteAllText(publicFile, twoPartKey.ToXmlString(false));
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error occurred while generating public key:" +
                    Environment.NewLine + Environment.NewLine + ex.Message,
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }
            RefreshFolderStatus(KeyFolderStatus.MissingPrivateFile);

            // ----- Save the private key.
            try
            {
                System.IO.File.WriteAllText(privateFile, twoPartKey.ToXmlString(true));
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error occurred while generating private key:" +
                    Environment.NewLine + Environment.NewLine + ex.Message,
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }
            RefreshFolderStatus(KeyFolderStatus.ValidFolder);

            MessageBox.Show("Key files generated successfully.",
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Information);
        }

        private void ActOK_Click(object sender, RoutedEventArgs e)
        {
            // ----- Return the key folder to the calling form.
            this.DialogResult = true;
        }

        private void ActCancel_Click(object sender, RoutedEventArgs e)
        {
            // ----- Close the form without confirming location.
            this.DialogResult = false;
        }
    }
}
