﻿// ----- The Library Project: License Generator
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.ComponentModel;
using System.Security.Cryptography;
using System.Security.Cryptography.Xml;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using System.Xml;
using System.Xml.Linq;
using static LibraryLicensing.General;

namespace LibraryLicensing
{
    public partial class MainWindow : Window
    {
        private KeyFolderStatus ActiveStatus;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void ActGenerate_Click(object sender, RoutedEventArgs e)
        {
            // ----- Generate the license key.
            XDocument licenseXMLNew;
            XmlDocument licenseXML;
            Microsoft.Win32.SaveFileDialog licenseSaveLocation;

            // ----- Make sure we have valid data for generation.
            if (VerifyFormData() == false)
                return;

            // ----- Generate the XML content.
            licenseXMLNew = BuildXmlLicenseContent();
            if (licenseXMLNew == null)
                return;

            // ----- Unfortunately, XDocuments, i.e., "the new way," do not offer
            //       an easy way to digitally sign the content. This means we
            //       need to convert it to "the old way."
            licenseXML = new XmlDocument();
            licenseXML.LoadXml(licenseXMLNew.ToString());

            // ----- Digitally sign the file.
            if (SignXmlLicenseContent(licenseXML) == false)
                return;

            // ----- Prompt the user for the output location.
            licenseSaveLocation = new Microsoft.Win32.SaveFileDialog();
            licenseSaveLocation.FileName = "LibraryLicense.lic";
            licenseSaveLocation.Filter = "License Files|*.lic|XML Files|*.xml|All Files|*.*";
            licenseSaveLocation.Title = "Save License File";
            if (licenseSaveLocation.ShowDialog() == false)
                return;

            // ----- Delete any existing license file.
            if (System.IO.File.Exists(licenseSaveLocation.FileName) == true)
            {
                try
                {
                    System.IO.File.Delete(licenseSaveLocation.FileName);
                }
                catch
                {
                    MessageBox.Show("The license could not be saved because an existing " +
                        "license file with the same name could not be replaced.",
                        ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                    return;
                }
            }

            // ----- Save the license to the file.
            try
            {
                licenseXML.Save(licenseSaveLocation.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show("The license could not be saved due to the following error:" +
                    Environment.NewLine + Environment.NewLine + ex.Message,
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return;
            }

            // ----- Finished.
            MessageBox.Show("License file saved successfully.",
                ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Information);
            LicenseeName.Focus();
        }

        private void ActLocation_Click(object sender, RoutedEventArgs e)
        {
            // ----- Prompt the user to alter the key pair.
            string useFolder;
            bool result;

            // ----- Promt the user.
            useFolder = KeyLocation.Text;
            result = (new KeyLocationWindow()).GetKeyFolder(ref useFolder);
            if (result == true)
            {
                // ----- Save the changes.
                Properties.Settings.Default.PathForKeyPair = useFolder;
                ActiveStatus = GetFolderStatus(useFolder);

                // ----- Update the display.
                KeyLocation.Text = useFolder;
                if (ActiveStatus == KeyFolderStatus.ValidFolder)
                    StatusDetailColor.Fill = Brushes.Green;
                else
                    StatusDetailColor.Fill = Brushes.Red;
                StatusDetailText.Content = GetFolderStatusText(ActiveStatus);
            }
        }

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            // ----- Prepare the form.
            string useFolder;

            // ----- Licensing defaults to today.
            LicenseDate.SelectedDate = DateTime.Today;

            // ----- Display the folder with the key.
            useFolder = Properties.Settings.Default.PathForKeyPair;
            KeyLocation.Text = useFolder;
            ActiveStatus = GetFolderStatus(useFolder);
            if (ActiveStatus == KeyFolderStatus.ValidFolder)
                StatusDetailColor.Fill = Brushes.Green;
            else
                StatusDetailColor.Fill = Brushes.Red;
            StatusDetailText.Content = GetFolderStatusText(ActiveStatus);

            // ----- If the keys are not valid, prompt immediately.
            if (ActiveStatus != KeyFolderStatus.ValidFolder)
                ActLocation.RaiseEvent(new RoutedEventArgs(ButtonBase.ClickEvent));
        }

        private bool VerifyFormData()
        {
            // ----- Confirm that all form data is valid. Return True on success.
            string[] versionParts;
            int versionPart;
            const string VersionMessage =
                "The Covered Version must contain between two and four " +
                "dot-delimited components, and each component must be a " +
                "number between 0 and 9999, or the '*' character.";

            // ----- The folder status must be valid.
            if (ActiveStatus != KeyFolderStatus.ValidFolder)
            {
                MessageBox.Show("The public-private key location and status must both be valid.",
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return false;
            }

            // ----- The version number must contain between two and four
            //       dot-delimited parts, and the parts must be numbers
            //       (0 through 9999) or "*".
            versionParts = CoveredVersion.Text.Trim().Split(new string[] {"."}, StringSplitOptions.None);
            if ((versionParts.Length < 2) | (versionParts.Length > 4))
            {
                MessageBox.Show(VersionMessage,
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                CoveredVersion.Focus();
                return false;
            }
            foreach (string scanVersion in versionParts)
            {
                if (int.TryParse(scanVersion, out versionPart) == true)
                {
                    if ((versionPart < 0) | (versionPart > 9999))
                    {
                        MessageBox.Show(VersionMessage,
                            ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                        CoveredVersion.Focus();
                        return false;
                    }
                }
                else if (scanVersion != "*")
                {
                    MessageBox.Show(VersionMessage,
                        ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                    CoveredVersion.Focus();
                    return false;
                }
            }

            // ----- Licensee Name is required.
            if (LicenseeName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Licensee Name is required.",
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                LicenseeName.Focus();
                return false;
            }

            // ----- Serial Number is required.
            if (SerialNumber.Text.Trim().Length == 0)
            {
                MessageBox.Show("Serial Number is required.",
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                SerialNumber.Focus();
                return false;
            }

            // ----- Success.
            return true;
        }

        private XDocument BuildXmlLicenseContent()
        {
            // ----- Create the XML license content from the user-supplied data.
            //       The final XML content will look something like this.
            //         <?xml version="1.0" encoding="utf-8"?>
            //         <License>
            //           <Product>Library Project</Product>
            //           <LicenseDate>1/1/2007</LicenseDate>
            //           <ExpireDate>12/31/2999</ExpireDate>
            //           <CoveredVersion>1.*</CoveredVersion>
            //           <Licensee>Your Name Here</Licensee>
            //           <SerialNumber>LIB-123456789</SerialNumber>
            //         </License>
            return new XDocument(
                new XDeclaration("1.0", "utf-8", "yes"),
                new XElement("License",
                    new XElement("Product", ApplicationProductName),
                    new XElement("LicenseDate", LicenseDate.SelectedDate.Value.ToString("M/d/yyyy")),
                    new XElement("ExpireDate", ExpirationDate.SelectedDate.Value.ToString("M/d/yyyy")),
                    new XElement("CoveredVersion", CoveredVersion.Text.Trim()),
                    new XElement("Licensee", LicenseeName.Text.Trim()),
                    new XElement("SerialNumber", SerialNumber.Text.Trim())
                )
            );
        }

        private bool SignXmlLicenseContent(XmlDocument sourceXML)
        {
            // ----- Add a digital signature to an XML document.
            string privateKeyFile;
            RSA privateKey;
            SignedXml signature;
            Reference referenceMethod;

            // ----- Locate the private key file.
            privateKeyFile = System.IO.Path.Combine(KeyLocation.Text, PrivateKeyFilename);
            if (System.IO.File.Exists(privateKeyFile) == false)
            {
                MessageBox.Show("Could not locate the private key file.",
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return false;
            }

            try
            {
                // ----- Load in the private key.
                privateKey = RSA.Create();
                privateKey.FromXmlString(System.IO.File.ReadAllText(privateKeyFile));

                // ----- Create the object that generates the signature.
                signature = new SignedXml(sourceXML);
                signature.SignedInfo.CanonicalizationMethod =
                    SignedXml.XmlDsigCanonicalizationUrl;
                signature.SigningKey = privateKey;

                // ----- The signature will appear as a <reference> element in the
                //       XML. The signature object can generate that automatically
                //       if we tell it to. It's more important later when trying to
                //       verify the signature. This transform says to ingore the
                //       signature itself when comparing the signature.
                referenceMethod = new Reference("");
                referenceMethod.AddTransform(new XmlDsigEnvelopedSignatureTransform(false));
                signature.AddReference(referenceMethod);

                // ----- Add the signature to the XML content.
                signature.ComputeSignature();
                sourceXML.DocumentElement.AppendChild(signature.GetXml());
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    "The license could not be digitally signed due to the following error:" +
                    Environment.NewLine + Environment.NewLine + ex.Message,
                    ProgramTitle, MessageBoxButton.OK, MessageBoxImage.Exclamation);
                return false;
            }

            // ----- Finished.
            return true;
        }

        private void TextFields_GotFocus(object sender, RoutedEventArgs e)
        {
            // ----- Highlight the entire text.
            ((TextBox)sender).SelectAll();
        }

        private void MainWindow_Closing(object sender, CancelEventArgs e)
        {
            // ----- Save settings changes.
            Properties.Settings.Default.Save();
        }
    }
}
