﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class Patron : Library.BaseCodeForm
    {
        private long ActiveID;
        private bool StartingOver;
        private bool AllowPatronRecord;

        public Patron()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            ActiveID = -1L;
            AllowPatronRecord = false;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManagePatrons];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManagePatrons];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManagePatrons];
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the records.
            string sqlText;
            DataCommand sqlRun;
            LibraryTransaction transInfo = null;

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete the patron?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                    DialogResult.Yes)
                return false;

            // ----- Make sure this record is not in use.
            sqlText = "SELECT COUNT(*) FROM PatronCopy WHERE Patron = @RecordID " +
                "AND Returned = 0 AND Missing = 0";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    MessageBox.Show("You cannot delete this record because " +
                        "the patron still has items checked out.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("Patron.DeleteRecord.Check", ex);
                return false;
            }

            // ----- Delete the records.
            try
            {
                transInfo = TransactionBegin();

                sqlText = "DELETE FROM PatronCopy WHERE Patron = @Record";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", recordID);
                ExecuteSQL(sqlRun, transInfo);

                sqlText = "DELETE FROM Patron WHERE ID = @Record";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddLong("RecordID", recordID);
                ExecuteSQL(sqlRun, transInfo);

                TransactionCommit(transInfo);
            }
            catch (Exception ex)
            {
                if (transInfo != null)
                    TransactionRollback(transInfo);
                GeneralError("Patron.DeleteRecord.Delete", ex);
                return false;
            }
            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record. NOTE: If you modify this routine,
            //       you must also modify the EditRecordLimited routine.
            ActiveID = recordID;
            AllowPatronRecord = true;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public long EditRecordLimited(long recordID)
        {
            // ----- Edit an existing record. NOTE: If you modify this routine,
            //       you must also modify the EditRecord routine. The Limited
            //       version does not allow access to the PatronRecord form.
            ActiveID = recordID;
            AllowPatronRecord = false;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items. Prompt the
            //       user first.
            destList.Items.Clear();
            SearchForRecord(destList, ref exceededMatches);
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            string nameText;

            // ----- Initialize.
            nameText = "";

            // ----- Retrieve the value from the database.
            sqlText = "SELECT LastName, FirstName, Active FROM Patron WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Patron.FormatRecordName", ex);
                return "Error";
            }
            if (dbInfo.Rows.Count > 0)
            {
                oneRow = dbInfo.Rows[0];
                nameText = DBGetText(oneRow["LastName"]) + ", " +
                    DBGetText(oneRow["FirstName"]);
                if (DBGetBool(oneRow["Active"]) == false)
                    nameText += " (Inactive)";
            }
            dbInfo.Dispose();
            dbInfo = null;
            return nameText;
        }

        public override string GetEditDescription()
        {
            // ----- Return the list form display title.
            return "Use this form to manage the patrons who have check-out " +
                "privileges at the library.";
        }

        public override string GetEditTitle()
        {
            // ----- Return the list form display title.
            return "Patrons";
        }

        private void PrepareFormFields()
        {
            // ----- Fill in the data.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;

            // ----- Fill in the list of patron groups, if needed.
            if (RecordGroup.Items.Count == 0)
            {
                RecordGroup.Items.Add(new ListItemData("<Not Selected>", -1L));
                sqlText = "SELECT * FROM CodePatronGroup ORDER BY FullName";
                try
                {
                    dbInfo = CreateDataTable(sqlText);
                }
                catch (Exception ex)
                {
                    GeneralError("Patron.PrepareFormFields.Groups", ex);
                    return;
                }
                foreach (DataRow dbRow in dbInfo.Rows)
                    RecordGroup.Items.Add(new ListItemData(
                        DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
                dbInfo.Dispose();
            }

            // ----- Clear the existing data.
            StartingOver = true;
            RecordLastName.Text = "";
            RecordFirstName.Text = "";
            RecordActive.Checked = true;
            RecordBarcode.Text = "";
            RecordPassword.Text = "";
            RecordConfirm.Text = "";
            RecordPhone.Text = "";
            RecordEmail.Text = "";
            RecordAddress.Text = "";
            RecordCity.Text = "";
            RecordState.Text = "";
            RecordPostal.Text = "";
            RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(-1L);
            RecordComments.Text = "";
            RecordMessage.Text = "";
            RecordLastActivity.Text = "Not Available";
            TabPatron.SelectedTab = TabGeneral;
            ActItems.Visible = false;
            if (ActiveID == -1L)
                return;

            // ----- Load in the values stored in the database.
            sqlText = "SELECT * FROM Patron WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Patron.PrepareFormFields.Retrieve", ex);
                return;
            }
            if (dbInfo.Rows.Count > 0)
            {
                oneRow = dbInfo.Rows[0];
                RecordLastName.Text = DBGetText(oneRow["LastName"]);
                RecordFirstName.Text = DBGetText(oneRow["FirstName"]);
                RecordActive.Checked = DBGetBool(oneRow["Active"]);
                RecordBarcode.Text = DBGetText(oneRow["Barcode"]);
                RecordPhone.Text = DBGetText(oneRow["Phone"]);
                RecordEmail.Text = DBGetText(oneRow["Email"]);
                RecordAddress.Text = DBGetText(oneRow["Address"]);
                RecordCity.Text = DBGetText(oneRow["City"]);
                RecordState.Text = DBGetText(oneRow["State"]);
                RecordPostal.Text = DBGetText(oneRow["Postal"]);
                if (Convert.IsDBNull(oneRow["PatronGroup"]) == false)
                {
                    RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(
                        DBGetLong(oneRow["PatronGroup"]));
                    if (RecordGroup.SelectedIndex == -1)
                        RecordGroup.SelectedIndex = RecordGroup.Items.IndexOf(-1L);
                }
                RecordComments.Text = DBGetText(oneRow["Comments"]);
                RecordMessage.Text = DBGetText(oneRow["AdminMessage"]);
                if (Convert.IsDBNull(oneRow["LastActivity"]) == false)
                    RecordLastActivity.Text = DBGetDate(
                        oneRow["LastActivity"]).ToString("MMMM d, yyyy");
                ActItems.Visible = AllowPatronRecord;
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override void SearchForRecord(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            string limitText;
            string nameText;
            int resultCount;
            string limitLast;
            string limitFirst;

            // ----- Prompt the user for the limited item title.
            exceededMatches = false;
            limitText = (new PatronLimit()).PromptUser();

            // ----- Use the limit to prepare the selection criteria.
            limitLast = limitText.GetSubStr(",", 1).Trim();
            limitFirst = limitText.GetSubStr(",", 2).Trim();
            if ((limitLast.Length == 0) & (limitFirst.Length == 0))
                return;
            if (limitLast.Contains("*") == false)
                limitLast += "*";
            if (limitFirst.Contains("*") == false)
                limitFirst += "*";
            limitLast = limitLast.Replace("*", "%");
            limitFirst = limitFirst.Replace("*", "%");

            // ----- Clear any existing values.
            destList.Items.Clear();
            resultCount = 0;

            // ----- Retrieve the values from the database.
            sqlText = "SELECT ID, LastName, FirstName, Active FROM Patron " +
                "WHERE LastName LIKE @TestLast " +
                "AND FirstName LIKE @TestFirst " +
                "ORDER BY LastName, FirstName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestLast", limitLast);
            sqlRun.AddText("TestFirst", limitFirst);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Patron.SearchForRecord", ex);
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                resultCount += 1;
                if (resultCount > SearchMatchLimit)
                {
                    exceededMatches = true;
                    break;
                }
                nameText = DBGetText(dbRow["LastName"]) + ", " +
                    DBGetText(dbRow["FirstName"]);
                if (DBGetBool(dbRow["Active"]) == false)
                    nameText += " (Inactive)";
                destList.Items.Add(new ListItemData(nameText, DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override bool UsesSearch()
        {
            // ----- Patron records are searchable.
            return true;
        }

        private bool SaveFormData()
        {
            // ----- Save the record to the database.
            string sqlText;
            DataCommand sqlRun;
            long newID;

            this.Cursor = Cursors.WaitCursor;

            // ----- Save the data.
            if (ActiveID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO Patron (LastName, FirstName, Active, Comments, " +
                    "AdminMessage, Barcode, [Password], Phone, Email, Address, City, " +
                    "State, Postal, PatronGroup) OUTPUT INSERTED.ID VALUES (@NewLast, " +
                    "@NewFirst, @NewActive, @NewComments, @NewAdmin, @NewBarcode, " +
                    "@NewPassword, @NewPhone, @NewEmail, @NewAddress, @NewCity, " +
                    "@NewState, @NewPostal, @NewGroup)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewLast", RecordLastName.Text.Trim());
                sqlRun.AddText("NewFirst", RecordFirstName.Text.Trim());
                sqlRun.AddBool("NewActive", RecordActive.Checked);
                sqlRun.AddText("NewComments", RecordComments.Text.Trim());
                sqlRun.AddText("NewAdmin", RecordMessage.Text.Trim());
                sqlRun.AddText("NewBarcode", RecordBarcode.Text.Trim());
                sqlRun.AddText("NewPassword", EncryptPassword("patron",
                    RecordPassword.Text.Trim()));
                sqlRun.AddText("NewPhone", RecordPhone.Text.Trim());
                sqlRun.AddText("NewEmail", RecordEmail.Text.Trim());
                sqlRun.AddText("NewAddress", RecordAddress.Text.Trim());
                sqlRun.AddText("NewCity", RecordCity.Text.Trim());
                sqlRun.AddText("NewState", RecordState.Text.Trim());
                sqlRun.AddText("NewPostal", RecordPostal.Text.Trim());
                sqlRun.AddLong("NewGroup", DBComboParam(RecordGroup));
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    GeneralError("Patron.SaveFormData.Add", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
                ActiveID = newID;
            }
            else
            {
                // ----- Update the existing entry.
                sqlText = "UPDATE Patron SET LastName = @NewLast, " +
                    "FirstName = @NewFirst, Active = @NewActive, " +
                    "Comments = @NewComments, AdminMessage = @NewAdmin, " +
                    "Barcode = @NewBarcode, Phone = @NewPhone, " +
                    "Email = @NewEmail, Address = @NewAddress, " +
                    "City = @NewCity, State = @NewState, " +
                    "Postal = @NewPostal, PatronGroup = @NewGroup";
                if (RecordPassword.Text.Trim().Length != 0)
                    sqlText += ", [Password] = @NewPassword";
                sqlText += " WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewLast", RecordLastName.Text.Trim());
                sqlRun.AddText("NewFirst", RecordFirstName.Text.Trim());
                sqlRun.AddBool("NewActive", RecordActive.Checked);
                sqlRun.AddText("NewComments", RecordComments.Text.Trim());
                sqlRun.AddText("NewAdmin", RecordMessage.Text.Trim());
                sqlRun.AddText("NewBarcode", RecordBarcode.Text.Trim());
                sqlRun.AddText("NewPhone", RecordPhone.Text.Trim());
                sqlRun.AddText("NewEmail", RecordEmail.Text.Trim());
                sqlRun.AddText("NewAddress", RecordAddress.Text.Trim());
                sqlRun.AddText("NewCity", RecordCity.Text.Trim());
                sqlRun.AddText("NewState", RecordState.Text.Trim());
                sqlRun.AddText("NewPostal", RecordPostal.Text.Trim());
                sqlRun.AddLong("NewGroup", DBComboParam(RecordGroup));
                if (RecordPassword.Text.Trim().Length != 0)
                    sqlRun.AddText("NewPassword",
                        EncryptPassword("patron", RecordPassword.Text.Trim()));
                sqlRun.AddLong("RecordID", ActiveID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("Patron.SaveFormData.Update", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }

            // ----- Success.
            this.Cursor = Cursors.Default;
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            string sqlText;
            DataCommand sqlRun;

            // ----- Last name is a required field.
            if (RecordLastName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the last name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordLastName.Focus();
                return false;
            }

            // ----- First name is a required field.
            if (RecordFirstName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the first name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordFirstName.Focus();
                return false;
            }

            // ----- Barcode, if supplied, must be numeric.
            if (RecordBarcode.Text.Trim().Length != 0)
            {
                if (DigitsOnly(RecordBarcode.Text) != RecordBarcode.Text.Trim())
                {
                    MessageBox.Show("Barcode, if supplied, must be numeric.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    TabPatron.SelectedTab = TabGeneral;
                    RecordBarcode.Focus();
                    return false;
                }
            }

            // ----- Make sure the barcode is not already in use.
            if (RecordBarcode.Text.Trim().Length != 0)
            {
                sqlText = "SELECT COUNT(*) FROM Patron WHERE UPPER(Barcode) = @TestBarcode";
                if (ActiveID != -1L)
                    sqlText += " AND ID <> @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("TestBarcode", RecordBarcode.Text.Trim().ToUpper());
                if (ActiveID != -1L)
                    sqlRun.AddLong("RecordID", ActiveID);
                try
                {
                    if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                    {
                        // ----- Duplicate entry.
                        MessageBox.Show("A record already exists with the barcode'" +
                            RecordBarcode.Text.Trim() + "'.", ProgramTitle,
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        TabPatron.SelectedTab = TabGeneral;
                        RecordBarcode.Focus();
                        return false;
                    }
                }
                catch (Exception ex)
                {
                    GeneralError("Patron.ValidateFormData", ex);
                    return false;
                }
            }

            // ----- Password and confirm must match.
            if (RecordPassword.Text != RecordConfirm.Text)
            {
                MessageBox.Show("The Password and Confirm fields must match.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                TabPatron.SelectedTab = TabGeneral;
                RecordPassword.Focus();
                return false;
            }

            // ----- For new patrons, a password is required.
            if ((ActiveID == -1L) & (RecordPassword.Text.Trim().Length == 0))
            {
                MessageBox.Show("You must supply a password for new patron records. " +
                    "For an initial password, consider using the last four digits " +
                    "of the patron's phone number.", ProgramTitle,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                TabPatron.SelectedTab = TabGeneral;
                RecordPassword.Focus();
                return false;
            }

            // ----- Check the matching patrons, just in case.
            if (ActiveID == -1L)
            {
                RefreshMatchingPatrons();
                if (ShowMatches.ImageIndex == MatchPresent)
                {
                    if (MessageBox.Show("The name you supplied is similar to one or more names " +
                            "already in the database. Do you still want to save this record?",
                            ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                            DialogResult.Yes)
                        return false;
                }
            }

            // ----- Success.
            return true;
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void Patron_Activated(object sender, EventArgs e)
        {
            // ----- Return the focus to the main field.
            if (StartingOver)
                RecordLastName.Focus();
            StartingOver = false;
        }

        private void RefreshMatchingPatrons()
        {
            // ----- See if there are name matches for this patron.
            string sqlText;
            DataCommand sqlRun;
            string holdText;
            int matches;

            // ----- Only performed for new records.
            if (ActiveID != -1L)
                return;

            // ----- Reset the display.
            ShowMatches.Text = "      No name matches";
            ShowMatches.ImageIndex = MatchNone;

            // ----- Ignore if we have insufficient data.
            if (RecordLastName.Text.Trim().Length == 0)
                return;
            if (RecordFirstName.Text.Trim().Length == 0)
                return;

            // ----- Retrieve a count of matching records.
            sqlText = "SELECT COUNT(*) FROM Patron WHERE LastName LIKE @TestLast";
            if (RecordFirstName.Text.Trim().Length != 0)
                sqlText += " AND FirstName LIKE @TestFirst";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestLast", RecordLastName.Text.Trim());
            if (RecordFirstName.Text.Trim().Length != 0)
                sqlRun.AddText("TestFirst",
                    RecordFirstName.Text.Trim().Substring(0, 1) + "%");
            try
            {
                matches = DBGetInteger(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("Patron.RefreshMatchingPatrons", ex);
                matches = 0;
            }
            if (matches > 0)
            {
                // ----- There are matches.
                if (matches == 1)
                    holdText = "1 match";
                else
                    holdText = matches.ToString() + " matches";
                holdText += ". Click for details.";
                ShowMatches.Text = "      " + holdText;
                ShowMatches.ImageIndex = MatchPresent;
            }
        }

        private void RecordLastName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordLastName.SelectAll();
        }

        private void RecordFirstName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordFirstName.SelectAll();
        }

        private void RecordBarcode_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordBarcode.SelectAll();

            // ----- Don't allow an enter to close the form.
            this.AcceptButton = null;
        }

        private void RecordPassword_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordPassword.SelectAll();
        }

        private void RecordConfirm_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordConfirm.SelectAll();
        }

        private void RecordPhone_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordPhone.SelectAll();
        }

        private void RecordEmail_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordEmail.SelectAll();
        }

        private void RecordAddress_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordAddress.SelectAll();
        }

        private void RecordCity_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordCity.SelectAll();
        }

        private void RecordState_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordState.SelectAll();
        }

        private void RecordPostal_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordPostal.SelectAll();
        }

        private void RecordLastName_Leave(object sender, EventArgs e)
        {
            // ----- Update the matching display.
            RefreshMatchingPatrons();
        }

        private void RecordFirstName_Leave(object sender, EventArgs e)
        {
            // ----- Update the matching display.
            RefreshMatchingPatrons();
        }

        private void ShowMatches_Click(object sender, EventArgs e)
        {
            // ----- The user wants more information on the matches.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            string holdText;
            string nameText;
            string message;
            int matches;
            int counter;

            // ----- Only performed for new records.
            if (ActiveID != -1L)
                return;

            // ----- Ignore if there are no matches.
            if (ShowMatches.ImageIndex == MatchNone)
            {
                MessageBox.Show("No matches.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            // ----- Retrieve a count of matching records.
            if (int.TryParse(ShowMatches.Text, out matches) == false)
                matches = 0;
            this.Cursor = Cursors.WaitCursor;

            // ----- Retrieve the matching records.
            sqlText = "SELECT * FROM Patron WHERE LastName LIKE @TestLast";
            if (RecordFirstName.Text.Trim().Length != 0)
                sqlText += " AND FirstName LIKE @TestFirst";
            sqlText += " ORDER BY LastName, FirstName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestLast", RecordLastName.Text.Trim());
            if (RecordFirstName.Text.Trim().Length != 0)
                sqlRun.AddText("TestFirst",
                    RecordFirstName.Text.Trim().Substring(0, 1) + "%");
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("Author.ShowMatches_Click", ex);
                return;
            }

            counter = 0;
            holdText = "";
            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add this author.
                nameText = DBGetText(dbRow["LastName"]) + ", " +
                    DBGetText(dbRow["FirstName"]);
                if (DBGetBool(dbRow["Active"]) == false)
                    nameText += " (Inactive)";
                holdText += Environment.NewLine + "\t" + nameText;

                // ----- Limit to ten matches.
                counter += 1;
                if (counter >= 10)
                    break;
            }
            dbInfo.Dispose();
            dbInfo = null;
            this.Cursor = Cursors.Default;

            // ----- Format and display the message.
            message = "The database contains " + matches.ToString() + " match";
            if (matches > 1)
                message += "es";
            message += " for the name:" + Environment.NewLine +
                Environment.NewLine + "\t" + RecordLastName.Text.Trim();
            if (RecordFirstName.Text.Trim().Length != 0)
                message += ", " + RecordFirstName.Text.Trim();
            if (matches > 10)
                message += Environment.NewLine + Environment.NewLine +
                    "The first ten matches are:";
            else
                message += Environment.NewLine + Environment.NewLine +
                    "The matches are:";
            message += Environment.NewLine + holdText;
            MessageBox.Show(message, ProgramTitle,
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void ActItems_Click(object sender, EventArgs e)
        {
            // ----- Provide access to patron's circulation records and fees.

            // ***** Insert Code Snippet #13 here.

        }

        private void RecordBarcode_Leave(object sender, EventArgs e)
        {
            // ----- Allow an enter to close the form again.
            this.AcceptButton = ActOK;
        }

        private void RecordBarcode_KeyPress(object sender, KeyPressEventArgs e)
        {
            // ----- Ignore the Enter key.
            if (e.KeyChar == (char)Keys.Return)
                e.Handled = true;
        }
    }
}
