﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class PatronAccess : Form
    {
        private long SelectedPatron;
        private bool InListMode;

        public PatronAccess()
        {
            InitializeComponent();
        }

        public long SelectPatron()
        {
            // ----- Let the user connect as a patron. Return the patron ID,
            //       or -1 if not selected.
            InListMode = false;
            this.ShowDialog();
            if (this.DialogResult == DialogResult.Cancel)
                return -1L;
            else
                return SelectedPatron;
        }

        private void RecordBarcode_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordBarcode.SelectAll();
        }

        private void RecordPassword_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordPassword.SelectAll();
        }

        private void PatronAccess_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- F1 shows online help.
            if (e.KeyCode == Keys.F1)
                OnlineHelp(this, "PatronAccess.htm");
        }

        private void PatronAccess_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.

            // ----- Move the list box to a reasonable position.
            PatronList.Location = new Point(LabelBarcode.Left, RecordBarcode.Top);
            PatronList.Size = new Size(RecordBarcode.Width +
                (RecordBarcode.Left - LabelBarcode.Left), PatronList.Height);

            // ----- Reposition the password field.
            LabelNotRequired.Location = new Point(RecordPassword.Left, LabelPassword.Top);

            // ----- Display the right password field.
            RecordPassword.Visible = !SecurityProfile[(int)LibrarySecurity.PatronOverride];
            LabelNotRequired.Visible = SecurityProfile[(int)LibrarySecurity.PatronOverride];
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to access a patron.
            string barcodeText;
            string lastName;
            string firstName;
            string passwordText;
            string holdText;
            string sqlText;
            DataTable dbInfo;
            DataRow oneRow;
            DataCommand sqlRun;
            long patronID;

            // ----- Special code for list mode.
            if (InListMode == true)
            {
                // ----- A patron selection is required.
                if (PatronList.SelectedIndex == -1)
                {
                    MessageBox.Show("Please select a patron from the list.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    PatronList.Focus();
                    return;
                }

                // ----- Save the selection and close the form.
                SelectedPatron = (long)(ListItemData)PatronList.SelectedItem;
                this.DialogResult = DialogResult.OK;
                return;
            }

            // ----- A patron name or barcode is required.
            if (RecordBarcode.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply a barcode or patron name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordBarcode.Focus();
                return;
            }

            // ----- Get the name or barcode text components.
            if (CheckDecimal(RecordBarcode.Text) == true)
            {
                // ----- Perform a barcode search.
                barcodeText = DigitsOnly(RecordBarcode.Text);
                lastName = "";
                firstName = "";
            }
            else
            {
                // ----- Perform a name search.
                barcodeText = "";
                lastName = RecordBarcode.Text.GetSubStr(",", 1).Trim();
                firstName = RecordBarcode.Text.GetSubStr(",", 2).Trim();

                // ----- A last name is required.
                if (lastName.Length == 0)
                {
                    MessageBox.Show("Please supply a patron name in 'Last, First' format.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordBarcode.Focus();
                    return;
                }

                // ----- No wildcards allowed in the last name.
                if (lastName.Contains("*") == true)
                {
                    MessageBox.Show("The last name may not include wildcard characters.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordBarcode.Focus();
                    return;
                }

                // ----- First name may contain wildcards.
                if (firstName.Contains("*") == false)
                    firstName += "*";
            }

            // ----- Check the password, if required.
            if (SecurityProfile[(int)LibrarySecurity.PatronOverride] == true)
                passwordText = "";
            else
                passwordText = EncryptPassword("patron", RecordPassword.Text);

            // ----- Try to access the record.
            this.Cursor = Cursors.WaitCursor;
            sqlText = "SELECT ID, LastName, FirstName, BarCode, AdminMessage " +
                "FROM Patron WHERE ";
            if (barcodeText.Length != 0)
            {
                // ----- Barcode search.
                sqlText += "Barcode = @TestBarcode";
            }
            else
            {
                // ----- Name search.
                sqlText += "LastName LIKE @TestLast " +
                    "AND FirstName LIKE @TestFirst";
            }
            if (passwordText.Length != 0)
                sqlText += " AND [Password] = @TestPassword";
            sqlRun = new DataCommand(sqlText);
            if (barcodeText.Length != 0)
                sqlRun.AddText("TestBarcode", barcodeText);
            else
            {
                sqlRun.AddText("TestLast", lastName);
                sqlRun.AddText("TestFirst", firstName.Replace("*", "%"));
            }
            if (passwordText.Length != 0)
                sqlRun.AddText("TestPassword", passwordText);

            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("PatronAccess.ActOK_Click.Retrieve", ex);
                return;
            }

            if (dbInfo.Rows.Count == 0)
            {
                this.Cursor = Cursors.Default;
                dbInfo.Dispose();
                dbInfo = null;

                // ----- Perhaps a barcode was being scanned, and the enter key added to
                //       the end caused the OK button to trigger. If so, silently move
                //       to the password field.
                if ((barcodeText.Length != 0) & (RecordPassword.Text.Length == 0) &
                    (this.ActiveControl.Name == RecordBarcode.Name))
                {
                    if (RecordPassword.CanFocus)
                        RecordPassword.Focus();
                }
                else
                {
                    MessageBox.Show("Invalid patron or password.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                return;
            }
            this.Cursor = Cursors.Default;

            // ----- We found at least one matching patron.
            patronID = DBGetLong(dbInfo.Rows[0]["ID"]);
            if (dbInfo.Rows.Count > 1)
            {
                // ----- There is a second record. If in patron mode, then the user
                //       was too vague.
                if (passwordText.Length != 0)
                {
                    dbInfo.Dispose();
                    dbInfo = null;
                    MessageBox.Show("Invalid patron or password.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }

                // ----- Allow the user to choose the patron.
                foreach (DataRow dbRow in dbInfo.Rows)
                {
                    // ----- Add one patron to the list.
                    holdText = DBGetText(dbRow["LastName"]) + ", " +
                        DBGetText(dbRow["FirstName"]);
                    if (Convert.IsDBNull(dbRow["Barcode"]) == false)
                        holdText += " (" + DBGetText(dbRow["Barcode"]) + ")";
                    PatronList.Items.Add(new ListItemData(
                        holdText, DBGetLong(dbRow["ID"])));
                }
                dbInfo.Dispose();
                dbInfo = null;

                // ----- Change into "list" mode.
                InfoText.Text = "Your criteria matched more than one patron. " +
                    "Please select the correct patron from the list.";
                LabelBarcode.Visible = false;
                RecordBarcode.Visible = false;
                LabelPassword.Visible = false;
                RecordPassword.Visible = false;
                LabelNotRequired.Visible = false;
                PatronList.Visible = true;
                PatronList.Focus();
                InListMode = true;
                return;
            }
            else
            {
                // ----- Found a valid match.
                dbInfo.Dispose();
                dbInfo = null;
            }

            // ----- If authorized, show the administrative message for this patron.
            if (SecurityProfile[(int)LibrarySecurity.ViewAdminPatronMessages] == true)
            {
                sqlText = "SELECT FirstName, LastName, AdminMessage " +
                    "FROM Patron WHERE ID = " + patronID.ToString();
                sqlRun = new DataCommand(sqlText);
                try
                {
                    dbInfo = CreateDataTable(sqlRun);
                }
                catch (Exception ex)
                {
                    this.Cursor = Cursors.Default;
                    GeneralError("PatronAccess.ActOK_Click.Message", ex);
                    return;
                }

                oneRow = dbInfo.Rows[0];
                if (Convert.IsDBNull(oneRow["AdminMessage"]) == false)
                    MessageBox.Show("Message concerning " +
                        DBGetText(oneRow["FirstName"]) + " " + DBGetText(oneRow["LastName"]) +
                        ":" + Environment.NewLine + Environment.NewLine +
                        DBGetText(oneRow["AdminMessage"]), ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Information);
                dbInfo.Dispose();
            }

            // ----- Finished.
            SelectedPatron = patronID;
            this.DialogResult = DialogResult.OK;
        }

        private void PatronList_DoubleClick(object sender, EventArgs e)
        {
            // ----- Same as a click on the OK button.
            if (PatronList.SelectedIndex == -1)
                return;
            ActOK.PerformClick();
        }
    }
}
