﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Diagnostics;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class Publisher : Library.BaseCodeForm
    {
        private long ActiveID;
        private bool StartingOver;

        public Publisher()
        {
            InitializeComponent();
        }

        public override long AddRecord()
        {
            // ----- Add a new record.
            ActiveID = -1L;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public long AddRecord(string pubText)
        {
            // ----- Add a new record, but use a starting value previously entered
            //       by the user.
            ActiveID = -1L;
            PrepareFormFields();
            RecordFullName.Text = pubText;
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override bool CanUserAdd()
        {
            // ----- Check the user for security access: add.
            return SecurityProfile[(int)LibrarySecurity.ManagePublishers];
        }

        public override bool CanUserEdit()
        {
            // ----- Check the user for security access: edit.
            return SecurityProfile[(int)LibrarySecurity.ManagePublishers];
        }

        public override bool CanUserDelete()
        {
            // ----- Check the user for security access: delete.
            return SecurityProfile[(int)LibrarySecurity.ManagePublishers];
        }

        public override bool DeleteRecord(long recordID)
        {
            // ----- The user wants to delete the records.
            string sqlText;
            DataCommand sqlRun;

            // ----- Confirm with the user.
            if (MessageBox.Show("Do you really wish to delete the publisher?",
                    ProgramTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) !=
                    DialogResult.Yes)
                return false;

            // ----- Make sure this record is not in use.
            sqlText = "SELECT COUNT(*) FROM NamedItem WHERE Publisher = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    MessageBox.Show("You cannot delete this record because it is in use.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("Publisher.DeleteRecord.Check", ex);
                return false;
            }

            // ----- Delete the record.
            sqlText = "DELETE FROM Publisher WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                ExecuteSQL(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Publisher.DeleteRecord.Delete", ex);
                return false;
            }
            return true;
        }

        public override long EditRecord(long recordID)
        {
            // ----- Edit an existing record.
            ActiveID = recordID;
            PrepareFormFields();
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
                return ActiveID;
            else
                return -1L;
        }

        public override void FillListWithRecords(ListBox destList, ref bool exceededMatches)
        {
            // ----- Given a list, fill it with all defined items. Prompt the
            //       user first.
            destList.Items.Clear();
            SearchForRecord(destList, ref exceededMatches);
        }

        public override string FormatRecordName(long recordID)
        {
            // ----- Given an ID, return the name.
            string sqlText;
            DataCommand sqlRun;

            // ----- Retrieve the value from the database.
            sqlText = "SELECT FullName FROM Publisher WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", recordID);
            try
            {
                return DBGetText(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("Publisher.FormatRecordName", ex);
                return "Error";
            }
        }

        public override string GetEditDescription()
        {
            // ----- Return the list form display title.
            return "This form lets you defined the list of publishers that distribute " +
                "items found in the library.";
        }

        public override string GetEditTitle()
        {
            // ----- Return the list form display title.
            return "Publishers";
        }

        private void PrepareFormFields()
        {
            // ----- Fill in the data.
            string sqlText;
            DataCommand sqlRun;
            DataTable dbInfo;
            DataRow oneRow;
            int inUse;

            // ----- Clear the existing data.
            StartingOver = true;
            RecordFullName.Text = "";
            RecordWeb.Text = "";
            RecordStatus.Text = "New publisher record";
            if (ActiveID == -1L)
                return;

            // ----- Load in the values stored in the database.
            sqlText = "SELECT * FROM Publisher WHERE ID = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Publisher.PrepareFormFields.Retrieve", ex);
                return;
            }
            if (dbInfo.Rows.Count > 0)
            {
                oneRow = dbInfo.Rows[0];
                RecordFullName.Text = DBGetText(oneRow["FullName"]);
                RecordWeb.Text = DBGetText(oneRow["WebSite"]);
            }
            dbInfo.Dispose();

            // ----- Find out how many records link to this publisher.
            sqlText = "SELECT COUNT(*) FROM NamedItem WHERE Publisher = @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                inUse = DBGetInteger(ExecuteSQLReturn(sqlRun));
            }
            catch (Exception ex)
            {
                GeneralError("Publisher.PrepareFormFields.Link", ex);
                inUse = 0;
            }
            RecordStatus.Text = "Linked to " + inUse.ToString() + " library item";
            if (inUse != 1)
                RecordStatus.Text += "s";
        }

        public override void SearchForRecord(ListBox destList, ref bool exceededMatches)
        {
            // ----- Search for publishers using user-specified criteria.
            string sqlText;
            DataTable dbInfo;
            DataCommand sqlRun;
            string limitPattern;
            int resultCount;

            // ----- Prompt the user for the limited publisher name.
            exceededMatches = false;
            limitPattern = (new PublisherLimit()).PromptUser();
            if (limitPattern.Length == 0)
                return;
            if (limitPattern.Contains("*") == false)
                limitPattern += "*";

            // ----- Clear any existing values.
            destList.Items.Clear();
            resultCount = 0;

            // ----- Retrieve the values from the database.
            sqlText = "SELECT ID, FullName FROM Publisher " +
                "WHERE FullName LIKE @TestName ORDER BY FullName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestName", limitPattern.Trim().Replace("*", "%"));
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                GeneralError("Publisher.SearchForRecord", ex);
                return;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                resultCount += 1;
                if (resultCount > SearchMatchLimit)
                {
                    exceededMatches = true;
                    break;
                }
                destList.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
            }
            dbInfo.Dispose();
            dbInfo = null;
        }

        public override bool UsesSearch()
        {
            // ----- Publisher records are searchable.
            return true;
        }

        private bool SaveFormData()
        {
            // ----- Save changes to the record.
            string sqlText;
            DataCommand sqlRun;
            long newID;

            this.Cursor = Cursors.WaitCursor;

            // ----- Save the data.
            if (ActiveID == -1L)
            {
                // ----- Create a new entry.
                sqlText = "INSERT INTO Publisher (FullName, WebSite) " +
                    "OUTPUT INSERTED.ID VALUES (@NewName, @NewSite)";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                sqlRun.AddText("NewSite", RecordWeb.Text.Trim());
                try
                {
                    newID = DBGetLong(ExecuteSQLReturn(sqlRun));
                }
                catch (Exception ex)
                {
                    GeneralError("Publisher.SaveFormData.Add", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
                ActiveID = newID;
            }
            else
            {
                // ----- Update the existing entry.
                sqlText = "UPDATE Publisher SET FullName = @NewName, " +
                    "WebSite = @NewSite WHERE ID = @RecordID";
                sqlRun = new DataCommand(sqlText);
                sqlRun.AddText("NewName", RecordFullName.Text.Trim());
                sqlRun.AddText("NewSite", RecordWeb.Text.Trim());
                sqlRun.AddLong("RecordID", ActiveID);
                try
                {
                    ExecuteSQL(sqlRun);
                }
                catch (Exception ex)
                {
                    GeneralError("Publisher.SaveFormData.Update", ex);
                    return false;
                }
                finally
                {
                    this.Cursor = Cursors.Default;
                }
            }

            // ----- Success.
            return true;
        }

        private bool ValidateFormData()
        {
            // ----- Make sure the user supplied valid data. Return True on success.
            string sqlText;
            DataCommand sqlRun;

            // ----- Name is a required field.
            if (RecordFullName.Text.Trim().Length == 0)
            {
                MessageBox.Show("Please supply the publisher name.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                RecordFullName.Focus();
                return false;
            }

            // ----- Make sure the name is not already in use.
            sqlText = "SELECT COUNT(*) FROM Publisher WHERE UPPER(FullName) = @TestName";
            if (ActiveID != -1L)
                sqlText += " AND ID <> @RecordID";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("TestName", RecordFullName.Text.Trim().ToUpper());
            if (ActiveID != -1L)
                sqlRun.AddLong("RecordID", ActiveID);
            try
            {
                if (DBGetInteger(ExecuteSQLReturn(sqlRun)) > 0)
                {
                    // ----- Duplicate entry.
                    MessageBox.Show("A record already exists with the name '" +
                        RecordFullName.Text.Trim() + "'.", ProgramTitle,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    RecordFullName.Focus();
                    return false;
                }
            }
            catch (Exception ex)
            {
                GeneralError("Publisher.ValidateFormData", ex);
                return false;
            }

            // ----- Success.
            return true;
        }

        private void ActOK_Click(object sender, EventArgs e)
        {
            // ----- The user wants to save changes.
            if (ValidateFormData() == false)
                return;
            if (SaveFormData() == false)
                return;
            this.DialogResult = DialogResult.OK;
        }

        private void Publisher_Activated(object sender, EventArgs e)
        {
            // ----- Return the focus to the main field.
            if (StartingOver)
                RecordFullName.Focus();
            StartingOver = false;
        }

        private void RecordFullName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordFullName.SelectAll();
        }

        private void RecordWeb_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            RecordWeb.SelectAll();
        }

        private void ShowWeb_Click(object sender, EventArgs e)
        {
            // ----- Show the web site displayed in the field.
            ProcessStartInfo newProcess;

            if (RecordWeb.Text.Trim().Length == 0)
                return;
            try
            {
                newProcess = new ProcessStartInfo(RecordWeb.Text.Trim());
                Process.Start(newProcess);
            }
            catch
            {
                // ----- Ignore errors.
            }
        }

        private void RecordWeb_TextChanged(object sender, EventArgs e)
        {
            // ----- Enable the web site button as needed.
            ShowWeb.Enabled = (RecordWeb.Text.Trim().Length != 0);
        }

        private void Publisher_KeyDown(object sender, KeyEventArgs e)
        {
            // ----- F1 shows online help.
            if (e.KeyCode == Keys.F1)
                OnlineHelp(this, "Publisher.htm");
        }
    }
}
