﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Windows.Forms;
using System.Xml.Linq;
using static Library.General;

namespace Library
{
    public partial class ReportSelect : Form
    {
        public ReportSelect()
        {
            InitializeComponent();
        }

        private void LoadReportGroup(XElement groupNode, int indentLevel)
        {
            // ----- Add the groups and items at this level,
            //       and recurse as needed.
            XElement detailNode;
            ReportItem reportEntry;
            int holdValue;

            // ----- Process each item or group.
            foreach (XElement scanNode in groupNode.Nodes())
            {
                // ----- Build a content item for the list.
                reportEntry = new ReportItem();
                reportEntry.Indent = indentLevel;

                // ----- Get the display name.
                detailNode = (XElement)scanNode.Elements("displayText");
                if (detailNode == null)
                    continue;
                reportEntry.DisplayText = detailNode.Value;

                if (scanNode.Name == "reportGroup")
                {
                    // ----- Start a new display group.
                    reportEntry.ItemType = ReportItemEnum.GroupLabel;
                    AllReports.Items.Add(reportEntry);

                    // ----- Recurse to child items.
                    LoadReportGroup(scanNode, indentLevel + 1);
                }
                else if (scanNode.Name == "reportItem")
                {
                    // ----- This is an item. Record its location.
                    detailNode = (XElement)scanNode.Elements("reportPath");
                    if (detailNode != null)
                        reportEntry.ReportPath = detailNode.Value;

                    // ----- Get any command-line arguments.
                    detailNode = (XElement)scanNode.Elements("reportArgs");
                    if (detailNode != null)
                        reportEntry.ReportArgs = detailNode.Value;

                    // ----- Get any item-specific flags.
                    detailNode = (XElement)scanNode.Elements("reportFlags");
                    if (detailNode != null)
                    {
                        // ---- "U" adds "-u loginid" to the command.
                        if ((detailNode.Value.ToUpper().Contains("U") == true) &
                                (LoggedInUserName.Length != 0))
                            reportEntry.ReportArgs = (reportEntry.ReportArgs +
                                " -u " + LoggedInUserName).Trim();
                    }

                    // ----- Store the full description.
                    detailNode = (XElement)scanNode.Elements("description");
                    if (detailNode != null)
                        reportEntry.Description = detailNode.Value;

                    // ----- So, what type of entry is it?
                    if (scanNode.Attributes("type").ToString() == "built-in")
                    {
                        // ----- Built-in program. Check for valid ID.
                        if (int.TryParse(reportEntry.ReportPath, out holdValue) == false)
                            holdValue = 0;
                        if ((holdValue < 1) | (holdValue > (int)ReportItemEnum.BuiltInStatistics))
                            continue;
                        reportEntry.ItemType = (ReportItemEnum)Convert.ToInt32(reportEntry.ReportPath);
                        AllReports.Items.Add(reportEntry);
                    }
                    else if (scanNode.Attributes("type").ToString() == "program")
                    {
                        // ----- EXE program-based report.
                        if (reportEntry.ReportPath.Length == 0)
                            continue;
                        reportEntry.ItemType = ReportItemEnum.ExeProgram;
                        AllReports.Items.Add(reportEntry);
                    }
                    else if (scanNode.Attributes("type").ToString() == "url")
                    {
                        // ----- URL-based report.
                        if (reportEntry.ReportPath.Length == 0)
                            continue;
                        reportEntry.ItemType = ReportItemEnum.UrlProgram;
                        AllReports.Items.Add(reportEntry);
                    }
                }
            }
        }

        private void RefreshReportList()
        {
            // ----- Load in the list of available reports.
            string configFile;
            XDocument configData;
            ReportItem reportEntry;

            // ----- Clear the existing list.
            AllReports.Items.Clear();

            // ----- Get the location of the configuration file.
            configFile = Properties.Settings.Default.ReportConfig + "";

            // ----- Load the configuration file.
            if (configFile.Length != 0)
            {
                if (System.IO.File.Exists(configFile))
                {
                    try
                    {
                        // ----- Load in the file.
                        configData = XDocument.Load(configFile);

                        // ----- Process the configuration file.
                        LoadReportGroup((XElement)configData.FirstNode, 0);
                    }
                    catch (Exception ex)
                    {
                        GeneralError("ReportSelect.RefreshReportList", ex);
                    }
                }
            }

            // ----- If the configuration file resulted in no reports
            //       appearing in the list, add the default reports.
            if (AllReports.Items.Count == 0)
            {
                for (int counter = 1; counter <= Convert.ToInt32(
                    ReportItemEnum.BuiltInStatistics); counter++)
                {
                    // ----- Build the report entry.
                    reportEntry = new ReportItem();
                    reportEntry.Indent = 0;
                    reportEntry.ItemType = (ReportItemEnum)counter;
                    switch (reportEntry.ItemType)
                    {
                        case ReportItemEnum.BuiltInCheckedOut:
                            reportEntry.DisplayText = "Items Checked Out";
                            break;
                        case ReportItemEnum.BuiltInOverdue:
                            reportEntry.DisplayText = "Items Overdue";
                            break;
                        case ReportItemEnum.BuiltInMissing:
                            reportEntry.DisplayText = "Items Missing";
                            break;
                        case ReportItemEnum.BuiltInFinesOwed:
                            reportEntry.DisplayText = "Patron Fines Owed";
                            break;
                        case ReportItemEnum.BuiltInStatistics:
                            reportEntry.DisplayText = "Database Statistics";
                            break;
                    }

                    // ----- Add the report entry to the list.
                    AllReports.Items.Add(reportEntry);
                }
            }
        }

        private void AllReports_SelectedIndexChanged(object sender, EventArgs e)
        {
            // ----- Display a description of the report, if available.
            ReportItem reportEntry;

            // ----- Clear any previous description.
            FullDescription.Text = "No report selected.";
            if (AllReports.SelectedIndex != -1)
            {
                // ----- Locate the content and display it.
                reportEntry = (ReportItem)AllReports.SelectedItem;
                FullDescription.Text = reportEntry.Description;
            }
        }

        private void ReportSelect_Load(object sender, EventArgs e)
        {
            // ----- Display the list of reports.
            RefreshReportList();
        }

        private void ActRun_Click(object sender, EventArgs e)
        {
            // ----- Run the selected report.
            ReportItem reportEntry;

            // ----- Make sure a report is selected.
            if (AllReports.SelectedIndex == -1)
            {
                MessageBox.Show("Please select a report from the list.",
                    ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // ----- Different code for each type of entry.
            reportEntry = (ReportItem)AllReports.SelectedItem;
            this.Cursor = Cursors.WaitCursor;
            switch (reportEntry.ItemType)
            {
                case ReportItemEnum.GroupLabel:
                    // ----- No report for group entries.
                    MessageBox.Show("Please select a report from the list.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    break;
                case ReportItemEnum.BuiltInCheckedOut:
                    // ----- Items Checked Out
                    // TODO: Write BasicReportCheckedOut();
                    break;
                case ReportItemEnum.BuiltInOverdue:
                    // ----- Items Overdue
                    // TODO: Write BasicReportOverdue();
                    break;
                case ReportItemEnum.BuiltInMissing:
                    // ----- Items Missing
                    // TODO: Write BasicReportMissing();
                    break;
                case ReportItemEnum.BuiltInFinesOwed:
                    // ----- Fines Owed by Patrons
                    // TODO: Write BasicReportFines();
                    break;
                case ReportItemEnum.BuiltInStatistics:
                    // ----- Library Database Statistics
                    // TODO: Write BasicReportStatistics();
                    break;
                case ReportItemEnum.ExeProgram:
                    // ----- Start a program.
                    try
                    {
                        Process.Start("\"" + reportEntry.ReportPath +
                            "\" " + reportEntry.ReportArgs);
                    }
                    catch (Exception ex)
                    {
                        GeneralError("ReportSelect.ActRun_Click.Exe", ex);
                    }
                    break;
                case ReportItemEnum.UrlProgram:
                    // ----- Start a URL.
                    Process.Start(reportEntry.ReportPath);
                    break;
            }
            this.Cursor = Cursors.Default;
        }
    }
}
