﻿// ----- The Library Project
//       Start-to-Finish Visual C# 2015
//       Copyright (c) 2016 by Tim Patrick

using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using static Library.General;

namespace Library
{
    public partial class SeriesAddLocate : Form
    {
        private int ActivePanel;

        private const int PanelCriteria = 1;
        private const int PanelResults = 2;

        public SeriesAddLocate()
        {
            InitializeComponent();
        }

        private int FindUserRecords(string searchText)
        {
            // ----- Search for series codes using user-specified criteria.
            string sqlText;
            DataTable dbInfo;
            DataCommand sqlRun;
            int matches;

            // ----- Clear any existing values.
            SearchResults.Items.Clear();
            this.Cursor = Cursors.WaitCursor;

            // ----- Retrieve the values from the database.
            matches = 0;
            sqlText = "SELECT ID, FullName FROM CodeSeries " +
                "WHERE FullName LIKE @SearchText ORDER BY FullName";
            sqlRun = new DataCommand(sqlText);
            sqlRun.AddText("SearchText", searchText);
            try
            {
                dbInfo = CreateDataTable(sqlRun);
            }
            catch (Exception ex)
            {
                this.Cursor = Cursors.Default;
                GeneralError("SeriesAddLocate.FindUserRecords", ex);
                return -1;
            }

            foreach (DataRow dbRow in dbInfo.Rows)
            {
                // ----- Add one item.
                SearchResults.Items.Add(new ListItemData(
                    DBGetText(dbRow["FullName"]), DBGetLong(dbRow["ID"])));
                matches += 1;
                if (matches >= SearchMatchLimit)
                    break;
            }
            dbInfo.Dispose();
            dbInfo = null;
            this.Cursor = Cursors.Default;
            return matches;
        }

        public long PromptUser()
        {
            // ----- Prompt the user for a series. Return the matching ID. Return
            //       -1 if no change made, or -2 to clear the active setting.
            ActivePanel = PanelCriteria;
            this.ShowDialog();
            if (this.DialogResult == DialogResult.OK)
            {
                if ((long)(ListItemData)SearchResults.SelectedItem == -2L)
                {
                    // ----- Clear the current series.
                    return -2L;
                }
                else if ((long)(ListItemData)SearchResults.SelectedItem == -1L)
                {
                    // ----- Prompt for a new series using the original typed name.
                    return (new CodeSeries()).AddRecord(SeriesName.Text.Trim());
                }
                else
                {
                    // ----- The user selected a match.
                    return (long)(ListItemData)SearchResults.SelectedItem;
                }
            }
            else
            {
                // ----- Cancelled by user.
                return -1L;
            }
        }

        private void SeriesName_Enter(object sender, EventArgs e)
        {
            // ----- Highlight the entire text.
            SeriesName.SelectAll();
        }

        private void ActBack_Click(object sender, EventArgs e)
        {
            // ----- Go back to the first panel.
            ActivePanel = PanelCriteria;
            SecondPanel.Visible = false;
            FirstPanel.Visible = true;
            ActBack.Enabled = false;
            ActNext.Text = "&Next >";
            SeriesName.Focus();
        }

        private void ActNext_Click(object sender, EventArgs e)
        {
            // ----- Take action on the user input.
            string searchText;
            int matches;

            // ----- Different code for each of the panels.
            if (ActivePanel == PanelCriteria)
            {
                // ----- Determine the search string.
                searchText = SeriesName.Text.Trim();
                if (searchText.Length == 0)
                {
                    MessageBox.Show("Please supply a series name.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    SeriesName.Focus();
                    return;
                }
                if (searchText.Contains("*") == false)
                    searchText += "*";
                searchText = searchText.Replace("*", "%");

                // ----- Search for matches.
                matches = FindUserRecords(searchText);
                if (matches == -1)
                    return;

                // ----- If there are no matches, then either error out, or
                //       add a new record, depending on the user's security.
                //       If there are matches, show them.
                if (matches == 0)
                {
                    if (SecurityProfile[(int)LibrarySecurity.ManageSeries])
                    {
                        // ----- Prompt to add a new record.
                        SearchResults.ClearSelected();
                        SearchResults.Items.Add(new ListItemData(
                            "<No matches. Add new series.>", -1L));
                        SearchResults.SelectedIndex = 0;

                        // ----- Return to the PromptUser() routine.
                        this.DialogResult = DialogResult.OK;
                    }
                    else
                    {
                        // ----- Give an error.
                        MessageBox.Show("No matches.", ProgramTitle,
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        return;
                    }
                }
                else
                {
                    // ----- If the user can add new series, add that option.
                    if (SecurityProfile[(int)LibrarySecurity.ManageSeries])
                        SearchResults.Items.Insert(0, new ListItemData(
                            "<None of these matches. Add new series.>", -1L));
                    ActivePanel = PanelResults;
                    SecondPanel.Visible = true;
                    FirstPanel.Visible = false;
                    ActBack.Enabled = true;
                    ActNext.Text = "Finish";
                    SearchResults.Focus();
                }
            }
            else
            {
                // ----- Check for a selected result.
                if (SearchResults.SelectedIndex == -1)
                {
                    MessageBox.Show("Please select a match from the list.",
                        ProgramTitle, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    SearchResults.Focus();
                    return;
                }

                // ----- Return to the PromptUser() routine.
                this.DialogResult = DialogResult.OK;
            }
        }

        private void SearchResults_DoubleClick(object sender, EventArgs e)
        {
            // ----- Same as the finish button.
            if (SearchResults.SelectedIndex == -1)
                return;
            ActNext.PerformClick();
        }

        private void ActClear_Click(object sender, EventArgs e)
        {
            // ----- Clear the current series.
            SearchResults.ClearSelected();
            SearchResults.Items.Add(new ListItemData("Clear item", -2L));
            SearchResults.SelectedIndex = 0;

            // ----- Return to the PromptUser() routine.
            this.DialogResult = DialogResult.OK;
        }

        private void SeriesAddLocate_Load(object sender, EventArgs e)
        {
            // ----- Prepare the form.
            SecondPanel.Location = new Point(FirstPanel.Location.X, FirstPanel.Location.Y);
        }
    }
}
